SystemOrganization addCategory: #'Swazoo-Compatibility'!
SystemOrganization addCategory: #'Swazoo-Core'!
SystemOrganization addCategory: #'Swazoo-Exceptions'!
SystemOrganization addCategory: #'Swazoo-HTTP'!
SystemOrganization addCategory: #'Swazoo-Headers'!
SystemOrganization addCategory: #'Swazoo-Messages'!
SystemOrganization addCategory: #'Swazoo-Resources'!
SystemOrganization addCategory: #'Swazoo-Tests'!

TestCase subclass: #CompositeResourceTest
	instanceVariableNames: 'composite'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!CompositeResourceTest methodsFor: 'testing' stamp: ' 10/7/07 17:21'!
sampleInSite
	| site |
	site := Site new.
	site host: 'swazoo.org' ip: '127.0.0.1' port: 8200.
	site addResource: composite! !

!CompositeResourceTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
setUp
	composite := CompositeResource uriPattern: '/'! !

!CompositeResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testAddResource
	| child |
	composite addResource: (child := HelloWorldResource uriPattern: 'hello.html').
	self assert: composite children size = 1.
	self assert: composite children first == child.
	self assert: child parent == composite! !

!CompositeResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testAddResources
	| child1 child2 |
	child1 := HelloWorldResource uriPattern: 'hello1.html'.
	child2 := HelloWorldResource uriPattern: 'hello2.html'.
	composite addResources: (Array with: child1 with: child2).
	self assert: composite children size = 2.
	composite children
		do: 
			[:each | 
			self assert: (composite children includes: each).
			self assert: each parent == composite]! !

!CompositeResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testCurrentUrl
	| child leaf |	
	self sampleInSite.
	self assert: composite currentUrl = 'http://swazoo.org:8200/'.
	composite addResource: (child := CompositeResource uriPattern: 'foo').
	self assert: child currentUrl = 'http://swazoo.org:8200/foo/'.
	child addResource: (leaf := HelloWorldResource uriPattern: 'hi.html').
	self assert: leaf currentUrl = 'http://swazoo.org:8200/foo/hi.html'.! !

!CompositeResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testEmptyURIPatternInvalid
	composite uriPattern: ''.
	self deny: composite isValidlyConfigured! !

!CompositeResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testNilURIPatternDoesNothing
	| pattern |
	pattern := composite uriPattern.
	composite uriPattern: nil.
	self assert: composite uriPattern = pattern! !

!CompositeResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testValidlyConfigured
	self assert: composite isValidlyConfigured! !

TestCase subclass: #FileResourceTest
	instanceVariableNames: 'resource'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!FileResourceTest methodsFor: 'running' stamp: 'JM 4/18/2007 15:42'!
setUp
	| directory firstFile ws |
	directory := 'fResTest' asFilename.
	directory exists ifFalse: [directory makeDirectory].
	firstFile := 'fResTest' asFilename construct: 'abc.html'.
	ws := firstFile writeStream.
	[ws nextPutAll: 'hello'] ensure: [ws close].
	resource := FileResource uriPattern: 'foo' filePath: 'fResTest'! !

!FileResourceTest methodsFor: 'running' stamp: 'JM 4/18/2007 15:43'!
tearDown
	('fResTest' asFilename construct: 'abc.html') delete.
	'fResTest' asFilename delete! !

!FileResourceTest methodsFor: 'testing' stamp: 'JM 4/18/2007 16:34'!
testContentType
	self assert: (resource contentTypeFor: '.txt') = 'text/plain'.
	self assert: (resource contentTypeFor: '.html') = 'text/html'
! !

!FileResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testDirectoryIndex
	| request response |
	request := HTTPGet request: 'foo/'.
	resource directoryIndex: 'abc.html'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response code = 200.
	self assert: request resourcePath size = 1.
	self assert: request resourcePath first = 'foo'! !

!FileResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testETag
	"Filename etags do not have the leading and trailing double quotes.  Header fields add the quotes as necessary"

	| request response etag |
	request := HTTPGet request: 'foo/abc.html'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response code = 200.
	self 
		assert: (etag := (response headers fieldOfClass: HTTPETagField) entityTag) 
				notNil.
	request := HTTPGet request: 'foo/abc.html'.
	request headers addField: (HTTPIfNoneMatchField new addEntityTag: etag).
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response code = 304.
	self assert: (response headers fieldOfClass: HTTPETagField) entityTag = etag.
	request := HTTPGet request: 'foo/abc.html'.
	request headers addField: (HTTPIfNoneMatchField new valueFrom: '"wrong"').
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response code = 200.
	self assert: (response headers fieldOfClass: HTTPETagField) entityTag = etag! !

!FileResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testExistantFile
	| request response |
	request := HTTPGet request: 'foo/abc.html'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response code = 200.
	self assert: request resourcePath size = 1.
	self assert: request resourcePath first = 'foo'! !

!FileResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testNonexistantFile
	| request response |
	request := HTTPGet request: 'foo/notThere.html'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response isNil! !

!FileResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testRedirection
	| request response |
	request := HTTPGet request: 'foo'.
	resource directoryIndex: 'abc.html'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response code = 301.
	self assert: (response headers fieldNamed: 'Location') uri asString 
				= 'http://foo/'.
	self assert: (response headers fieldNamed: 'Location') uri host = 'foo'! !

!FileResourceTest methodsFor: 'testing' stamp: 'JM 4/18/2007 16:33'!
testRelativeFile
"it doesn't work anyway!!
	| request response |
	request := HTTPGet request: 'foo/../', resource fileDirectory tail, '/abc.html'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response isNil
"! !

!FileResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testSafeConstruct
	| request response |
	request := HTTPGet request: 'foo/../abc.html'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response code = 200.
	request := HTTPGet request: 'foo/.. /./abc.html'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response code = 200! !

TestCase subclass: #HTTPRequestTest
	instanceVariableNames: 'request'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!HTTPRequestTest methodsFor: 'setup' stamp: ' 10/7/07 18:48'!
basicGet
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.1';
		nextPutLine: 'Host: foo.com';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'setup' stamp: ' 10/7/07 18:48'!
basicGetHTTP10
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.0';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'setup' stamp: ' 10/7/07 18:48'!
basicGetHTTP10Keepalive
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.0';
		nextPutLine: 'Connection: Keep-Alive';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'setup' stamp: ' 10/7/07 18:48'!
basicHead
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'HEAD / HTTP/1.1';
		nextPutLine: 'Host: foo.com';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
crlfOn: aStream 
	aStream
		nextPut: Character cr;
		nextPut: Character lf! !

!HTTPRequestTest methodsFor: 'setup' stamp: 'mivsek 7/26/2007 12:26'!
fileContents
	| stream |
	stream := SwazooStream on: String new.
	stream
		nextPutLine: 'BEGIN:VCALENDAR';
		nextPutLine: 'PRODID:-//Squeak-iCalendar//-';
		nextPutLine: 'VERSION:2.0';
		nextPutLine: 'X-WR-CALNAME:test';
		nextPutLine: 'METHOD:PUBLISH';
		nextPutLine: 'BEGIN:VEVENT';
		nextPutLine: 'UID:an event with a start date and not date and time';
		nextPutLine: 'CATEGORIES:category1,category2';
		nextPutLine: 'CREATED:20050501T110231Z';
		nextPutLine: 'SEQUENCE:0';
		nextPutLine: 'SUMMARY:aTitle';
		nextPutLine: 'PRIORITY:5';
		nextPutLine: 'DTSTART;VALUE=DATE:20050425';
		nextPutLine: 'END:VEVENT';
		nextPutLine: 'END:VCALENDAR'.

	^stream writeBuffer contents asString! !

!HTTPRequestTest methodsFor: 'setup' stamp: ' 10/7/07 18:48'!
fullGet
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET /aaa/bbb/ccc.html?foo=bar&baz=quux HTTP/1.1';
		nextPutLine: 'Connection: Keep-Alive';
		nextPutLine: 'User-Agent: Mozilla/4.72 [en] (X11; I; Linux 2.3.51 i686)';
		nextPutLine: 'Host: foo.com:8888';
		nextPutLine: 'Referer: http://www.bar.com/takeMeThere.html';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'setup' stamp: 'mivsek 7/23/2007 22:55'!
fullPostFile
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST  /document/aab.html HTTP/1.1';
		nextPutLine: 'Connection: Keep-Alive';
		nextPutLine: 'User-Agent: Mozilla/4.72 [en] (X11; I; Linux 2.3.51 i686)';
		nextPutLine: 'Host: biart.eranova.si';
		nextPutLine: 'Referer: http://www.bar.com/takeMeThere.html';
		nextPutLine: 'Content-Type: multipart/form-data; boundary= -----------------20752836116568320241700153999';
		nextPutLine: 'Content-Length: 845';
            crlf;
		nextPutLine: '-------------------20752836116568320241700153999';
		nextPutLine: 'Content-Disposition: form-data; name="id5273"';
		crlf; 
		nextPutLine: 'main';
		nextPutLine: '-------------------20752836116568320241700153999';
		nextPutLine: 'Content-Disposition: form-data; name="field2"';
		crlf; crlf;
		nextPutLine: '-------------------20752836116568320241700153999';
		nextPutLine: 'Content-Disposition: form-data; name="field7"; filename="event.ical"';
		nextPutLine: 'Content-Type: application/octet-stream';
		crlf;

		nextPutAll: self fileContents;

		crlf;
		nextPutLine: '-------------------20752836116568320241700153999';
		nextPutLine: 'Content-Disposition: form-data; name="attach"';
		crlf;
		nextPutLine: 'Attach';
		nextPutLine: '-------------------20752836116568320241700153999--'.

	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'setup' stamp: ' 10/7/07 18:48'!
portedGet
	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.1';
		nextPutLine: 'Host: foo.com:8888';
		crlf.
	^HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)! !

!HTTPRequestTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
test10ConnectionClose
	request := self basicGetHTTP10.
	self assert: request wantsConnectionClose! !

!HTTPRequestTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
test10KeepAliveConnectionClose
	request := self basicGetHTTP10Keepalive.
	self deny: request wantsConnectionClose! !

!HTTPRequestTest methodsFor: 'testing-gets' stamp: ' 10/7/07 17:21'!
testBasicGet
	request := self basicGet.
	self assert: request isGet.
	self deny: request isHead.
	self deny: request isPost.
	self deny: request isPut! !

!HTTPRequestTest methodsFor: 'testing-gets' stamp: ' 10/7/07 17:21'!
testBasicGetHost
	request := self basicGet.
	self assert: request host = 'foo.com'! !

!HTTPRequestTest methodsFor: 'testing-gets' stamp: ' 10/7/07 17:21'!
testBasicGetPort
	request := self basicGet.
	self assert: request port = 80! !

!HTTPRequestTest methodsFor: 'testing-gets' stamp: ' 10/7/07 17:21'!
testBasicHead
	request := self basicHead.
	self assert: request isHead.
	self deny: request isGet.
	self deny: request isPost.
	self deny: request isPut! !

!HTTPRequestTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testConnection
	request := self fullGet.
	self assert: request connection = 'Keep-Alive'! !

!HTTPRequestTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testHeaderAtIfPresent
	request := self basicGet.
	self assert: (request headers 
				fieldOfClass: HTTPIfRangeField
				ifPresent: [:header | header == (request headers fieldOfClass: HTTPIfRangeField)]
				ifAbsent: [true]).
	self assert: (request headers 
				fieldOfClass: HTTPHostField
				ifPresent: [:header | header == (request headers fieldOfClass: HTTPHostField)]
				ifAbsent: [false])! !

!HTTPRequestTest methodsFor: 'testing-bad requests' stamp: 'mivsek 7/21/2007 22:52'!
testMissingContentType
	| requestStream result |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST /foobar HTTP/1.0';
		nextPutLine: 'Host: foo.com';
"	      nextPutLine: 'Content-Type: text/plain'. <-- this is missing!! - and should be for this test"
		nextPutLine: 'Content-Length: 12';
		crlf;
		nextPutLine: 'Hello, World'.
	result := SpExceptionContext 
		for: [(HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)) 
				ensureFullRead] "because of defered post data parsing"
		on: SpError
		do: [:ex | ex].
	self assert: result class == SwazooHTTPPostError.
	^self! !

!HTTPRequestTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testNo11ConnectionClose
	request := self basicGet.
	self deny: request wantsConnectionClose! !

!HTTPRequestTest methodsFor: 'testing' stamp: ' 10/7/07 18:48'!
testNoEqualsQueries
	"The last assert here used to check that 'request queryAt: 'WSDL'' is nil, but a test for an empty string is more consistent with query argument formats."

	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET /test/typed.asmx?WSDL HTTP/1.1';
		nextPutLine: 'Host: foo.com:8888';
		crlf.
	request := HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents).
	self assert: (request includesQuery: 'WSDL').
	self assert: (request queryAt: 'WSDL') isEmpty! !

!HTTPRequestTest methodsFor: 'testing-gets' stamp: ' 10/7/07 17:21'!
testPortedGetPort
	request := self portedGet.
	self assert: request port = 8888! !

!HTTPRequestTest methodsFor: 'testing-posts' stamp: 'mivsek 7/26/2007 13:37'!
testPostFile
	| post |
	post := self fullPostFile.
	self assert: post isPostDataEmpty not.
	self assert: (post postDataStringAt: 'id5273') = 'main'.
	self assert: (post postDataStringAt: 'field2') = ''.
	self assert: (post postDataAt: 'field7') filename = 'event.ical'.
	self assert: 
		((post postDataStringAt: 'field7') readStream upTo: Character cr) = 'BEGIN:VCALENDAR'.
	self assert: (post postDataStringAt: 'field7') = self fileContents.
	self assert: (post postDataStringAt: 'attach') = 'Attach'.! !

!HTTPRequestTest methodsFor: 'testing-posts' stamp: 'mivsek 7/26/2007 13:37'!
testPostFileStreamed
	| post stream |
	post := self fullPostFile.
	stream := WriteStream on: ByteArray new.
	post postDataAt: 'field7' streamTo: stream.
	self assert: (post isPostDataStreamedAt: 'field7').
	self deny: post postData isParsed. "post data read from socket defered"
	self assert: (post postDataStringAt: 'id5273') = 'main'. 
	self assert: post postData isParsed.  "first access to post data trigger full read and parse"
	self assert: (post postDataAt: 'field7') filename = 'event.ical'.
	self assert: 
		(stream contents asString readStream upTo: Character cr) = 'BEGIN:VCALENDAR'.
	self assert: stream contents asString = self fileContents.
	self assert: (post postDataStringAt: 'attach') = 'Attach'.! !

!HTTPRequestTest methodsFor: 'testing-posts' stamp: 'mivsek 7/19/2007 22:18'!
testPostRawEntity
	| requestStream post |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST /foobar HTTP/1.0';
		nextPutLine: 'Host: foo.com';
		nextPutLine: 'Content-Type: text/plain';
		nextPutLine: 'Content-Length: 12';
		crlf;
		nextPutLine: 'Hello, World'.
	post := HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents).
	self assert: post isPostDataEmpty.
	self assert: post entityBody = 'Hello, World'! !

!HTTPRequestTest methodsFor: 'testing-posts' stamp: 'mivsek 7/10/2007 18:55'!
testPostReadToBoundary
	"for MIME multipart parser in HTTPPost"
	| boundary stream |
	boundary := '--------23453443421'.
	"just boundary"
	stream := SwazooStream new readBuffer: (boundary) asByteArray readStream.
	self assert: nil = (HTTPPost new nextFrom: stream boundary: boundary asByteArray).
	"text before boundary, just first chr returned"
	stream := SwazooStream new readBuffer: ('text', boundary) asByteArray readStream.
	self assert: 't' = (HTTPPost new nextFrom: stream boundary: boundary asByteArray) asString.
	"part of boundary"
	stream := SwazooStream new readBuffer: ('---', boundary) asByteArray readStream.
	self assert: '---' = (HTTPPost new nextFrom: stream boundary: boundary asByteArray) asString.
	"text after boundary"
	stream := SwazooStream new readBuffer: (boundary, 'sfsdsg') asByteArray readStream.
	self assert: nil = (HTTPPost new nextFrom: stream boundary: boundary asByteArray).
	"mixed with one boundary chr"
	stream := SwazooStream new readBuffer: ('-1-') asByteArray readStream.
	self assert: '-' = (HTTPPost new nextFrom: stream boundary: boundary asByteArray) asString.! !

!HTTPRequestTest methodsFor: 'testing-posts' stamp: ' 10/7/07 18:48'!
testPostUrlEncodedData
	| requestStream post |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'POST / HTTP/1.1';
		nextPutLine: 'Host: foo.com';
		nextPutLine: 'Content-Type: application/x-www-form-urlencoded';
		nextPutLine: 'Content-Length: 31';
		crlf;
		nextPutLine: 'address=+fs&product=&quantity=1'.
	post := HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents).
	self assert: (post postDataAt: 'address') value = ' fs'.
	self assert: (post postDataAt: 'product') value = ''.
	self assert: (post postDataAt: 'quantity') value = '1'! !

!HTTPRequestTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testReferer
	request := self fullGet.
	self 
		assert: request referer asString = 'http://www.bar.com/takeMeThere.html'! !

!HTTPRequestTest methodsFor: 'testing-bad requests' stamp: 'mivsek 7/21/2007 22:46'!
testRequestWithCRButNoLF

"	| requestStream result |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutAll: 'GET / HTTP/1.1';
		cr.
	result := SpExceptionContext 
				for: [HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents)]
				on: SpError
				do: [:ex | ex].
	self assert: result class == SwazooHTTPParseError.
	^self"! !

!HTTPRequestTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testUserAgent
	request := self fullGet.
	self 
		assert: request userAgent = 'Mozilla/4.72 [en] (X11; I; Linux 2.3.51 i686)'! !

TestCase subclass: #HTTPResponseTest
	instanceVariableNames: 'response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!HTTPResponseTest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
crlf
	^String with: Character cr with: Character lf! !

!HTTPResponseTest methodsFor: 'testing' stamp: 'mivsek 7/19/2007 19:28'!
testInternalServerError
	| ws rs |
	response := HTTPResponse internalServerError.
	ws := SwazooStream on: String new.
	response printStatusOn: ws.
	rs := SwazooStream on: ws writeBuffer contents.
	self assert: rs nextLine = 'HTTP/1.1 500 Internal Server Error'! !

!HTTPResponseTest methodsFor: 'testing' stamp: 'mivsek 7/19/2007 19:28'!
testOK
	| ws rs |
	response := HTTPResponse ok.
	ws := SwazooStream on: String new.
	response printStatusOn: ws.
	rs := SwazooStream on: ws writeBuffer contents.
	self assert: rs nextLine = 'HTTP/1.1 200 OK'! !

!HTTPResponseTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testResponseTypes
	self assert: (HTTPResponse badRequest) isBadRequest.
	self assert: (HTTPResponse found) isFound.
	self assert: (HTTPResponse internalServerError) isInternalServerError.
	self assert: (HTTPResponse movedPermanently) isMovedPermanently.
	self assert: (HTTPResponse notFound) isNotFound.
	self assert: (HTTPResponse notImplemented) isNotImplemented.
	self assert: (HTTPResponse notModified) isNotModified.
	self assert: (HTTPResponse ok) isOk.
	self assert: (HTTPResponse redirectLink) isRedirectLink.
	self assert: (HTTPResponse seeOther) isSeeOther.! !

TestCase subclass: #HTTPServerTest
	instanceVariableNames: 'server stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!HTTPServerTest methodsFor: 'running' stamp: 'JM 4/16/2007 22:52'!
setUp
	| socket |
	(Delay forMilliseconds: 100) wait.
	server := HTTPServer new.
	[server ip: 'localhost'; port: 8123.
	server start]
		fork.
	(Delay forMilliseconds: 100) wait.
" 	stream := (SocketAccessor newTCPclientToHost: 'localhost' port: 8123) 
				readAppendStream"
	socket := SpSocket connectToServerOnHost:  'localhost' port: 8123.
	stream := SwazooStream socket: socket ! !

!HTTPServerTest methodsFor: 'running' stamp: 'JM 4/16/2007 22:57'!
tearDown
	server stop.
	stream close.
	stream := nil.
	Delay forMilliseconds: 500.
! !

!HTTPServerTest methodsFor: 'tests' stamp: ' 14/4/07 23:10'!
testServing
	self assert: server isServing! !

!HTTPServerTest methodsFor: 'tests' stamp: ' 14/4/07 23:10'!
testStopServing
	server stop.
	self deny: server isServing! !

TestCase subclass: #HeaderFieldTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!HeaderFieldTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testCombine
	"Entity tags must be quoted strings - RFC 2616 3.11"

	| header1 header2 header3 |
	header1 := HeaderField fromLine: 'If-Match: "a"'.
	header2 := HeaderField fromLine: 'If-Match: "b","c"'.
	header3 := HeaderField fromLine: 'If-Match: "d"'.
	header1 combineWith: header2.
	self assert: header1 valuesAsString = '"a","b","c"'.
	header1 combineWith: header3.
	self assert: header1 valuesAsString = '"a","b","c","d"'! !

!HeaderFieldTest methodsFor: 'testing' stamp: 'mivsek 7/19/2007 19:28'!
testContentTypeMultiple
	"   HTTP/1.1 header field values can be folded onto multiple lines if the
   continuation line begins with a space or horizontal tab. All linear
   white space, including folding, has the same semantics as SP. A
   recipient MAY replace any linear white space with a single SP before
   interpreting the field value or forwarding the message downstream.

       LWS            = [CRLF] 1*( SP | HT )"

	| requestStream request field |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET / HTTP/1.1';
		nextPutLine: 'Host: 127.0.0.1';
		nextPutLine: 'Content-Type: text/html; ';
		nextPutLine: ' charset=iso-8859-1';
		crlf.
	request := HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents).
	field := request headers fieldNamed: 'content-type'.
	self assert: field name = 'Content-Type'.
	self assert: field mediaType = 'text/html'.
	self assert: (field transferCodings at: 'charset') = 'iso-8859-1'! !

!HeaderFieldTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testValues
	"Entity tags are held internally as simple strings.  Any necessary leading and trailing double quotes are added by the header fields as needed.  Note that it is OK to have a comma in an entity tag - see the second of the group of 3 tags below."

	| header |
	header := HeaderField fromLine: 'If-Match: "xyzzy" '.
	self assert: header name = 'If-Match'.
	self assert: header entityTags first = 'xyzzy'.
	header := HeaderField 
				fromLine: 'If-Match: "xyzzy", "r2d2,xxxx", "c3piozzzz" '.
	self assert: header name = 'If-Match'.
	self assert: header entityTags first = 'xyzzy'.
	self assert: (header entityTags at: 2) = 'r2d2,xxxx'.
	self assert: header entityTags last = 'c3piozzzz'! !

TestCase subclass: #HelloWorldResourceTest
	instanceVariableNames: 'hello'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!HelloWorldResourceTest methodsFor: 'running' stamp: 'JM 4/18/2007 18:00'!
setUp
	hello := HelloWorldResource uriPattern: 'hello.html'! !

!HelloWorldResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testResponse
	| request response |
	request := HTTPGet request: 'hello.html'.
	response := URIResolution resolveRequest: request startingAt: hello.
	self assert: response code = 200.
	self assert: request resourcePath size = 1.
	self assert: request resourcePath first = 'hello.html'! !

TestCase subclass: #HomeResourceTest
	instanceVariableNames: 'resource'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!HomeResourceTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
setUp
	resource := HomeResource uriPattern: '/' filePath: 'home'! !

!HomeResourceTest methodsFor: 'running' stamp: 'JM 4/18/2007 22:43'!
testRootFileFor
	| request |
	request := HTTPGet request: '/~someUser'.
	URIResolution new initializeRequest: request.
	self assert: (resource rootFileFor: request) asString 
				= (('home' asFilename construct: 'someUser') construct: 'html') asString! !

!HomeResourceTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
testValidateHomePath
	self assert: (resource validateHomePath: '~somebody').
	self assert: (resource validateHomePath: '~somebodyElse').
	self deny: (resource validateHomePath: 'someplace').
	self deny: (resource validateHomePath: 'some~body')! !

TestCase subclass: #RedirectionResourceTest
	instanceVariableNames: 'resource'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!RedirectionResourceTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
setUp
	resource := RedirectionResource uriPattern: 'foo'
				targetUri: 'http://abc.def.com'! !

!RedirectionResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testGetResource
	| request response |
	request := HTTPGet request: 'foo'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response code = 301.
	self assert: (response headers fieldNamed: 'Location') uri asString 
				= 'http://abc.def.com'.
	self assert: request resourcePath size = 1.
	self assert: request resourcePath first = 'foo'! !

TestCase subclass: #ResourceTest
	instanceVariableNames: 'resource'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!ResourceTest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
basicGet: uri
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: 'GET ', uri, ' HTTP/1.1'.
	self crlfOn: ws.
	ws nextPutAll: 'Host: swazoo.org'.
	self crlfOn: ws.
	self crlfOn: ws.
	^HTTPRequest readFrom: (ReadStream on: ws contents)! !

!ResourceTest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
basicGetUri: uriString
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: 'GET ' , uriString , ' HTTP/1.1'.
	self crlfOn: ws.
	ws nextPutAll: 'Host: swazoo.org'.
	self crlfOn: ws.
	self crlfOn: ws.
	^HTTPRequest readFrom: (ReadStream on: ws contents)! !

!ResourceTest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
basicGetUri: uriString host: hostname port: port 
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: 'GET ' , uriString , ' HTTP/1.1'.
	self crlfOn: ws.
	ws nextPutAll: 'Host: ' , hostname.
	port notNil 
		ifTrue: 
			[ws
				nextPut: $:;
				print: port].
	self crlfOn: ws.
	self crlfOn: ws.
	^HTTPRequest readFrom: (ReadStream on: ws contents)! !

!ResourceTest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
crlfOn: aStream 
	aStream
		nextPut: Character cr;
		nextPut: Character lf! !

!ResourceTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
setUp
	resource := Resource uriPattern: 'foo'! !

!ResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testEmptyURIPatternInvalid
	resource uriPattern: ''.
	self deny: resource isValidlyConfigured! !

!ResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testEnabledByDefault
	self assert: resource isEnabled! !

!ResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testNilURIPatternDoesNothing
	| pattern |
	pattern := resource uriPattern.
	resource uriPattern: nil.
	self assert: resource uriPattern = pattern! !

!ResourceTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testValidlyConfigured
	self assert: resource isValidlyConfigured! !

TestCase subclass: #SiteIdentifierTest
	instanceVariableNames: 'identifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!SiteIdentifierTest methodsFor: 'running' stamp: ' 10/7/07 17:21'!
setUp
	identifier := SiteIdentifier 
				host: 'localhost'
				ip: '127.0.0.1'
				port: 80! !

!SiteIdentifierTest methodsFor: 'testing' stamp: ' 10/7/07 17:21'!
testCaseInsensitiveMatch
	| another |
	another := SiteIdentifier 
				host: 'lOCaLhOST'
				ip: '127.0.0.1'
				port: 80.
	self assert: (identifier match: another)! !

!SiteIdentifierTest methodsFor: 'testing' stamp: ' 10/7/07 17:21'!
testCurrentUrl
	self assert: identifier currentUrl = 'http://localhost'.
		identifier := SiteIdentifier 
				host: 'localhost'
				ip: '127.0.0.1'
				port: 81.
	self assert: identifier currentUrl = 'http://localhost:81'! !

!SiteIdentifierTest methodsFor: 'testing' stamp: ' 10/7/07 17:21'!
testHostMismatch
	| another |
	another := SiteIdentifier 
				host: 'thisIsMyMachine'
				ip: '127.0.0.1'
				port: 80.
	self deny: (identifier match: another)! !

!SiteIdentifierTest methodsFor: 'testing' stamp: ' 10/7/07 17:21'!
testIPMismatch
	| another |
	another := SiteIdentifier 
				host: 'localhost'
				ip: '127.0.0.2'
				port: 80.
	self deny: (identifier match: another)! !

!SiteIdentifierTest methodsFor: 'testing' stamp: ' 10/7/07 17:21'!
testMatch
	| another |
	another := SiteIdentifier 
				host: 'localhost'
				ip: '127.0.0.1'
				port: 80.
	self assert: (identifier match: another)! !

!SiteIdentifierTest methodsFor: 'testing' stamp: ' 10/7/07 17:21'!
testPortMismatch
	| another |
	another := SiteIdentifier 
				host: 'localhost'
				ip: '127.0.0.1'
				port: 81.
	self deny: (identifier match: another)! !

TestCase subclass: #SiteTest
	instanceVariableNames: 'site'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!SiteTest methodsFor: 'running' stamp: ' 10/7/07 17:21'!
addSecondAlias
	site addAlias: (SiteIdentifier 
				host: 'swazoo2.org'
				ip: '127.0.0.2'
				port: 8202).! !

!SiteTest methodsFor: 'running' stamp: ' 10/7/07 17:21'!
setUp
	super setUp.
	site := Site new.
	site addAlias: (SiteIdentifier 
				host: 'swazoo.org'
				ip: '127.0.0.1'
				port: 8200)! !

!SiteTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testCurrentUrl
	site currentUrl = 'http://swazoo.org:8200'.
	self addSecondAlias.
	site currentUrl = 'http://swazoo.org:8200'.! !

!SiteTest methodsFor: 'testing' stamp: ' 10/7/07 17:21'!
testCurrentUrl80
	| aSite |
	aSite := Site new.
	aSite addAlias: (SiteIdentifier
			host: 'swazoo.org'
			ip: '127.0.0.1'
			port: 80).
	aSite currentUrl = 'http://swazoo.org'.
	aSite currentUrl = 'http://swazoo.org'! !

TestCase subclass: #SwazooBaseExtensionsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!SwazooBaseExtensionsTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testCharacterArrayTrimBlanks
	self 
		assert: (HTTPString trimBlanksFrom: '   a b c d e f g') = 'a b c d e f g'.
	self assert: (HTTPString trimBlanksFrom: 'no blanks') = 'no blanks'.
	self assert: (HTTPString trimBlanksFrom: ' leading') = 'leading'.
	self assert: (HTTPString trimBlanksFrom: 'trailing ') = 'trailing'.
	self assert: (HTTPString trimBlanksFrom: '') = ''.
	self 
		assert: (HTTPString 
				trimBlanksFrom: (String with: Character cr with: Character lf)) isEmpty! !

!SwazooBaseExtensionsTest methodsFor: 'testing' stamp: 'mivsek 7/21/2007 22:39'!
testFilenameEtag
	"The filename etag is a simple string and does not contain double quotes.  Header fields apply double quotes as necessary when writing themselves."

	| fn etag1 etag2 |
	fn := 'etagTest' asFilename.
	[fn writeStream nextPut: $-; close. "create file"
	etag1 := fn etag.
	(Delay forSeconds: 1) wait.
	fn appendStream nextPut: $-; close. "modify file"
	etag2 := fn etag.

	self assert: (etag1 isKindOf: String).
	self assert: (etag2 isKindOf: String).
	self deny: etag1 = etag2] 
		ensure: [fn delete]! !

!SwazooBaseExtensionsTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testStringNewRandom
	| sizes strings |
	sizes := #(5 20 6127 2 100).
	strings := sizes collect: [:each |  HTTPString newRandomString: each].
	strings with: sizes do: [:string :size | self assert: string size = size]! !

TestCase subclass: #SwazooCacheControlTest
	instanceVariableNames: 'resource cacheTarget request cacheControl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!SwazooCacheControlTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
setUp
	| directory firstFile ws |
	directory := 'fResTest' asFilename.
	directory exists ifFalse: [directory makeDirectory].
	firstFile := directory construct: 'abc.html'.
	ws := firstFile writeStream.
	[ws nextPutAll: 'hello'] ensure: [ws close].
	resource := FileResource uriPattern: 'foo' filePath: 'fResTest'.
	request := HTTPGet request: 'foo/abc.html'.
	URIResolution resolveRequest: request startingAt: resource.
	cacheControl := SwazooCacheControl new request: request
				cacheTarget: (cacheTarget := resource fileFor: request)! !

!SwazooCacheControlTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
tearDown
	('fResTest' asFilename construct: 'abc.html') delete.
	'fResTest' asFilename delete! !

!SwazooCacheControlTest methodsFor: 'testing' stamp: 'JM 4/18/2007 16:39'!
testIfModifiedSinceModified
	| response timestampInThePast |
	request := HTTPGet request: 'foo/abc.html'.
	timestampInThePast := SpTimestamp fromDate: (Date today subtractDays: 1)
				andTime: Time now.
	request headers addField: (HTTPIfModifiedSinceField new 
				valueFrom: timestampInThePast asRFC1123String).
	cacheControl := SwazooCacheControl new request: request
				cacheTarget: cacheTarget.
	self assert: cacheControl isNotModified not.
	self assert: cacheControl isIfModifiedSince.
	response := HTTPResponse ok.
	cacheControl addResponseHeaders: response.
	self 
		assert: (response headers fieldNamed: 'ETag') entityTag = cacheTarget etag.
	self assert: (response headers fieldNamed: 'Last-Modified') timestamp 
				= cacheTarget lastModified! !

!SwazooCacheControlTest methodsFor: 'testing' stamp: 'JM 4/18/2007 16:40'!
testIfModifiedSinceNot
	| response |
	request headers addField: (HTTPIfModifiedSinceField new 
				valueFrom: cacheTarget lastModified asRFC1123String).
	self assert: cacheControl isNotModified.
	self assert: cacheControl isIfModifiedSince not.
	response := HTTPResponse notModified.
	cacheControl addResponseHeaders: response.
	self 
		assert: (response headers fieldNamed: 'ETag') entityTag = cacheTarget etag.
	self assert: (response headers fieldNamed: 'Last-Modified') timestamp 
				= cacheTarget lastModified! !

!SwazooCacheControlTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testIfNoneMatchHeaderMatch
	"same etag"

	| response |
	request headers addField: (HTTPIfNoneMatchField new addEntityTag: cacheTarget etag).
	self assert: cacheControl isNotModified.
	self deny: cacheControl isIfNoneMatch.

	"do NOT include last-modified"
	response := HTTPResponse notModified.
	cacheControl addResponseHeaders: response.
	self assert: (response headers fieldNamed: 'ETag') entityTag = cacheTarget etag.
	self assert: (response headers fieldNamed: 'Last-Modified' ifNone: [nil])  isNil! !

!SwazooCacheControlTest methodsFor: 'testing' stamp: 'JM 4/18/2007 16:40'!
testIfNoneMatchHeaderNone
	"same etag"

	| response |
	request := HTTPGet request: 'foo/abc.html'.
	request headers addField: (HTTPIfNoneMatchField new valueFrom: 'blah').
	cacheControl := SwazooCacheControl new request: request
				cacheTarget: cacheTarget.
	self assert: cacheControl isNotModified not.
	self assert: cacheControl isIfNoneMatch.
	response := HTTPResponse ok.
	cacheControl addResponseHeaders: response.
	self 
		assert: (response headers fieldNamed: 'ETag') entityTag = cacheTarget etag.
	self assert: (response headers fieldNamed: 'Last-Modified') timestamp 
				= cacheTarget lastModified! !

!SwazooCacheControlTest methodsFor: 'testing' stamp: 'JM 4/18/2007 16:42'!
testNoHeaders
	| response |
	self assert: cacheControl isNotModified not.
	self assert: cacheControl isIfNoneMatch.
	self assert: cacheControl isIfModifiedSince.

	"add both"
	response := HTTPResponse ok.
	cacheControl addResponseHeaders: response.
	self 
		assert: (response headers fieldNamed: 'ETag') entityTag = cacheTarget etag.
	self assert: (response headers fieldNamed: 'Last-Modified') timestamp 
				= cacheTarget lastModified! !

TestCase subclass: #SwazooCompilerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!SwazooCompilerTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
testEvaluate
	self assert: (SwazooCompiler evaluate: '1 + 2 * 3') = 9! !

!SwazooCompilerTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
testEvaluateReceiver
	self assert: (SwazooCompiler evaluate: 'self + 2 * 3' receiver: 1) = 9! !

TestCase subclass: #SwazooConfigurationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!SwazooConfigurationTest methodsFor: 'testing' stamp: 'JM 4/18/2007 16:47'!
testCompositeResourceSite
	| rs site composite howdy duh hithere |
	rs := ReadStream 
				on: '<Site>
 <CompositeResource uriPattern: ''/''>
  <HelloWorldResource uriPattern: ''howdy''>
  <CompositeResource uriPattern: ''duh''>
   <HelloWorldResource uriPattern: ''hithere''>
  </CompositeResource>
 </CompositeResource>
</Site>'.
	site := Site new readFrom: rs.
	self assert: site children size = 1.
	composite := site children first.
	self assert: composite class == CompositeResource.
	self assert: composite uriPattern = '/'.
	self assert: composite children size = 2.
	self assert: composite parent == site.
	howdy := composite children first.
	self assert: (howdy class == HelloWorldResource).
	self assert: howdy uriPattern = 'howdy'.
	self assert: howdy parent == composite.
	duh := composite children last.
	self assert: duh children size = 1.
	self assert: duh class == CompositeResource.
	self assert: duh uriPattern = 'duh'.
	self assert: duh parent == composite.
	hithere := duh children first.
	self assert: hithere class == HelloWorldResource.
	self assert: hithere uriPattern = 'hithere'.
	self assert: hithere parent == duh.! !

!SwazooConfigurationTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testEmptySite
	| rs site alias |
	rs := ReadStream 
				on: '<Site>
 <SiteIdentifier ip: ''192.168.1.66'' port: 80 host: ''swazoo.org''>
</Site>'.
	site := Site new readFrom: rs.
	self assert: site aliases size = 1.
	self assert: site currentUrl = 'http://swazoo.org/'.
	alias := site aliases first.
	self assert: alias host = 'swazoo.org'.
	self assert: alias ip = '192.168.1.66'.
	self assert: alias port = 80! !

!SwazooConfigurationTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testFileResourceSite
	| rs site resource |
	rs := ReadStream 
				on: '<Site>
<SiteIdentifier ip: ''192.168.1.66'' port: 80 host: ''swazoo.org''>
 <FileResource uriPattern: ''/'' filePath: ''files''>
</Site>'.
	site := Site new readFrom: rs.
	self assert: site children size = 1.
	resource := site children first.
	self assert: resource class == FileResource.
	self assert: resource uriPattern = '/'.
	self assert: resource filePath = 'files'.
	self assert: resource parent == site.
	self assert: resource currentUrl = 'http://swazoo.org/'.! !

!SwazooConfigurationTest methodsFor: 'testing' stamp: 'JM 4/18/2007 16:47'!
testMultipleResourcesSite
	| rs site resource1 resource2 |
	rs := ReadStream 
				on: '<Site>
 <HelloWorldResource uriPattern: ''/''>
 <HelloWorldResource uriPattern: ''/''>
</Site>'.
	site := Site new readFrom: rs.
	self assert: site children size = 2.
	resource1 := site children first.
	self assert: (resource1 class == HelloWorldResource).
	self assert: resource1 uriPattern = '/'.
	resource2 := site children last.
	self assert: resource2 class == HelloWorldResource.
	self assert: resource2 uriPattern = '/'! !

!SwazooConfigurationTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testMultipleSites
	| rs sites site alias1 alias2 |
	rs := ReadStream 
				on: '<Site>
 <SiteIdentifier ip: ''192.168.1.66'' port: 80 host: ''swazoo.org''>
 <SiteIdentifier ip: ''192.168.1.66'' port: 81 host: ''swazoo.org''>
</Site>
<Site>
</Site>'.
	sites := SwazooServer readSitesFrom: rs.
	self assert: sites size = 2.
	site := sites first.
	self assert: site aliases size = 2.
	alias1 := site aliases first.
	self assert: alias1 host = 'swazoo.org'.
	self assert: alias1 ip = '192.168.1.66'.
	self assert: alias1 port = 80.
	alias2 := site aliases last.
	self assert: alias2 host = 'swazoo.org'.
	self assert: alias2 ip = '192.168.1.66'.
	self assert: alias2 port = 81! !

!SwazooConfigurationTest methodsFor: 'testing' stamp: 'JM 4/18/2007 16:48'!
testSingleResourceSite
	| rs site resource |
	rs := ReadStream on: '<Site>
<SiteIdentifier ip: ''192.168.1.66'' port: 80 host: ''swazoo.org''>
 <HelloWorldResource uriPattern: ''/''>
</Site>'.
	site := Site new readFrom: rs.
	self assert: site children size = 1.
	resource := site children first.
	self assert: (resource class == HelloWorldResource).
	self assert: resource uriPattern = '/'.
	self assert: resource parent == site.
	self assert: resource currentUrl = 'http://swazoo.org/'.! !

!SwazooConfigurationTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testSiteTag
	| rs config tag |
	rs := ReadStream on: '  <Site>  

</Site>   '.
	config := Site new.
	tag := config nextTagFrom: rs.
	self assert: tag = 'Site'.
	tag := config nextTagFrom: rs.
	self assert: tag = '/Site'.
	self assert: (config nextTagFrom: rs) isNil! !

TestCase subclass: #SwazooServerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!SwazooServerTest methodsFor: 'testing' stamp: 'mivsek 7/21/2007 22:41'!
testAccessingSite
	| site |
	site := Site new name: 'aabbcc';
		host: 'test.org' ip: 'localhost' port: 8543.
	[SwazooServer singleton addSite: site.
	self assert: (SwazooServer siteNamed: 'aabbcc') notNil.
	site := SwazooServer siteNamed: 'aabbcc'.
	self assert: (site name = 'aabbcc').
	self assert: (SwazooServer siteHostnamed: 'test.org') notNil.
	site := SwazooServer siteHostnamed: 'test.org'.
	self assert: (site host = 'test.org')]
		ensure: [SwazooServer singleton removeSite: site].! !

!SwazooServerTest methodsFor: 'testing' stamp: 'mivsek 7/21/2007 22:42'!
testAddingSite
	| site server nrSites |
	server := SwazooServer singleton.
	nrSites := server sites size.
	self assert: (server siteNamed: 'aabbcc') isNil.
	self assert: (server siteHostnamed: 'test.org') isNil.
	site := Site new name: 'aabbcc';
		host: 'test.org' ip: 'localhost' port: 80.
	server addSite: site.
	self assert: (server siteNamed: 'aabbcc') notNil.
	self assert: (server siteHostnamed: 'test.org') notNil.
	server removeSite: site.
	self assert: server sites size = nrSites.! !

!SwazooServerTest methodsFor: 'testing' stamp: 'mivsek 7/21/2007 22:42'!
testDuplicateNames
	| site server |
	server := SwazooServer singleton.
	site := Site new name: 'aabbcc';
		host: 'test.org' ip: 'localhost' port: 80.
	[server addSite: site.
	self should: [site name: 'aabbcc'] raise: Error.
	self shouldnt: [site host: 'test.org'] raise: Error.
	self should: [Site new name: 'aabbcc';
		host: 'test.org' ip: 'localhost' port: 80] raise: Error]
			ensure: [server removeSite: site].! !

!SwazooServerTest methodsFor: 'testing' stamp: 'mivsek 7/21/2007 22:42'!
testStartingSite
	| site server nrServers |
	server := SwazooServer singleton.
	nrServers := server servers size.
	site := Site new name: 'aabbcc'; host: 'test.org' ip: 'localhost' port: 8765.
	[server addSite: site.
	self assert: site isServing not.
	SwazooServer startSite: 'aabbcc'.
	self assert: server servers size = (nrServers+1).
	self assert: site isServing.
	SwazooServer stopSite: 'aabbcc'.
	self assert: site isServing not.
	self assert: server servers size = nrServers]
		ensure: [site stop. server removeSite: site].! !

TestCase subclass: #SwazooSocketTest
	instanceVariableNames: 'input output'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!SwazooSocketTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
setUp
	| pair |
	pair := SwazooSocket connectedPair.
	input := pair first.
	output := pair last! !

!SwazooSocketTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
tearDown
	input close.
	output close! !

!SwazooSocketTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testConnectedPair
	(Array with: input with: output) 
		do: [:each | self assert: (each isKindOf: SwazooSocket)]! !

!SwazooSocketTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testNetworkConnection
	| server sem |
	input close.
	output close.
	sem := Semaphore new.
	
	[server := SwazooSocket serverOnIP: '127.0.0.1' port: 65423.
	server listenFor: 50.
	
	[input := server accept.
	sem signal] fork.
	output := SwazooSocket connectTo: 'localhost' port: 65423.
	sem wait.
	self testReadWrite] 
			ensure: [server close]! !

!SwazooSocketTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testPartialRead
	| bytes |
	bytes := ByteArray withAll: #(5 4 3).
	self assert: (input write: bytes) = 3.
	self assert: (output read: 5) = bytes! !

!SwazooSocketTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testReadTimeout
	input write: (ByteArray withAll: #(1 2 3)).
	self assert: (output read: 3 timeout: 40) = (ByteArray withAll: #(1 2 3)).
	self assert: (output read: 3 timeout: 40) = ByteArray new! !

!SwazooSocketTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testReadWrite
	| bytes |
	bytes := ByteArray withAll: #(1 2 3 4 5).
	self assert: (input write: bytes) = 5.
	self assert: (output read: 5) = bytes.
	bytes := ByteArray with: 4.
	self assert: (input write: bytes) = 1.
	self assert: (output read: 1) = bytes! !

TestCase subclass: #SwazooStreamTest
	instanceVariableNames: 'input output'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!SwazooStreamTest methodsFor: 'running' stamp: 'mivsek 7/19/2007 19:26'!
crlfOn: aSwazooStream
	aSwazooStream
		nextPut: Character cr;
		nextPut: Character lf! !

!SwazooStreamTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
setUp
	| pair |
	pair := SwazooStream connectedPair.
	input := pair first.
	output := pair last! !

!SwazooStreamTest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
tearDown
	input close.
	output close! !

!SwazooStreamTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testConnectedPair
	(Array with: input with: output) 
		do: [:each | self assert: (each isKindOf: SwazooStream)]! !

!SwazooStreamTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testErrorOnInputClose
	self should: 
			[input close.
			output next]
		raise: Error! !

!SwazooStreamTest methodsFor: 'testing-lines' stamp: 'mivsek 7/19/2007 19:27'!
testLinesWithDoubleCRLF
	| ws rs comparisonString |
	comparisonString := 'abcd'.
	ws := SwazooStream on: String new.
	ws nextPutAll: comparisonString.
	self crlfOn: ws.
	self crlfOn: ws.
	rs := SwazooStream on: ws writeBuffer contents.
	self assert: rs nextLine = comparisonString.
	self assert: rs nextLine = ''! !

!SwazooStreamTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testNextPut
	#($A $M $Y $b $r $z) do: 
			[:each | 
			self assert: (input nextPut: each) = each. 
			input flush.
			self assert: output next = each]! !

!SwazooStreamTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testNextPutAll
	#('123' 'abc' 'swazoo') do: 
			[:each | 
			self assert: (input nextPutAll: each) = each. 
			input flush.
			self assert: (output next: each size) = each]! !

!SwazooStreamTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testNextPutByte
	| bytes |
	bytes := ByteArray 
				with: 6
				with: 5
				with: 0
				with: 2.
	bytes do: 
			[:each | 
			self assert: (input nextPutByte: each) = each. 
			input flush.
			self assert: output nextByte = each]! !

!SwazooStreamTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testNextPutBytes
	| bytes1 bytes2 bytes3 |
	bytes1 := ByteArray withAll: #(1 2 3 4).
	bytes2 := ByteArray withAll: #(5 4 3 2 1).
	bytes3 := ByteArray withAll: #(1 1 2 3 5).
	(Array 
		with: bytes1
		with: bytes2
		with: bytes3) do: 
				[:each | 
				self assert: (input nextPutBytes: each) = each.
				input flush.
				self assert: (output nextBytes: each size) = each]! !

!SwazooStreamTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testPeek
	#($K $J $D $j $m $z) do: 
			[:each | 
			input nextPut: each.
			input flush.
			self assert: output peek = each.
			output next]! !

!SwazooStreamTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testPeekByte
	| bytes |
	bytes := ByteArray withAll: #(5 2 8 4 11 231).
	bytes do: 
			[:each | 
			input nextPutByte: each.
			input flush.
			self assert: output peekByte = each.
			output nextByte]! !

!SwazooStreamTest methodsFor: 'testing-lines' stamp: 'mivsek 7/19/2007 19:27'!
testSingleLineWithCR
	| ws rs comparisonString errored |
	comparisonString := 'abcd' , (String with: Character cr) , 'efg'.
	ws := SwazooStream on: String new.
	ws nextPutAll: comparisonString.
	ws nextPut: Character cr.
	rs := SwazooStream on: ws writeBuffer contents.
	errored := false.
	SpExceptionContext 
		for: [rs nextLine]
		on: SpError
		do: [:ex | errored := true].
	self assert: errored! !

!SwazooStreamTest methodsFor: 'testing-lines' stamp: 'mivsek 7/19/2007 19:27'!
testSingleLineWithCRLF
	| ws rs comparisonString |
	comparisonString := 'abcd'.
	ws := SwazooStream on: String new.
	ws nextPutAll: comparisonString.
	self crlfOn: ws.
	rs := SwazooStream on: ws writeBuffer contents.
	self assert: rs nextLine = comparisonString! !

TestCase subclass: #SwazooURITest
	instanceVariableNames: 'fooURI barURI queryURI'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!SwazooURITest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
setUp
	fooURI := SwazooURI fromString: 'www.foo.com/index.html'.
	queryURI := SwazooURI fromString: 'www.foo.com/index.html?foo=1&bar=hi'.
	barURI := SwazooURI fromString: 'www.bar.com:8080/files/'! !

!SwazooURITest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
testHostname
	self assert: fooURI hostname = 'www.foo.com'.
	self assert: queryURI hostname = 'www.foo.com'.
	self assert: barURI hostname = 'www.bar.com'! !

!SwazooURITest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
testIdentifier
	self assert: fooURI identifier = '/index.html'.
	self assert: queryURI identifier = '/index.html'.
	self assert: barURI identifier = '/files/'! !

!SwazooURITest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
testIdentifierPath
	self assert: fooURI identifierPath 
				= (OrderedCollection with: '/' with: 'index.html').
	self assert: queryURI identifierPath 
				= (OrderedCollection with: '/' with: 'index.html').
	self 
		assert: barURI identifierPath = (OrderedCollection with: '/' with: 'files')! !

!SwazooURITest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
testIsDirectory
	self deny: fooURI isDirectory.
	self deny: queryURI isDirectory.
	self assert: barURI isDirectory! !

!SwazooURITest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
testPort
	self assert: fooURI port = 80.
	self assert: queryURI port = 80.
	self assert: barURI port = 8080! !

!SwazooURITest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
testQueries
	self deny: (queryURI includesQuery: 'hi').
	self assert: (queryURI includesQuery: 'foo').
	self assert: (queryURI includesQuery: 'bar').
	self assert: (queryURI queryAt: 'foo') = '1'.
	self assert: (queryURI queryAt: 'bar') = 'hi'! !

!SwazooURITest methodsFor: 'running' stamp: ' 14/4/07 23:10'!
testValue
	self assert: fooURI value = 'http://www.foo.com/index.html'.
	self assert: queryURI value = 'http://www.foo.com/index.html?foo=1&bar=hi'.
	self assert: barURI value = 'http://www.bar.com:8080/files/'! !

TestCase subclass: #URIParsingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!URIParsingTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
test05SimpleFullURI

|uri|
uri := SwazooURI fromString: 'http://abc.com:8080/smith/home.html'.
self assert: (uri protocol = 'http').
self assert: (uri hostname = 'abc.com'). 
self assert: (uri port = 8080).
self assert: (uri identifier = '/smith/home.html').
self assert: (uri asString = 'http://abc.com:8080/smith/home.html').! !

!URIParsingTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
test10SimpleFullURIWithQuery

|uri|
uri := SwazooURI fromString: 'http://abc.com:8080/smith/home.html?a=1&b=2'.
self assert: (uri protocol = 'http').
self assert: (uri hostname = 'abc.com'). 
self assert: (uri port = 8080).
self assert: (uri identifier = '/smith/home.html').
self assert: (uri asString = 'http://abc.com:8080/smith/home.html?a=1&b=2').! !

!URIParsingTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
test15SimpleFullURIWithPort80

|uri|
uri := SwazooURI fromString: 'http://abc.com:80/smith/home.html?a=1&b=2'.
self assert: (uri protocol = 'http').
self assert: (uri hostname = 'abc.com'). 
self assert: (uri port = 80).
self assert: (uri identifier = '/smith/home.html').
self assert: (uri asString = 'http://abc.com/smith/home.html?a=1&b=2').! !

!URIParsingTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
test20SimpleFullURIWithNoPort

|uri|
uri := SwazooURI fromString: 'http://abc.com/smith/home.html?a=1&b=2'.
self assert: (uri protocol = 'http').
self assert: (uri hostname = 'abc.com'). 
self assert: (uri port = 80).
self assert: (uri identifier = '/smith/home.html').
self assert: (uri asString = 'http://abc.com/smith/home.html?a=1&b=2').! !

TestCase subclass: #URIResolutionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!URIResolutionTest methodsFor: 'testing' stamp: 'JM 4/18/2007 22:29'!
testCompositeAnswer
	| resource request response | 
	resource := CompositeResource uriPattern: 'base'.
	resource addResource: (HelloWorldResource uriPattern: 'hi').
	request := HTTPGet request: 'base/hi'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response code = 200.
	self assert: request resourcePath size = 2.
	self assert: request resourcePath first = 'base'.
	self assert: request resourcePath last = 'hi'! !

!URIResolutionTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testCompositeItselfCannotAnswer
	| resource request response |
	resource := CompositeResource uriPattern: 'base'.
	request := HTTPGet request: 'base'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response isNil! !

!URIResolutionTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testCompositeNoAnswer
	| resource request response |
	resource := CompositeResource uriPattern: 'base'.
	resource addResource: (HelloWorldResource uriPattern: 'hi').
	request := HTTPGet request: 'tail/hi'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response isNil! !

!URIResolutionTest methodsFor: 'testing' stamp: 'JM 4/18/2007 22:30'!
testLeafAnswer
	| resource request response | 
	resource := HelloWorldResource uriPattern: 'hi'.
	request := HTTPGet request: 'hi'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response code = 200.
	self assert: request resourcePath size = 1.
	self assert: request resourcePath first = 'hi'! !

!URIResolutionTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testNoAnswerWhenDisabled
	| resource request response |
	resource := HelloWorldResource uriPattern: 'hi'.
	resource disable.
	request := HTTPGet request: 'hi'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response isNil! !

!URIResolutionTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testResourcePath
	| request resolution |
	request := HTTPGet 
				request: 'foo/bar/baz/quux'
				from: 'localhost:1234'
				at: '1.2.3.4'.
	resolution := URIResolution new initializeRequest: request.
	self assert: resolution resourcePath = #('foo') asOrderedCollection.
	resolution advance.
	self assert: resolution resourcePath = #('foo' 'bar') asOrderedCollection.
	resolution advance.
	self 
		assert: resolution resourcePath = #('foo' 'bar' 'baz') asOrderedCollection.
	resolution advance.
	self assert: resolution resourcePath = #('foo' 'bar' 'baz' 'quux') asOrderedCollection! !

!URIResolutionTest methodsFor: 'testing' stamp: ' 10/7/07 17:21'!
testSiteAnswer
	| resource request response |
	resource := Site new host: 'foo.com' ip: '1.2.3.4' port: 80.
	resource addResource: (HelloWorldResource uriPattern: '/').
	request := HTTPGet
				request: '/'
				from: 'foo.com'
				at: '1.2.3.4'.
	response := URIResolution resolveRequest: request startingAt: resource.
	self assert: response code = 200.
	self assert: request resourcePath size = 1.
	self assert: request resourcePath first = '/'! !

!URIResolutionTest methodsFor: 'testing' stamp: ' 10/7/07 17:21'!
testSiteMatch
	| request site visitor |
	request := HTTPGet 
				request: 'foo'
				from: 'localhost:1234'
				at: '1.2.3.4'.
	visitor := URIResolution new initializeRequest: request.
	site := Site new host: 'localhost' ip: '1.2.3.4' port: 1234.
	self assert: (visitor siteMatch: site)! !

!URIResolutionTest methodsFor: 'testing' stamp: ' 10/7/07 17:21'!
testSiteMismatch
	| request site visitor |
	request := HTTPGet 
				request: 'foo'
				from: 'localhost:1234'
				at: '1.2.3.4'.
	visitor := URIResolution new initializeRequest: request.
	site := Site new host: 'remotehost' ip: '1.2.3.4' port: 1234.
	self deny: (visitor siteMatch: site)! !

!URIResolutionTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testStringMatch
	| request visitor resource |
	request := HTTPGet request: 'foo'.
	visitor := URIResolution new initializeRequest: request.
	resource := HelloWorldResource uriPattern: 'foo'.
	self assert: (visitor stringMatch: resource)! !

!URIResolutionTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testStringMismatch
	| request visitor resource |
	request := HTTPGet request: 'foo'.
	visitor := URIResolution new initializeRequest: request.
	resource := HelloWorldResource uriPattern: 'Foo'.
	self deny: (visitor stringMatch: resource)! !

!URIResolutionTest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
testTailPath
	| request resolution |
	request := HTTPGet 
				request: 'foo/bar/baz/quux'
				from: 'localhost:1234'
				at: '1.2.3.4'.
	resolution := URIResolution new initializeRequest: request.
	self 
		assert: resolution tailPath = #('bar' 'baz' 'quux') asOrderedCollection.
	resolution advance.
	self assert: resolution tailPath = #('baz' 'quux') asOrderedCollection.
	resolution advance.
	self assert: resolution tailPath = #('quux') asOrderedCollection.
	resolution advance.
	self assert: resolution tailPath isEmpty! !

ReadWriteStream subclass: #SwazooBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Compatibility'!

!SwazooBuffer commentStamp: 'mivsek 7/22/2007 19:10' prior: 0!
SwazooBuffer is used for efficient buffering of receiving or sending dato to TCP socket. Efficiency is achieved with reusing of stream content array instead of initializing it everytime buffer is emptied, as was in previous Swazoo versions. 

SwazooBuffer is a subclass of ReadWriteStream, with additional #clear to empty buffer.!

!SwazooBuffer class methodsFor: 'defaults' stamp: 'mivsek 7/22/2007 19:13'!
bufferSize
	"approximate length of buffer. not nessesary exactly this number!! "
	^16384 "16KB"! !

!SwazooBuffer class methodsFor: 'instance creation' stamp: 'mivsek 7/22/2007 19:12'!
new
	^super on: (ByteArray new: self bufferSize)! !

!SwazooBuffer methodsFor: 'clearing' stamp: 'mivsek 7/22/2007 19:10'!
clear
	"reset as you'd make another one, just don't replace content array"
	self reset.
	readLimit := 0.! !

!SwazooBuffer methodsFor: 'writing' stamp: 'mivsek 7/22/2007 19:11'!
flushTo: aSocket
	"actually write to the tcp socket as direclty as possible"
	| contents remaining start written |
	contents := self contents.
	remaining := contents size.
	[remaining > 0] whileTrue: 
		[start := contents size - remaining + 1.
		written := aSocket 
			writeFrom: contents startingAt: start for: (contents size - start + 1).
"			write: (contents copyFrom: contents size - remaining + 1 to: contents size). "
		remaining := remaining - written].
	self clear.! !

Object subclass: #HTTPConnection
	instanceVariableNames: 'stream loop server lastResponse'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!HTTPConnection class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
socket: aSocket 
	^self new stream: aSocket stream! !

!HTTPConnection methodsFor: 'serving' stamp: 'mivsek 7/23/2007 22:51'!
close
	self stream notNil ifTrue: [self stream close. stream := nil].
	self server removeConnection: self.
	self loop notNil ifTrue: [self loop terminate. self loop: nil]! !

!HTTPConnection methodsFor: 'private-handling' stamp: ' 14/4/07 23:09'!
getAndDispatchMessages
	"^self
	The HTTPRequest is read from my socket stream.  I then pass this request to my server to get a response.  If somethng goes wrong while getting hold of the request, we call that a 400 - a bad request.  If anything happens after that, it's an internal server error.  (hmm - should we close the image if we get an internal server error?  If we don't, what state is the image in, then ... ??)"

	| task |
	task := SwazooTask new.
	self readRequestFor: task.
	self produceResponseFor: task.
	"self close"! !

!HTTPConnection methodsFor: 'serving' stamp: ' 14/4/07 23:09'!
interact
	"longer description is below method"
	| interactionBlock |
	interactionBlock := 
		[SpExceptionContext 
			for: [[true] whileTrue: 
				[self getAndDispatchMessages.
				Processor yield] ]
		on: SpError
		do: [:ex | self close]].
	self server isMultiThreading 
		ifTrue: [self loop: (interactionBlock forkAt: Processor userBackgroundPriority)]
		ifFalse: [interactionBlock value].
	^self

"I represent a specifc connection with an HTTP client (a browser, probably) over which will come an HTTP request.  Here, I fork the handling of the request so that the current thread (which is most likely the HTTP server main loop) can carry on with the next request.  This means that more than one request may being handled in the image at a time, and that means that the application developer must worry about thread safety - e.g the problem of a given business object being updated by more than one HTTP request thread.
For a GemStone implementation of Swazoo, one may want only one request is handled at a time, multi-threadedness being handled by having multiple gems.  This is a nice option because the application developer does not have to worry about thread safety in this case - GemStone handles the hard stuff.
*And* the thing called a loop that was in this method was no such thing.  In all circumstances, >>getAndDispatchMessages handles exactly one requst and then closes the socket!! (very non-HTTP1.1).  Anyway, for now I'm just going to make that explicit.  This needs to be re-visited to support HTTP 1.1."! !

!HTTPConnection methodsFor: 'private-accessing' stamp: ' 14/4/07 23:09'!
lastResponse
	"to pair with new request, for http digest authentication etc."
	^lastResponse! !

!HTTPConnection methodsFor: 'private-accessing' stamp: ' 14/4/07 23:09'!
lastResponse: aHTTPResponse
	lastResponse := aHTTPResponse! !

!HTTPConnection methodsFor: 'private-accessing' stamp: ' 14/4/07 23:09'!
loop
	^loop! !

!HTTPConnection methodsFor: 'private-accessing' stamp: ' 14/4/07 23:09'!
loop: aProcess
	loop := aProcess! !

!HTTPConnection methodsFor: 'private-handling' stamp: ' 14/4/07 23:09'!
nextPutError: aResponse 
	aResponse informConnectionClose.
	aResponse writeTo: self stream.
	self stream flush.! !

!HTTPConnection methodsFor: 'private-handling' stamp: ' 14/4/07 23:09'!
nextPutResponse: aMessage toRequest: aRequest 
	| shouldClose |
	shouldClose := aRequest wantsConnectionClose.
	shouldClose ifTrue: [aMessage informConnectionClose].
	aRequest isHead 
		ifTrue: [aMessage writeHeaderTo: self stream]
		ifFalse: [aMessage writeTo: self stream].
	self stream flush.
	shouldClose ifTrue: [self close].
	aMessage isRedirectLink ifTrue: [self close]. "otherwise browser does not redirect?!! "! !

!HTTPConnection methodsFor: 'private-handling' stamp: 'mivsek 7/19/2007 22:17'!
produceResponseFor: aTask 
	"Given the request in aTask I try to make a response.  If there are any unhandled 
	exceptions, respond with an internal server error."
	aTask request isNil ifTrue: [^nil].
	SpExceptionContext for: 
		[aTask response: (self server answerTo: aTask request).
		aTask request ensureFullRead. "in case if defered parsing not done in HTTPost"
		self nextPutResponse: aTask response toRequest: aTask request]
	onAnyExceptionDo: 
		[:ex | 
		self nextPutError: HTTPResponse internalServerError.
		ex defaultAction. "usually raise an UHE window"
		self close]! !

!HTTPConnection methodsFor: 'private-handling' stamp: 'JM 6/6/2007 11:57'!
readRequestFor: aSwazooTask 
	"I read the next request from my socket and add it to aSwazooTask.  If I have any 
	problems and need to force a bad request (400) response, I add this response to aSwazooTask."
	| request |
	SpExceptionContext for: 
		[request := HTTPRequest readFrom: self stream.
		request uri port: self server port.
		(request httpVersion last = 1 
			and: [(request headers includesFieldOfClass: HTTPHostField) not]) 
				ifTrue: [aSwazooTask response: HTTPResponse badRequest].
		request
			peer: self stream socket remoteAddress;
			ip: self stream socket localAddress;
			setTimestamp; parent: self.
		aSwazooTask request: request]
	on: SpError, HTTPException
	do: [:ex | 
		aSwazooTask response: HTTPResponse badRequest.
		self nextPutError: aSwazooTask response.
		self close].! !

!HTTPConnection methodsFor: 'private-accessing' stamp: ' 14/4/07 23:09'!
server
	^server! !

!HTTPConnection methodsFor: 'private-accessing' stamp: ' 14/4/07 23:09'!
server: aServer 
	server := aServer! !

!HTTPConnection methodsFor: 'private-accessing' stamp: ' 14/4/07 23:09'!
socket
	^self stream socket! !

!HTTPConnection methodsFor: 'private-accessing' stamp: ' 14/4/07 23:09'!
stream
	^stream! !

!HTTPConnection methodsFor: 'private-accessing' stamp: ' 14/4/07 23:09'!
stream: aSwazooStream 
	stream := aSwazooStream! !

Object subclass: #HTTPHeaders
	instanceVariableNames: 'fields'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPHeaders class methodsFor: 'instance creation' stamp: ' 10/7/07 17:21'!
readFrom: aSwazooStream 
	"^an HTTPHeaders
I return a new instance of myself which contains fields parsed from aStream."

	^self new readFrom: aSwazooStream! !

!HTTPHeaders methodsFor: 'services' stamp: ' 14/4/07 23:10'!
addField: aField 
	"HTTPSpec1.1 Sec4.2
Multiple message-header fields with the same field-name MAY be present in a message if and only if the entire field-value for that header field is defined as a comma-separated list [i.e., #(values)]. It MUST be possible to combine the multiple header fields into one 'field-name: field-value' pair, without changing the semantics of the message, by appending each subsequent field-value to the first, each separated by a comma. The order in which header fields with the same field-name are received is therefore significant to the interpretation of the combined field value, and thus a proxy MUST NOT change the order of these field values when a message is forwarded.
Note that we have to use the field name here as we may be adding a field for which there is no class, i.e. it's a GenericHeaderField."

	(self includesFieldNamed: aField name) 
		ifTrue: [(self fieldNamed: aField name) combineWith: aField]
		ifFalse: [self fields at: aField name asUppercase put: aField].
	^self! !

!HTTPHeaders methodsFor: 'emitting' stamp: ' 14/4/07 23:10'!
crlfOn: aStream 
	aStream
		nextPut: Character cr;
		nextPut: Character lf! !

!HTTPHeaders methodsFor: 'services' stamp: ' 14/4/07 23:10'!
fieldNamed: aString 
	"^aString
If I contain a field named aString, I return it.  Otherwise an exception is thrown.
This is a bad way of getting a field.  Use >> fieldOfClass: instead."

	| targetString |
	targetString := aString asUppercase.
	^self fields detect: [:aField | aField name asUppercase = targetString]! !

!HTTPHeaders methodsFor: 'services' stamp: ' 14/4/07 23:10'!
fieldNamed: aString ifNone: aBlock 
	"^aString
If I contain a field named aString, I return it.  Otherwise I evaluate aBlock."

	^self fields at: aString asUppercase ifAbsent: aBlock! !

!HTTPHeaders methodsFor: 'services' stamp: ' 14/4/07 23:10'!
fieldNamed: aFieldName ifPresent: presentBlock ifAbsent: absentBlock 
	"^an Object
I look for a field named aFieldName among my fields.  If I find it, I return the result of evaluating presentBlock with the found field as an argument, otherwise I return the result of evaluate the absentBlock"

	| foundField |
	foundField := self fieldNamed: aFieldName ifNone: [nil].
	^foundField isNil 
		ifTrue: [absentBlock value]
		ifFalse: [presentBlock value: foundField]! !

!HTTPHeaders methodsFor: 'services' stamp: ' 14/4/07 23:10'!
fieldOfClass: aClass 
	"^aString
If I contain a field of class aClass, I return it.   Otherwise an exception is thrown."

	^self fields detect: [:aField | aField class == aClass] ifNone: [^nil]! !

!HTTPHeaders methodsFor: 'services' stamp: ' 14/4/07 23:10'!
fieldOfClass: aClass ifNone: aBlock 
	"^aString
If I contain a field of class aClass, I return it.   Otherwise I evaluate aBlock."

	^self fields detect: [:aField | aField class == aClass] ifNone: aBlock! !

!HTTPHeaders methodsFor: 'services' stamp: ' 14/4/07 23:10'!
fieldOfClass: fieldClass ifPresent: presentBlock ifAbsent: absentBlock 
	"^an Object
I look for a field of class fieldClass among my fields.  If I find it, I return the result of evaluating presentBlock with the found field as an argument, otherwise I return the result of evaluate the absentBlock"

	| foundField |
	foundField := self fieldOfClass: fieldClass ifNone: [nil].
	^foundField isNil 
		ifTrue: [absentBlock value]
		ifFalse: [presentBlock value: foundField]! !

!HTTPHeaders methodsFor: 'private' stamp: ' 14/4/07 23:10'!
fields
	fields isNil ifTrue: [fields := Dictionary new].
	^fields! !

!HTTPHeaders methodsFor: 'services' stamp: ' 14/4/07 23:10'!
getOrMakeFieldOfClass: aClass 
	"^a HeaderField
If I contain a field of class aClass, I return it.   Otherwise I create a new instance if the field class and add it to my collection of headers."

	^self fieldOfClass: aClass
		ifNone: 
			[| newField |
			newField := aClass new.
			self addField: newField.
			newField]! !

!HTTPHeaders methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
includesFieldNamed: aString 
	"^a Boolean
I return true if one of my fields has the name aString."

	| targetField |
	targetField := self fieldNamed: aString ifNone: [nil].
	^targetField notNil! !

!HTTPHeaders methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
includesFieldOfClass: aClass 
	"^a Boolean
I return true if one of my fields is of class aClass."

	^self 
		fieldOfClass: aClass
		ifPresent: [:aField | true]
		ifAbsent: [false]! !

!HTTPHeaders methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
readFieldFromString: aString 
	"^self
First I get the field parsed from aString, then I add the new field to my collection of fields.  Adding the new field may involve merging field values if I already have a field of that class."

	self addField: (HeaderField fromLine: aString).
	^self! !

!HTTPHeaders methodsFor: 'initialize-release' stamp: ' 10/7/07 17:21'!
readFrom: aSwazooStream 
	"^an HTTPHeaders
I return a new instance of myself which contains fields parsed from aStream.  Everything upto the next blank line is a header field."
	
	| nextLine |
	[nextLine := aSwazooStream nextUnfoldedLine.
	nextLine isEmpty]
		whileFalse: [self readFieldFromString: nextLine].
	^self! !

!HTTPHeaders methodsFor: 'emitting' stamp: ' 14/4/07 23:10'!
writeOn: aStream 
	"^self
I write all my fields to aStream."

	self fields do: 
			[:aField | 
			aField printOn: aStream.
			self crlfOn: aStream]! !

Object subclass: #HTTPMessage
	instanceVariableNames: 'parent headers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPMessage methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
addInitialHeaders
	"^self 
This is a no-op.  My subclasses may wish to add some initial headers."

	^self! !

!HTTPMessage methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
headers
	headers isNil ifTrue: [self initHeaders].
	^headers! !

!HTTPMessage methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
initHeaders
	headers := HTTPHeaders new.
	self addInitialHeaders.! !

!HTTPMessage methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
parent
	^parent! !

!HTTPMessage methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
parent: aConnection
	parent := aConnection! !

HTTPMessage subclass: #HTTPRequest
	instanceVariableNames: 'requestLine peer timestamp ip environmentData resolution encrypted authenticated'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

HTTPRequest subclass: #HTTPDelete
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPDelete commentStamp: '<historical>' prior: 0!
HTTPDelete 

rfc26216 section 9.7

The DELETE method requests that the origin server delete the resource
   identified by the Request-URI. This method MAY be overridden by human
   intervention (or other means) on the origin server. The client cannot
   be guaranteed that the operation has been carried out, even if the
   status code returned from the origin server indicates that the action
   has been completed successfully. However, the server SHOULD NOT
   indicate success unless, at the time the response is given, it
   intends to delete the resource or move it to an inaccessible
   location.
 ...
!

!HTTPDelete class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
methodName
	"HTTP method used for a request"
	^'DELETE'! !

!HTTPDelete methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isDelete
	^true! !

HTTPRequest subclass: #HTTPGet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPGet commentStamp: '<historical>' prior: 0!
HTTPGet 

rfc26216 section 9.3

   The GET method means retrieve whatever information (in the form of an
   entity) is identified by the Request-URI. If the Request-URI refers
   to a data-producing process, it is the produced data which shall be
   returned as the entity in the response and not the source text of the
   process, unless that text happens to be the output of the process.
!

!HTTPGet class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
methodName
	^'GET'! !

!HTTPGet methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isGet
	^true! !

HTTPRequest subclass: #HTTPHead
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPHead commentStamp: '<historical>' prior: 0!
HTTPHead

rfc26216 section 9.4

   The HEAD method is identical to GET except that the server MUST NOT
   return a message-body in the response. The metainformation contained
   in the HTTP headers in response to a HEAD request SHOULD be identical
   to the information sent in response to a GET request. This method can
   be used for obtaining metainformation about the entity implied by the
   request without transferring the entity-body itself. This method is
   often used for testing hypertext links for validity, accessibility,
   and recent modification.

!

!HTTPHead class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
methodName
	^'HEAD'! !

!HTTPHead methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isHead
	^true! !

HTTPRequest subclass: #HTTPOptions
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPOptions commentStamp: '<historical>' prior: 0!
HTTPOptions

rfc26216 section 9.2

   The OPTIONS method represents a request for information about the
   communication options available on the request/response chain
   identified by the Request-URI. This method allows the client to
   determine the options and/or requirements associated with a resource,
   or the capabilities of a server, without implying a resource action
   or initiating a resource retrieval.

!

!HTTPOptions class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
methodName
	^'OPTIONS'! !

!HTTPOptions methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isOptions
	^true! !

!HTTPOptions methodsFor: 'services' stamp: ' 14/4/07 23:10'!
respondUsing: responseBlock 
	"^an HTTPResponse
I represent a request for the options supported by this server.  I respond with a 200 (OK) and a list of my supported methods in an Allow: header.  I ignore the responseBlock."

	| response allowField |
	response := HTTPResponse ok.
	allowField := HTTPAllowField new.
	allowField methods addAll: self class allMethodNames.
	response headers addField: allowField.
	^response! !

HTTPRequest subclass: #HTTPPost
	instanceVariableNames: 'postData entityBody readPosition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPPost commentStamp: '<historical>' prior: 0!
HTTPPost 

rfc26216 section 9.5

   The POST method is used to request that the origin server accept the
   entity enclosed in the request as a new subordinate of the resource
   identified by the Request-URI in the Request-Line.

Instance Variables:
	entityBody	<>	
	postData	<HTTPPostDataArray>	

!

!HTTPPost class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
methodName
	^'POST'! !

!HTTPPost methodsFor: 'private' stamp: ' 14/4/07 23:10'!
applicationOctetStreamFrom: aStream 
	"^self 
rfc 2046 says:
The recommended action for an implementation that receives an 'application/octet-stream' entity is to simply offer to put the data in a file, with any Content-Transfer-Encoding undone, or perhaps to use it as input to a user-specified process.
This method used to do a crlf -> cr conversion on the octet-stream, but was not clear why."

	self entityBody: (HTTPString 
				stringFromBytes: (aStream nextBytes: self contentLength)).
	^self! !

!HTTPPost methodsFor: 'accessing' stamp: 'mivsek 7/19/2007 22:13'!
emptyData
	self ensureFullRead.
	^self postData select: [:each | each value isEmpty]! !

!HTTPPost methodsFor: 'parsing' stamp: 'mivsek 7/19/2007 22:11'!
ensureFullRead
	"that is, everything is read from a socket stream. Important because of defered parsing
	of postData"
	self postData isParsed ifFalse: 
		[self parsePostDataFrom: self postData stream.
		self postData setParsed]! !

!HTTPPost methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
entityBody
	^entityBody! !

!HTTPPost methodsFor: 'private' stamp: ' 14/4/07 23:10'!
entityBody: aString
	entityBody := aString! !

!HTTPPost methodsFor: 'parsing' stamp: 'mivsek 7/19/2007 22:12'!
for: aRequestLine readFrom: aSwazooStream 
	super for: aRequestLine readFrom: aSwazooStream.
	self initPostDataFor: aSwazooStream.
"	self parsePostDataFrom: aSwazooStream." "defered until first access of postData!! "! !

!HTTPPost methodsFor: 'private' stamp: ' 10/7/07 17:21'!
incReadPosition
	self readPosition: self readPosition + 1! !

!HTTPPost methodsFor: 'initialize-release' stamp: 'mivsek 7/19/2007 22:12'!
initPostDataFor: aSwazooStream
	postData := HTTPPostDataArray newOn: aSwazooStream! !

!HTTPPost methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isPost
	^true! !

!HTTPPost methodsFor: 'testing' stamp: 'mivsek 7/19/2007 22:13'!
isPostDataEmpty
	self ensureFullRead.
	^self postData isEmpty! !

!HTTPPost methodsFor: 'testing' stamp: 'mivsek 7/21/2007 22:47'!
isPostDataStreamedAt: aKey
	^(self postData at: aKey ifAbsent: [^false]) isStreamed! !

!HTTPPost methodsFor: 'private-parsing' stamp: 'mivsek 7/23/2007 22:59'!
multipartDataFrom: aSwazooStream 
	"read all mime parts and put them in postData" 
	"read directly from stream, without intermediate buffers"
	| boundary contentTypeField |
	self headers fieldOfClass: HTTPContentLengthField
		ifNone: [^SwazooHTTPPostError raiseSignal: 'Content-Length header missing'].
	contentTypeField := self headers fieldOfClass: ContentTypeField
				ifNone: [^aSwazooStream nextBytes: self contentLength]. "just skip"
	boundary := contentTypeField transferCodings at: 'boundary'
				ifAbsent: [^aSwazooStream nextBytes: self contentLength]. "just skip"
	[self readPosition < self contentLength] whileTrue: 
			[ | part |
			part := self partFromStream: aSwazooStream boundary: boundary asByteArray.
			part notNil ifTrue: [self postDataAt: part key put: part value]]! !

!HTTPPost methodsFor: 'private-parsing support' stamp: ' 10/7/07 17:21'!
nextFrom: aSwazooStream boundary: aBoundaryBytes
	"usually return one byte but it can be more if content looks like a boundary"
	| out inx shift |
	aSwazooStream peek = $-  ifFalse: "not part of boundary"
		[self incReadPosition. ^ByteArray with: aSwazooStream nextByte].
	out := WriteStream on: ByteArray new. 
	inx := 1. shift := 0.
	[inx <= aBoundaryBytes size] whileTrue: 
		[(aBoundaryBytes at: inx) ~= aSwazooStream peekByte ifTrue:  "not a boundary?"
			[(aSwazooStream peek = $- and: [shift = 0]) 
				ifTrue: 
					[ [aSwazooStream peek = $- ] whileTrue: 
						[out nextPut: aSwazooStream nextByte. 
						shift := shift + 1. self incReadPosition] ]
				ifFalse: [^out contents copyFrom: 1 to: out contents size - shift] ]. 
		out nextPut: aSwazooStream nextByte. self incReadPosition. inx := inx + 1].
	^out contents size = aBoundaryBytes size 
		ifTrue: [nil]  "boundary was actually found. We just skip it and return nil to signal a boundary"
		ifFalse: [out contents copyFrom: 1 to: out contents size - aBoundaryBytes size]! !

!HTTPPost methodsFor: 'private-parsing support' stamp: 'mivsek 7/23/2007 23:00'!
nextPut: byte to: aQueue afterFourTo: aSwazooStream
	"buffer four bytes which are at the end of mime part not writen to output stream (crlf --)"
	aQueue add: byte.
	aQueue size > 4 ifTrue: [aSwazooStream nextPut: aQueue removeFirst]! !

!HTTPPost methodsFor: 'private-parsing support' stamp: 'mivsek 7/24/2007 21:06'!
nextPutAll: aByteArray to: aQueue afterFourTo: aWriteStream
	"buffer four bytes which are at the end of mime part not writen to output stream (crlf --)"
	aByteArray do: [:byte | self nextPut: byte to: aQueue afterFourTo: aWriteStream]! !

!HTTPPost methodsFor: 'parsing' stamp: 'mivsek 7/23/2007 23:00'!
parsePostDataFrom: aSwazooStream 
	| mediaType |
	((self headers includesFieldOfClass: ContentTypeField) 
		and: [self headers includesFieldOfClass: HTTPContentLengthField]) 
			ifFalse: 
				[^SwazooHTTPPostError 
					raiseSignal: 'Both Content-Type and Content-Length needed'].
	mediaType := (self headers fieldOfClass: ContentTypeField) mediaType.
	mediaType = 'application/x-www-form-urlencoded' 
		ifTrue: [^self urlencodedDataFrom: aSwazooStream].
	mediaType = 'multipart/form-data' 
"		ifTrue: [^self multipartDataFrom: aStream]"
		ifTrue: [^self multipartDataFrom: aSwazooStream]
		ifFalse: [^self applicationOctetStreamFrom: aSwazooStream].! !

!HTTPPost methodsFor: 'private-parsing' stamp: 'mivsek 7/26/2007 13:37'!
partFromStream: aSwazooStream boundary: aBoundaryBytes
	"one mime part from a stream"
	| bytes name filename datum contentType |
	self trimBlanksFrom: aSwazooStream.
	bytes := self nextFrom: aSwazooStream boundary: aBoundaryBytes.
	(bytes isNil or: [bytes = '--' asByteArray])                                "end of multipart data"
		ifTrue: [aSwazooStream next. self incReadPosition. ^nil].  "one byte not read ??"
	name := nil. datum := nil. contentType := nil. "just to avoid compliation warning"
	[true] whileTrue: [| line |  "read all lines and at the end a body of that part"
		line := (aSwazooStream upTo: Character cr asInteger) asString. 
		self readPosition: self readPosition + line size + 1 "cr".
		line := bytes asString, line. bytes := ''.
		aSwazooStream peekByte = Character lf asInteger ifTrue: [| field | "this is a name line"
			aSwazooStream nextByte.  self incReadPosition. "skip linefeed"
			line isEmpty 	ifTrue: [| | "empty line indicates start of entity"
				name isNil ifTrue: [^nil].  "name must be read in previous circle"
				^name -> (self readEntityFrom: aSwazooStream 
					datum: datum boundary: aBoundaryBytes)].
			field := HeaderField fromLine: line.
			field isContentDisposition ifTrue: 
					[name := (field parameterAt: 'name') copyWithout: $". 
					datum := (self isPostDataStreamedAt: name)
						ifTrue: [self postData at: name]  "streamed datum must exist before"
						ifFalse: [HTTPPostDatum new].
					contentType notNil ifTrue: [datum contentType: contentType]. 
					filename := field parameterAt: 'filename' .   "only for file uploads"
					filename notNil ifTrue: [datum filename: (filename copyWithout: $")] ].
			field isContentType ifTrue: [contentType := field mediaType ] ] ]! !

!HTTPPost methodsFor: 'private' stamp: 'mivsek 7/19/2007 22:12'!
postData
	^postData! !

!HTTPPost methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
postDataAt: aKey 
	^self postDataAt: aKey ifAbsent: [nil]! !

!HTTPPost methodsFor: 'accessing' stamp: 'mivsek 7/24/2007 21:05'!
postDataAt: aKey beforeStreamingDo: aBlockClosure
	"announce that you want to receive post data directly to a binary stream, which will be set
	by aBlockClosure. That block must receive and argument, which is a HTTPostDatum and 
	here it can set a writeStream"
	"Fails if post data is already read"
	self postData isParsed 
		ifTrue: [^self error: 'HTTPost already parsed, streaming not possible anymore!!'].
	^self postDataAt: aKey put: (HTTPPostDatum new writeBlock: aBlockClosure)! !

!HTTPPost methodsFor: 'accessing' stamp: 'mivsek 7/19/2007 22:15'!
postDataAt: aKey do: aBlock 
	| val |
	self ensureFullRead. "defered parsing of postData"
	val := self postData at: aKey ifAbsent: [nil].
	val isNil ifFalse: [aBlock value: val]! !

!HTTPPost methodsFor: 'accessing' stamp: 'mivsek 7/19/2007 22:15'!
postDataAt: aKey ifAbsent: aBlock 
	self ensureFullRead. "defered parsing of postData"
	^self postData at: aKey ifAbsent: aBlock! !

!HTTPPost methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
postDataAt: aKey put: aPostDatum 
	"for testing purposes"

	self postData at: aKey put: aPostDatum! !

!HTTPPost methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
postDataAt: aKey putString: aString 
	"for testing purposes"

	self postDataAt: aKey put: (HTTPPostDatum new value: aString)! !

!HTTPPost methodsFor: 'accessing' stamp: 'mivsek 7/20/2007 23:45'!
postDataAt: aKey streamTo: aWriteStream
	"announce that you want to receive post data directly to aWriteStream, 
	which must be binary. Fails if post data is already read"
	self postData isParsed 
		ifTrue: [^self error: 'HTTPost already parsed, streaming not possible anymore!!'].
	^self postDataAt: aKey put: (HTTPPostDatum new writeStream: aWriteStream)! !

!HTTPPost methodsFor: 'accessing' stamp: 'mivsek 7/19/2007 22:15'!
postDataKeys
	self ensureFullRead. "defered parsing of postData"
	^self postData keys! !

!HTTPPost methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
postDataStringAt: aKey 
	^(self postDataAt: aKey ifAbsent: [^nil]) value! !

!HTTPPost methodsFor: 'accessing' stamp: 'mivsek 7/19/2007 22:15'!
postKeysAndValuesDo: aTwoArgBlock 
	self ensureFullRead. "defered parsing of postData"
	self postData 
		keysAndValuesDo: [:key :each | aTwoArgBlock value: key value: each value]! !

!HTTPPost methodsFor: 'private-parsing' stamp: 'mivsek 7/24/2007 21:09'!
readEntityFrom: aSwazooStream datum: aDatum boundary: aBoundaryBytes
	"read one entity from a stream and put into datum. Stream it if streamed. Also call a block 
	(if any) just before start of streaming, with a datum as parameter. This block can then set 
	a write stream in datum (for instance open a output file and stream on it)"
	| outStream bytes queue |
	aDatum writeBlock notNil 
		ifTrue: [aDatum writeBlock value: aDatum]. "this should set writeStream if not already!!" 
	outStream := aDatum isStreamed
		ifTrue: [aDatum writeStream] ifFalse: [WriteStream on: ByteArray new].
	queue := OrderedCollection new. "to buffer 4bytes for crlf -- detection at the end"
	[aSwazooStream peek = $- ] whileFalse:
		[self nextPut: aSwazooStream nextByte to: queue afterFourTo: outStream.
		self incReadPosition].
	bytes := ''. 
	[bytes size > 1] whileFalse:  "we are on boundary if bytes is longer than 1"
		[bytes := self nextFrom: aSwazooStream boundary: aBoundaryBytes.
		self nextPutAll: bytes to: queue afterFourTo: outStream].
	aDatum isStreamed not ifTrue: "otherwise entity is already streamed to the output"
		[aDatum value: outStream contents asString].
	^aDatum! !

!HTTPPost methodsFor: 'private' stamp: ' 10/7/07 17:21'!
readPosition
	"position in a read stream. just temporary"
	readPosition isNil ifTrue: [^1].
	^readPosition! !

!HTTPPost methodsFor: 'private' stamp: ' 10/7/07 17:21'!
readPosition: aNumber
	readPosition := aNumber! !

!HTTPPost methodsFor: 'private-parsing support' stamp: ' 10/7/07 17:21'!
trimBlanksFrom: aSwazooStream
	"skip over all blanks until next nonblank character"
	[aSwazooStream peek isSeparator]
		whileTrue: 
			[aSwazooStream nextByte. 
			self incReadPosition]! !

!HTTPPost methodsFor: 'private-parsing' stamp: ' 10/7/07 17:21'!
urlencodedDataFrom: aStream 
	| e tokens |
	(self headers includesFieldOfClass: HTTPContentLengthField) ifFalse: [^self].
	e := aStream nextBytes: self contentLength.
	tokens := HTTPString subCollectionsFrom: (HTTPString stringFromBytes: e)
				delimitedBy: $&.
	(tokens 
		collect: [:each | HTTPString subCollectionsFrom: each delimitedBy: $=]) 
			do: 
				[:keyVal | 
				| datum |
				datum := HTTPPostDatum new.
				datum 
					value: (HTTPString decodedHTTPFrom: (keyVal last 
									collect: [:char | char = $+ ifTrue: [Character space] ifFalse: [char]])).
				self postDataAt: keyVal first put: datum]! !

HTTPRequest subclass: #HTTPPut
	instanceVariableNames: 'putData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPPut commentStamp: '<historical>' prior: 0!
HTTPPut 

rfc26216 section 9.6

   The PUT method requests that the enclosed entity be stored under the
   supplied Request-URI. If the Request-URI refers to an already
   existing resource, the enclosed entity SHOULD be considered as a
   modified version of the one residing on the origin server. If the
   Request-URI does not point to an existing resource, and that URI is
   capable of being defined as a new resource by the requesting user
   agent, the origin server can create the resource with that URI. If a
   new resource is created, the origin server MUST inform the user agent
   via the 201 (Created) response. If an existing resource is modified,
   either the 200 (OK) or 204 (No Content) response codes SHOULD be sent
   to indicate successful completion of the request. If the resource
   could not be created or modified with the Request-URI, an appropriate
   error response SHOULD be given that reflects the nature of the
   problem. The recipient of the entity MUST NOT ignore any Content-*
   (e.g. Content-Range) headers that it does not understand or implement
   and MUST return a 501 (Not Implemented) response in such cases.

Instance Variables:
	putData	<>	

!

!HTTPPut class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
methodName
	^'PUT'! !

!HTTPPut methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isPut
	^true! !

!HTTPPut methodsFor: 'reading' stamp: ' 14/4/07 23:10'!
octetDataFrom: aStream 
	self headers fieldOfClass: HTTPContentLengthField
		ifNone: [^SwazooHTTPPutError raiseSignal: 'Missing Content-Length'].
	self putData: (aStream nextBytes: self contentLength)! !

!HTTPPut methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
putData
	^putData! !

!HTTPPut methodsFor: 'private' stamp: ' 14/4/07 23:10'!
putData: aString
	putData := aString! !

!HTTPPut methodsFor: 'reading' stamp: ' 14/4/07 23:10'!
readFrom: aStream 
	| contentTypeField |
	super readFrom: aStream.
	contentTypeField := self headers fieldOfClass: ContentTypeField
				ifNone: [SwazooHTTPPutError raiseSignal: 'Missing Content-Type'].
	contentTypeField mediaType = 'application/octet-stream' 
		ifTrue: [self octetDataFrom: aStream]
		ifFalse: [self urlencodedDataFrom: aStream].
	^self! !

!HTTPRequest class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
allMethodNames
	"...of all request methods we support there"
	self subclasses collect: [:each | each methodName].! !

!HTTPRequest class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
methodName
	"HTTP method used for a request"
	^self subclassResponsibility! !

!HTTPRequest class methodsFor: 'instance creation' stamp: ' 10/7/07 17:21'!
newFor: aRequestLine readFrom: aSwazooStream 
	"to support an additional http method, simply subclass a HTTPRequest!!"
	| targetClass |
	targetClass := aRequestLine method = 'GET' 
		ifTrue: [HTTPGet] "most used anyway"
		ifFalse: [aRequestLine method = 'POST'  
			ifTrue: [HTTPPost]  "second most used"
			ifFalse: [self subclasses detect: [:each | 
				each methodName = aRequestLine method] ifNone: [nil] ] ].
	targetClass isNil ifTrue: [^HTTPException notImplemented].
	^targetClass new for: aRequestLine readFrom: aSwazooStream! !

!HTTPRequest class methodsFor: 'instance creation' stamp: ' 10/7/07 17:21'!
readFrom: aSwazooStream 
	"^an HTTPRequest
I create and return a new instance of one of my subclasses which will represent the HTTP request presumed to be the contents of aStream.
The first step is to work out which of my subclasses to create.  I do this by parsing the 'request-line' from the stream.  The request-line contains the 'method', and I look for the subclass that handles this method and delegate the rest of the message parsing to a new instance of that class."

	| requestLine |
	requestLine := HTTPRequestLine readFrom: aSwazooStream.
	^self newFor: requestLine readFrom: aSwazooStream! !

!HTTPRequest class methodsFor: 'tests support' stamp: ' 14/4/07 23:10'!
request: aUriString 
	"For testing only (I'm guessing / hoping!!).  The idea to to create a request for a resource with the URI 'someHost/aUriString'."

	^self 
		request: aUriString
		from: 'someHost'
		at: 'someIP'! !

!HTTPRequest class methodsFor: 'tests support' stamp: ' 14/4/07 23:10'!
request: aUriString from: aHostString at: anIPString 
	"For testing only (I'm guessing / hoping!!).
A request is manufactured that has a request line method of >>methodName and a request line URI with an identifier of aUriString.  A Host header is added to the headers and the ip address is set to anIP string.
This may result in a corrupt or invalid request, but that's the natutre of testing, I guess."

	^self new 
		request: aUriString
		from: aHostString
		at: anIPString! !

!HTTPRequest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
authenticated
	^authenticated! !

!HTTPRequest methodsFor: 'services' stamp: ' 14/4/07 23:10'!
conditionalHeaderFields
	"^an OrderedCollection
I return my collection of conditional header fields.  A conditional GET requires that each of these is checked against the current state of the target resource."

	^self headers fields select: [:aField | aField isConditional]! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
connection
	^(self headers fieldOfClass: HTTPConnectionField ifNone: [^nil]) 
		connectionToken! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
contentLength
	^(self headers fieldOfClass: HTTPContentLengthField) contentLength! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
cookie
	| field |
	field := self headers fields at: 'COOKIE' ifAbsent: [^nil].
	^field value

"	field := self headers fieldOfClass: HTTPCookieField ifNone: [nil].
	^field isNil ifTrue: [nil] ifFalse: [field valuesAsString]
"! !

!HTTPRequest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
encrypted
	^encrypted! !

!HTTPRequest methodsFor: 'private' stamp: 'mivsek 7/19/2007 22:09'!
ensureFullRead
	"that is, that everything is read from a socket stream. Importanf for HTTPost 
	and defered parsing of postData"! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
environmentAt: aKey 
	^self environmentAt: aKey ifAbsent: [nil]! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
environmentAt: aKey ifAbsent: aBlock 
	^self environmentData at: aKey ifAbsent: aBlock! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
environmentAt: aKey put: aValue 
	self environmentData at: aKey put: aValue! !

!HTTPRequest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
environmentData
	environmentData isNil ifTrue: [self initEnvironmentData].
	^environmentData! !

!HTTPRequest methodsFor: 'initialize-release' stamp: ' 10/7/07 17:21'!
for: aRequestLine readFrom: aSwazooStream 
	"^self
I parse my headers from aStream and update my URI and HTTP version information from aRequest line.  I need to parse the headers first because, for some reason, the URI insists on knowing the host, and this is taken from the Host: header field."

	requestLine := aRequestLine.
	headers := HTTPHeaders readFrom: aSwazooStream.
	self setTimestamp.
	^self! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
hasCookie
	"check if  Cookie:  was in request header"

	"it is GenericHeaderField!!"
	^self headers fields includesKey: 'COOKIE'

"	^self headers includesFieldOfClass: HTTPCookieField"! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
headerAt: aKey ifAbsent: aBlock
	^self headers fieldNamed: aKey  ifNone: aBlock! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
host
	^(self headers fieldOfClass: HTTPHostField ifNone: [^String new]) 
		hostName! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
httpVersion
	^self requestLine httpVersion! !

!HTTPRequest methodsFor: 'accessing-queries' stamp: ' 14/4/07 23:10'!
includesQuery: aString 
	^self uri includesQuery: aString! !

!HTTPRequest methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
initEnvironmentData
	environmentData := Dictionary new! !

!HTTPRequest methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
initRequestLine
	requestLine := HTTPRequestLine new! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
ip
	^ip! !

!HTTPRequest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
ip: anObject
	ip := anObject! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isAuthenticated
	^self authenticated isNil not! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isClose
	| connectionField |
	connectionField := self headers fieldOfClass: HTTPConnectionField
				ifNone: [nil].
	^connectionField notNil and: [connectionField connectionTokenIsClose]! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isDelete
	^false! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isEncrypted
	^self encrypted isNil not! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isFromLinux
	^self userAgent notNil and: ['*Linux*' match: self userAgent]! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isFromMSIE
	^self userAgent notNil and: ['*MSIE*' match: self userAgent]! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isFromNetscape
	"NS>7.0 or Mozilla or Firefox"
	^self userAgent notNil and: ['*Gecko*' match: self userAgent]! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isFromWindows
	^self userAgent notNil and: ['*Windows*' match: self userAgent]! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isGet
	^false! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isHead
	^false! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isKeepAlive
	| header |
	header := self connection.
	header isNil ifTrue: [^false].
	^'*Keep-Alive*' match: self connection! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isOptions
	^false! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isPost
	^false! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isPut
	^false! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isTrace
	^false! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
methodName
	"HTTP method used for a request"
	^self class methodName! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
peer
	^peer! !

!HTTPRequest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
peer: anObject
	peer := anObject! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
port
	"^an Integer
I return the port number to which the request was directed."
	| host |
	host := self headers fieldOfClass: HTTPHostField.
	^(host notNil and: [(self httpVersion at: 2) = 1])
		ifTrue: [host portNumber]
		ifFalse: [self requestLine requestURI port]! !

!HTTPRequest methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
printOn: aStream 
	aStream nextPutAll: 'a HTTPRequest ', self methodName.
	self peer isNil ifFalse: [aStream cr; tab; nextPutAll: ' from: '; nextPutAll: self peer].
	aStream cr; tab; nextPutAll: ' at: '. self timestamp printOn: aStream.
	aStream cr; tab; nextPutAll: ' url: '. self uri printOn: aStream.
	^self! !

!HTTPRequest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
queries
	^self uri queries! !

!HTTPRequest methodsFor: 'accessing-queries' stamp: ' 14/4/07 23:10'!
queryAt: aKey 
	^self uri queryAt: aKey! !

!HTTPRequest methodsFor: 'accessing-queries' stamp: ' 14/4/07 23:10'!
queryAt: aKey ifAbsent: aBlock 
	^self uri queryAt: aKey ifAbsent: aBlock! !

!HTTPRequest methodsFor: 'accessing-queries' stamp: ' 14/4/07 23:10'!
queryData
	^self uri queryData! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
referer
	| field |
	field := self headers fieldOfClass: HTTPRefererField ifNone: [nil].
	^field isNil ifTrue: [nil] ifFalse: [field uri asString]! !

!HTTPRequest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
request: aUriString from: aHostString at: anIPString 
	"For testing only (I'm guessing / hoping!!).
A request is manufactured that has a request line method of >>methodName and a request line URI with an identifier of aUriString.  A Host header is added to the headers and the ip address is set to anIP string.  I also set the HTTP version to #(1 1).
This may result in a corrupt or invalid request, but that's the natutre of testing, I guess."

	requestLine := (HTTPRequestLine new)
				method: self class methodName;
				requestURI: ((SwazooURI new)
							identifier: aUriString;
							yourself);
				httpVersion: #(1 1);
				yourself.
	self headers addField: (HTTPHostField newWithValueFrom: aHostString).
	self ip: anIPString.
	^self! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
requestLine
	"^an HTTPRequestLine"

	requestLine isNil ifTrue: [self initRequestLine].
	^requestLine! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
resolution
	^resolution! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
resolution: anObject
	resolution := anObject! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
resourcePath
	^self resolution resourcePath! !

!HTTPRequest methodsFor: 'services' stamp: ' 14/4/07 23:10'!
respondUsing: responseBlock 
	"^an HTTPResponse
By default, I let aBlock handle creating the response by passing myself as the agrument to the block.  My subclasses may override this method and directly respond.  This is most likely for Unsupported requests and for things like OPTIONS requsts.  c.f. HTTPServer>>answerTo: "

	^responseBlock value: self! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
session
	^self environmentAt: #session! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
session: aSession 
	self environmentAt: #session put: aSession! !

!HTTPRequest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
setAuthenticated 	
	authenticated := true! !

!HTTPRequest methodsFor: 'private' stamp: ' 14/4/07 23:10'!
setEncrypted
	encrypted := true! !

!HTTPRequest methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
setTimestamp
	timestamp := SpTimestamp now! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
tailPath
	^self resolution tailPath! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
timestamp
	^timestamp! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
uri
	^self requestLine requestURI! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
uriString
	^self uri identifier! !

!HTTPRequest methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
urlString
	^self uri value! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
userAgent
	| userAgentField |
	userAgentField := self headers fieldOfClass: HTTPUserAgentField
				ifNone: [nil].
	^userAgentField isNil ifTrue: [nil] ifFalse: [userAgentField productTokens]! !

!HTTPRequest methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
wantsConnectionClose
	self isClose ifTrue: [^true].
	^self httpVersion last = 0 and: [self isKeepAlive not]! !

HTTPRequest subclass: #HTTPTrace
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPTrace commentStamp: '<historical>' prior: 0!
HTTPTrace 

rfc26216 section 9.8

   The TRACE method is used to invoke a remote, application-layer loop-
   back of the request message. The final recipient of the request
   SHOULD reflect the message received back to the client as the
   entity-body of a 200 (OK) response
!

!HTTPTrace class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
methodName
	^'TRACE'! !

!HTTPTrace methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isTrace
	^true! !

HTTPMessage subclass: #HTTPResponse
	instanceVariableNames: 'code entity'
	classVariableNames: 'StatusCodes'
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

HTTPResponse subclass: #FileResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!FileResponse methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
contentType
	^self entity contentType! !

!FileResponse methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
entity: aMimeObject 
	entity := aMimeObject! !

!FileResponse methodsFor: 'private-printing' stamp: ' 14/4/07 23:10'!
printContentLengthOn: aStream 
	self entity isNil 
		ifFalse: 
			[aStream
				nextPutAll: 'Content-Length: ';
				print: self entity value fileSize.
			self crlfOn: aStream]! !

!FileResponse methodsFor: 'private-printing' stamp: ' 14/4/07 23:10'!
printEntityOn: aStream 
	| rs |
	self entity isNil 
		ifFalse: 
			[rs := self entity value readStream.
			rs lineEndTransparent.
			SpExceptionContext 
				for: 
					[[[rs atEnd] whileFalse: [aStream nextPutAll: (rs nextAvailable: 2000)]] 
						ensure: [rs close]]
				on: SpError
				do: [:ex | ex return]]! !

!FileResponse methodsFor: 'private-printing' stamp: ' 14/4/07 23:10'!
printHeadersOn: aStream 
	self contentType: self entity contentType.
	super printHeadersOn: aStream! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
badRequest
	^super new code: 400! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
forbidden
	^super new
		code: 403;
		entity: '<HTML>
<HEAD><TITLE>Forbidden</TITLE></HEAD>
<BODY>
<H1>403 Forbidden</H1>
<P>Access to the requested resource is forbidden.</P>
</BODY></HTML>'! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
found
	^super new code: 302! !

!HTTPResponse class methodsFor: 'class initialization' stamp: ' 14/4/07 23:10'!
initialize
	"self initialize"

	StatusCodes := (Dictionary new)
				add: 100 -> 'Continue';
				add: 101 -> 'Switching Protocols';
				add: 200 -> 'OK';
				add: 201 -> 'Created';
				add: 202 -> 'Accepted';
				add: 203 -> 'Non-Authoritative Information';
				add: 204 -> 'No Content';
				add: 205 -> 'Reset Content';
				add: 206 -> 'Partial Content';
				add: 300 -> 'Multiple Choices';
				add: 301 -> 'Moved Permanently';
				add: 302 -> 'Found';
				add: 303 -> 'See Other';
				add: 304 -> 'Not Modified';
				add: 305 -> 'Use Proxy';
				add: 307 -> 'Temporary Redirect';
				add: 400 -> 'Bad Request';
				add: 401 -> 'Unauthorized';
				add: 402 -> 'Payment Required';
				add: 403 -> 'Forbidden';
				add: 404 -> 'Not Found';
				add: 405 -> 'Method Not Allowed';
				add: 406 -> 'Not Acceptable';
				add: 407 -> 'Proxy Authentication Required';
				add: 408 -> 'Request Time-out';
				add: 409 -> 'Conflict';
				add: 410 -> 'Gone';
				add: 411 -> 'Length Required';
				add: 412 -> 'Precondition Failed';
				add: 413 -> 'Request Entity Too Large';
				add: 414 -> 'Request-URI Too Large';
				add: 415 -> 'Unsupported Media Type';
				add: 416 -> 'Requested range not satisfiable';
				add: 417 -> 'Expectation Failed';
				add: 500 -> 'Internal Server Error';
				add: 501 -> 'Not Implemented';
				add: 502 -> 'Bad Gateway';
				add: 503 -> 'Service Unavailable';
				add: 504 -> 'Gateway Time-out';
				add: 505 -> 'HTTP Version not supported';
				yourself.
	self postInitialize.! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
internalServerError
	^super new
		code: 500;
		entity: '<HTML>
<HEAD><TITLE>Not Found</TITLE></HEAD>
<BODY>
<H1>500 Internal Server Error</H1>
<P>The server experienced an error while processing this request.  If this problem persists, please contact the webmaster.</P>
</BODY></HTML>'! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
methodNotAllowed
"c.f. RFC 2616  10.4.6
   The method specified in the Request-Line is not allowed for the
   resource identified by the Request-URI. The response MUST include an
   Allow header containing a list of valid methods for the requested
   resource. "
	^super new code: 405! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
movedPermanently
	^super new code: 301! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
notFound
	^super new
		code: 404;
		entity: '<HTML>
<HEAD><TITLE>Not Found</TITLE></HEAD>
<BODY>
<H1>404 Not Found</H1>
<P>The requested resource was not found on this server.</P>
</BODY></HTML>'! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
notImplemented
	^super new code: 501! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
notModified
	^super new code: 304! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
ok
	^super new code: 200! !

!HTTPResponse class methodsFor: 'class initialization' stamp: ' 14/4/07 23:10'!
postInitialize
	"extend it with your own codes"! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
redirectLink
	"^an HTTPResponse
Note that 302 is really the 'found' response.  This code should really be 303 (>>seeOther).  However, because many clients take 302 & 303 to be the same and because older clients don't understand 303, 302 is commonly used in this case.  See RFC 2616 10.3.4."

	^super new code: 302! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
seeOther
	"^an HTTPResponse
The response to the request can be found under a different URI and SHOULD be retrieved using a GET method on that resource. This method exists primarily to allow the output of a POST-activated script to redirect the user agent to a selected resource.
See RFC 2616 10.3.4."

	^super new code: 303! !

!HTTPResponse class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
statusTextForCode: aNumber
	^StatusCodes at: aNumber ifAbsent: ['']! !

!HTTPResponse class methodsFor: 'response types' stamp: ' 14/4/07 23:10'!
unauthorized
	^super new code: 401! !

!HTTPResponse methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
addDateHeader
	"^self
Note that the server must have it's clock set to GMT"

	self headers addField: (HTTPDateField new date: SpTimestamp now).
	^self! !

!HTTPResponse methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
addDefaultBody
	self entity: '<HTML>
<HEAD><TITLE>', (StatusCodes at: self code ifAbsent: [self code printString]), '</TITLE></HEAD>
  <BODY>
   <H2>', self code printString, ' ', (StatusCodes at: self code ifAbsent: [self code printString]), '</H2>
   <P>The server experienced an error while processing this request. <BR>
   If this problem persists, please contact the webmaster.</P>
  <P>Swazoo Smalltalk Web Server</P>
  </BODY>
</HTML>'! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
addHeaderName: aNameString value: aValueString
	^self headers 
		addField: (GenericHeaderField newForFieldName: aNameString withValueFrom: aValueString)! !

!HTTPResponse methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
addInitialHeaders
	self addServerHeader.
	self addDateHeader! !

!HTTPResponse methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
addServerHeader
	^self headers 
		addField: (HTTPServerField new productTokens: HTTPServer version)! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
cacheControl: aString
	"example: 'no-store, no-cache, must-revalidate'"
	self headers addField: (HTTPCacheControlField new directives: aString).! !

!HTTPResponse methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
code
	^code! !

!HTTPResponse methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
code: anInteger 
	code := anInteger.
	(#(200) includes: code) ifFalse: [self addDefaultBody].! !

!HTTPResponse methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
codeText
	^self class statusTextForCode: self code! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
contentType
	"^a String
Return the media type from my Content-Type header field."

	^self headers 
		fieldOfClass: ContentTypeField
		ifPresent: [:field | field mediaType]
		ifAbsent: ['application/octet-stream']! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
contentType: aString 
	self headers addField: (ContentTypeField new mediaType: aString).
	^self! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
cookie: aString 
	| newField |
	newField := HTTPSetCookieField new.
	newField addCookie: aString.
	self headers addField: newField.
	^self! !

!HTTPResponse methodsFor: 'private-sending' stamp: ' 14/4/07 23:10'!
crlfOn: aStream 
	aStream
		nextPut: Character cr;
		nextPut: Character lf! !

!HTTPResponse methodsFor: 'private-sending' stamp: ' 14/4/07 23:10'!
endHeaderOn: aStream 
	self crlfOn: aStream! !

!HTTPResponse methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
entity
	^entity! !

!HTTPResponse methodsFor: 'accessing' stamp: 'mivsek 7/22/2007 19:17'!
entity: anEntity 
	entity := anEntity asByteArray "if not already"! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
expires: aSpTimestamp  "from SPort "
	self headers addField: (HTTPExpiresField new timestamp: aSpTimestamp ).
	^self! !

!HTTPResponse methodsFor: 'private' stamp: ' 14/4/07 23:10'!
informConnectionClose
	self headers 
		fieldOfClass: HTTPConnectionField
		ifPresent: [:field | field setToClose]
		ifAbsent: [self headers addField: HTTPConnectionField new setToClose].
	^self! !

!HTTPResponse methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isBadRequest
	^self code = 400! !

!HTTPResponse methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isFound
	^self code = 302! !

!HTTPResponse methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isInternalServerError
	^self code = 500! !

!HTTPResponse methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isMovedPermanently
	^self code = 301! !

!HTTPResponse methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isNotFound
	^self code = 404! !

!HTTPResponse methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isNotImplemented
	^self code = 501! !

!HTTPResponse methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isNotModified
	^self code = 304! !

!HTTPResponse methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isOk
	^self code = 200! !

!HTTPResponse methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isRedirectLink
	^self code = 302! !

!HTTPResponse methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isSeeOther
	^self code = 303! !

!HTTPResponse methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isUnauthorized
	^self code = 401! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
lastModified: aSpTimestamp "from SPort "
	self headers addField: (HTTPLastModifiedField new timestamp: aSpTimestamp).
	^self! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: ' 14/4/07 23:10'!
location: aString 
	self headers addField: (HTTPLocationField new uriString: aString).
	^self! !

!HTTPResponse methodsFor: 'private-sending' stamp: ' 14/4/07 23:10'!
printContentLengthOn: aStream 
	aStream
		nextPutAll: 'Content-Length: ';
		print: (self entity notNil ifTrue: [self entity size] ifFalse: [0]).
	self crlfOn: aStream! !

!HTTPResponse methodsFor: 'private-sending' stamp: ' 14/4/07 23:10'!
printEntityOn: aStream 
	self entity isNil ifFalse: [aStream nextPutBytes: self entity]! !

!HTTPResponse methodsFor: 'private-sending' stamp: 'mivsek 7/22/2007 19:18'!
printHeadersOn: aSwazooStream 
	"^self
Write the headers (key-value pairs) to aStream.  The key
must be a String."

	self headers fields do: 
			[:aField | 
			aField printOn: aSwazooStream.
			self crlfOn: aSwazooStream]! !

!HTTPResponse methodsFor: 'private-sending' stamp: ' 10/7/07 17:21'!
printStatusOn: aSwazooStream 
	StatusCodes at: self code ifAbsent: [self class initialize]. "if some new status codes was added           													with #postInitialize method later"
	aSwazooStream
		nextPutAll: 'HTTP/1.1 ';
		print: self code;
		space;
		nextPutAll: (self class statusTextForCode: self code).
	self crlfOn: aSwazooStream! !

!HTTPResponse methodsFor: 'private-sending' stamp: ' 10/7/07 17:21'!
writeHeaderTo: aSwazooStream 
	self printStatusOn: aSwazooStream.
	self printHeadersOn: aSwazooStream.
	self printContentLengthOn: aSwazooStream.
	self endHeaderOn: aSwazooStream! !

!HTTPResponse methodsFor: 'sending' stamp: ' 10/7/07 17:21'!
writeTo: aSwazooStream 
	self writeHeaderTo: aSwazooStream.
	self printEntityOn: aSwazooStream! !

Object subclass: #HTTPPostDataArray
	instanceVariableNames: 'underlyingCollection stream parsed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPPostDataArray commentStamp: '<historical>' prior: 0!
Introduced the HTTPPostDataArray to hold post data in an HTTPRequest in place of a Dictionary.  This is because it is legal for there to be more than one entry with the same name (key) and using a Dictionary  looses data (!!).

Instance Variables:
	underlyingCollection	<>	

!

!HTTPPostDataArray class methodsFor: 'instance creation' stamp: 'mivsek 7/19/2007 22:09'!
newOn: aSwazooStream
	^super new 
		initialize;
		stream: aSwazooStream! !

!HTTPPostDataArray methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
allAt: aKey 
	| candidates |
	candidates := self underlyingCollection 
				select: [:anAssociation | anAssociation key = aKey].
	^candidates collect: [:anAssociation| anAssociation value]! !

!HTTPPostDataArray methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
allNamesForValue: aString 
	| candidates |
	candidates := self underlyingCollection 
				select: [:anAssociation | anAssociation value value = aString].
	^candidates collect: [:anAssociation| anAssociation key]! !

!HTTPPostDataArray methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
associations
	^self underlyingCollection! !

!HTTPPostDataArray methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
at: aKey 
	^(self allAt: aKey) last! !

!HTTPPostDataArray methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
at: aKey ifAbsent: aBlock 
	| candidates |
	candidates := self underlyingCollection 
				select: [:anAssociation | anAssociation key = aKey].
	^candidates isEmpty ifTrue: [aBlock value] ifFalse: [candidates last value]! !

!HTTPPostDataArray methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
at: key put: anObject 
	self underlyingCollection add: (Association key: key value: anObject).
	^anObject! !

!HTTPPostDataArray methodsFor: 'accessing' stamp: 'mivsek 7/19/2007 22:07'!
clearParsed
	parsed := false! !

!HTTPPostDataArray methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
includesKey: aKey 
	| candidates |
	candidates := self underlyingCollection 
				select: [:anAssociation | anAssociation key = aKey].
	^candidates notEmpty! !

!HTTPPostDataArray methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
includesValue: aString 
	| candidates |
	candidates := self underlyingCollection 
				select: [:anAssociation | anAssociation value value = aString].
	^candidates notEmpty! !

!HTTPPostDataArray methodsFor: 'initialize-release' stamp: 'mivsek 7/19/2007 22:07'!
initialize
	self clearParsed! !

!HTTPPostDataArray methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isEmpty
	^self underlyingCollection isEmpty! !

!HTTPPostDataArray methodsFor: 'testing' stamp: 'mivsek 7/19/2007 22:08'!
isParsed
	"postdata is already read and parsed from a request"
	^parsed! !

!HTTPPostDataArray methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
keys
"^a Set
I mimick the behavior of a Dictionay which I replace.  I return a set of the keys in my underlying collection of associations."

	^(self underlyingCollection collect: [:anAssociation| anAssociation key]) asSet! !

!HTTPPostDataArray methodsFor: 'enumerating' stamp: ' 14/4/07 23:10'!
keysAndValuesDo: aTwoArgumentBlock 
	self underlyingCollection do: 
		[:anAssociation | aTwoArgumentBlock value: anAssociation key value: anAssociation value]! !

!HTTPPostDataArray methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
nameForValue: aString
	^(self allNamesForValue: aString) last! !

!HTTPPostDataArray methodsFor: 'private' stamp: ' 14/4/07 23:10'!
printOn: aStream
	aStream nextPutAll: 'a Swazoo.HttpPostDataArray 
	'.
	self underlyingCollection do: [:each | aStream nextPutAll: each key printString , '->', 
		each value value printString, '
	'].! !

!HTTPPostDataArray methodsFor: 'enumerating' stamp: ' 14/4/07 23:10'!
select: aBlock 
"^an Object
I run the select on the values of the associations in my underlying collection.  This mimicks the behavior when a Dictionary was used in my place."
	^self underlyingCollection select:  [:anAssociation| aBlock value: anAssociation value]! !

!HTTPPostDataArray methodsFor: 'accessing' stamp: 'mivsek 7/19/2007 22:08'!
setParsed
	parsed := true! !

!HTTPPostDataArray methodsFor: 'private' stamp: 'mivsek 7/19/2007 22:08'!
stream
	^stream! !

!HTTPPostDataArray methodsFor: 'private' stamp: 'mivsek 7/19/2007 22:08'!
stream: aSwazooStream
	"needed for defered postData parsing"
	stream := aSwazooStream! !

!HTTPPostDataArray methodsFor: 'private' stamp: ' 14/4/07 23:10'!
underlyingCollection
	underlyingCollection isNil 
		ifTrue: [underlyingCollection := OrderedCollection new].
	^underlyingCollection! !

Object subclass: #HTTPRequestLine
	instanceVariableNames: 'method requestURI httpVersion'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPRequestLine class methodsFor: 'instance creation' stamp: ' 14/4/07 23:10'!
readFrom: aStream 
	"^an HTTPRequestLine
I return a new instance of myself which represents a request line read from aStream.  If no valid request line can be found, I throw an exception."

	^self new readFrom: aStream! !

!HTTPRequestLine methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
httpVersion
	^httpVersion! !

!HTTPRequestLine methodsFor: 'private' stamp: ' 14/4/07 23:10'!
httpVersion: anArray 
	httpVersion := anArray.
	^self! !

!HTTPRequestLine methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
method
	^method! !

!HTTPRequestLine methodsFor: 'private' stamp: ' 14/4/07 23:10'!
method: aString
"For development testing only"
	method := aString.
	^self! !

!HTTPRequestLine methodsFor: 'parsing' stamp: 'mivsek 7/26/2007 13:07'!
parseHTTPVersionFrom: aSwazooStream 
	| major minor |
	self skipSpacesIn: aSwazooStream.
	aSwazooStream upTo: $/ asInteger.
	major := (aSwazooStream upTo: $. asInteger) asString asNumber.
	minor := (aSwazooStream upTo: Character cr asInteger) asString asNumber.
	self httpVersion: (Array with: major with: minor).
	aSwazooStream next.
	^self! !

!HTTPRequestLine methodsFor: 'parsing' stamp: 'mivsek 7/26/2007 13:06'!
parseURIFrom: aSwazooStream 
	"^self
	Really, we should parse the URI directly out of the stream."

	self skipSpacesIn: aSwazooStream.
	requestURI := SwazooURI fromString: 
		(aSwazooStream upTo: Character space asInteger) asString.
	^self! !

!HTTPRequestLine methodsFor: 'parsing' stamp: 'mivsek 7/10/2007 18:52'!
readFrom: aSwazooStream 
	"^self
I initialize myself to represents a request line read from aStream.  If no valid request line can be found, I throw an exception."

	self skipLeadingBlankLinesIn: aSwazooStream.
	method := (aSwazooStream upTo: Character space asInteger) asString.
	self parseURIFrom: aSwazooStream.
	self parseHTTPVersionFrom: aSwazooStream.
	^self! !

!HTTPRequestLine methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
requestURI
	^requestURI! !

!HTTPRequestLine methodsFor: 'private' stamp: ' 14/4/07 23:10'!
requestURI: aString 
	"Development testing only!!"

	requestURI := aString.
	^self! !

!HTTPRequestLine methodsFor: 'private' stamp: ' 10/7/07 17:21'!
skipLeadingBlankLinesIn: aSwazooStream 
	"^self
RFC 2616:
In the interest of robustness, servers SHOULD ignore any empty
line(s) received where a Request-Line is expected. In other words, if
the server is reading the protocol stream at the beginning of a
message and receives a CRLF first, it should ignore the CRLF."

	[aSwazooStream peek == Character cr asInteger] whileTrue: 
			[(((aSwazooStream next: 2) at: 2) == Character lf asInteger) 
				ifFalse: [SwazooHTTPParseError raiseSignal: 'CR without LF']].
	^self! !

!HTTPRequestLine methodsFor: 'private' stamp: 'mivsek 7/26/2007 13:27'!
skipSpacesIn: aSwazooStream 
	[aSwazooStream peek = Character space] 
		whileTrue: [aSwazooStream next].
	^self! !

Object subclass: #HTTPServer
	instanceVariableNames: 'ip port connections sites socket loop isMultiThreading'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!HTTPServer class methodsFor: 'intialize-release' stamp: ' 14/4/07 23:09'!
initialize
	SpEnvironment addImageShutdownTask: [self shutDown] for: self! !

!HTTPServer class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
new
	^super new initialize! !

!HTTPServer class methodsFor: 'intialize-release' stamp: ' 14/4/07 23:09'!
shutDown 
	"HTTPServer shutDown"
	self allInstances do: [:each | each stop].
	SpEnvironment removeShutdownActionFor: self.! !

!HTTPServer class methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
version
	^'Swazoo 2.0 Smalltalk Web Server'! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
acceptConnection
	"^self
I accept the next inbound TCP/IP connection.  The operating system libraries queue these up for me, so I can just handle one at a time.  I create an HTTPConnection instance to actually handle the interaction with the client - if I am in single threaded mode, the connection will completely handle the request before returning control to me, but in multi-threaded mode the connection forks the work into a sepparate thread in this image and control is immediately returned to me (the application programmer must worry about thread safety in this case."

	| clientConnection |
	clientConnection := SpExceptionContext 
				for: [HTTPConnection socket: self socket accept]
				on: SpError
				do: [:ex | Transcript show: 'Socket accept error: ' , ex errorString; cr. ^self].
	self addConnection: clientConnection.
	clientConnection interact.
	^self! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
addConnection: aConnection 
	self connections add: aConnection.
	aConnection server: self! !

!HTTPServer methodsFor: 'sites' stamp: ' 14/4/07 23:09'!
addSite: aSite
	(self sites includesResource: aSite) ifFalse: [^self sites addResource: aSite]! !

!HTTPServer methodsFor: 'serving' stamp: ' 14/4/07 23:09'!
answerTo: aRequest 
	| response |
	response := URIResolution resolveRequest: aRequest startingAt: self sites.
	^response isNil
		ifTrue: [HTTPResponse notFound]
		ifFalse: [response]! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
connections
	connections isNil ifTrue: [self initConnections].
	^connections! !

!HTTPServer methodsFor: 'sites' stamp: ' 14/4/07 23:09'!
hasNoSites
	^self sites hasNoResources! !

!HTTPServer methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
initConnections
	connections := OrderedCollection new.! !

!HTTPServer methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
initSites
	sites := ServerRootComposite new! !

!HTTPServer methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
initialize
	self initConnections.
	self initSites! !

!HTTPServer methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
ip
	^ip! !

!HTTPServer methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
ip: anIPString 
	ip := anIPString! !

!HTTPServer methodsFor: 'multithreading' stamp: ' 14/4/07 23:09'!
isMultiThreading
	"^a Boolean
I return true if each inbound HTTP connection will be handled in its own thread.  See the senders of this message to see where that is important.  Note that the default mode is mult-threaded because this is how Swazoo has worked so far.  This is tricky for the application programmer, though, as they must ensure that they work in a thread safe way (e.g. avoid the many threads updating the same object).  For those deploying to GemStone, you wil find things much easier if you do *not* run multithreaded, but rather run many gems each with a single-threaded Swazoo instance (and your app logic) in each.  Also in GemStone, run the main loop in the foreground, c.f. >>mainLoopInForeground"

	isMultiThreading isNil ifTrue: [self setMultiThreading].
	^isMultiThreading! !

!HTTPServer methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
isServing
	^self loop notNil! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
loop
	^loop! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
loop: aProcess
	loop := aProcess! !

!HTTPServer methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
port
	^port! !

!HTTPServer methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
port: aNumber
	port := aNumber! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
removeConnection: aConnection 
	self connections remove: aConnection ifAbsent: [nil]! !

!HTTPServer methodsFor: 'sites' stamp: ' 14/4/07 23:09'!
removeSite: aSite 
	^self sites removeResource: aSite! !

!HTTPServer methodsFor: 'multithreading' stamp: ' 14/4/07 23:09'!
setMultiThreading
	"^self
I record that this HTTP server is to operate in a multi-threaded mode.  c.f. isMultiThreading"

	isMultiThreading := true.
	^self! !

!HTTPServer methodsFor: 'multithreading' stamp: ' 14/4/07 23:09'!
setSingleThreading
	"^self
I record that this HTTP server is to operate in a single-threaded mode.  c.f. isMultiThreading"

	isMultiThreading := false.
	^self! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
sites
	sites isNil ifTrue: [self initSites].
	^sites! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
socket
	^socket! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
socket: aSocket
	socket := aSocket! !

!HTTPServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
socketClass
	"^a Class
I use SwazooSocket to wrap the actual socket.  SwazooSocket does some of the byte translation work for me."

	^SwazooSocket! !

!HTTPServer methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
start
	self loop isNil 
		ifTrue: 
			[self socket: (self socketClass serverOnIP: self ip port: self port).
			self socket listenFor: 50.
			self loop: ([[self acceptConnection] repeat] 
						forkAt: Processor userBackgroundPriority)]! !

!HTTPServer methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
stop
	self loop isNil 
		ifFalse: 
			[self connections copy do: [:each | each close].
			self loop terminate.
			self loop: nil.
			self socket close.
			self socket: nil]! !

Object subclass: #HTTPString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Compatibility'!

!HTTPString commentStamp: '<historical>' prior: 0!
This class contains some utility methods that were previously implemented as extentions to system classes.  This is really a stop-gap until, perhaps, the SwazooStream yeilds HTTPStrings.

!

!HTTPString class methodsFor: 'decoding' stamp: ' 14/4/07 23:09'!
decodedHTTPFrom: aCharacterArray 
	"Code taken from the swazoo specific extention to the CharacterArray class"

	| targetStream sourceStream |
	targetStream := WriteStream on: aCharacterArray class new.
	sourceStream := ReadStream on: aCharacterArray.
	[sourceStream atEnd] whileFalse: 
			[| char |
			char := sourceStream next.
			char = $% 
				ifTrue: 
					[targetStream 
						nextPut: (SpEnvironment integerFromString: '16r' , (sourceStream next: 2)) 
								asCharacter]
				ifFalse: 
					[char == $+ 
						ifTrue: [targetStream nextPut: Character space]
						ifFalse: [targetStream nextPut: char]]].
	^targetStream contents! !

!HTTPString class methodsFor: 'decoding' stamp: 'mivsek 6/13/2007 14:06'!
encodedHTTPFrom: aCharacterArray 
	"Code taken from the swazoo specific extention to the CharacterArray class"

	| targetStream |
	targetStream := WriteStream on: aCharacterArray class new.
	aCharacterArray do: 
			[:char | 
			(self isHTTPReservedCharacter: char) 
				ifTrue: 
					[targetStream nextPut: $%.
					targetStream nextPutAll:
						(char asInteger printPaddedWith: $0 to: 2 base: 16)
"					char asInteger 
						printOn: targetStream
						paddedWith: $0
						to: 2
						base: 16" ]
				ifFalse: [targetStream nextPut: char]].
	^targetStream contents! !

!HTTPString class methodsFor: 'decoding' stamp: ' 14/4/07 23:09'!
isHTTPReservedCharacter: aCharacter 
	"Code taken from the swazoo specific extention to the Character class"

	^(aCharacter isAlphaNumeric or: ['-_.!!~*''()' includes: aCharacter]) not! !

!HTTPString class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
newRandomString: anInteger 
	| numbersThroughAlphas targetStream char random |
	numbersThroughAlphas := (48 to: 122) collect: [:each | each asCharacter].
	targetStream := WriteStream on: (String new: anInteger).
	random := Random new.
	[targetStream contents size < anInteger] whileTrue: 
			[char := numbersThroughAlphas 
						at: (random next * (numbersThroughAlphas size - 1)) rounded + 1.
			char isAlphaNumeric ifTrue: [targetStream nextPut: char]].
	^targetStream contents! !

!HTTPString class methodsFor: 'decoding' stamp: ' 14/4/07 23:09'!
stringFromBytes: aByteArray 
	"^a String
In GemStone ['Hello, World' asByteArray asString] returns the string 'aByteArray' !!
This is the boring long way of getting a string from a ByteArray - but it does work
in GemStone."

	"HTTPString stringFromBytes: ('Hello, World' asByteArray)"

	| targetStream |
	targetStream := WriteStream on: String new.
	aByteArray do: [:aByte | targetStream nextPut: aByte asCharacter].
	^targetStream contents! !

!HTTPString class methodsFor: 'tokens' stamp: ' 14/4/07 23:09'!
subCollectionsFrom: aCollection delimitedBy: anObject 
	"^an OrderedCollection
I return the ordered collection of sub-collections from aCollection, delimited
by anObject."

	"HTTPString subCollectionsFrom: 'aaa/bbb/' delimitedBy: $/"

	| subCollections sourceStream |
	subCollections := OrderedCollection new.
	sourceStream := ReadStream on: aCollection.
	[sourceStream atEnd] 
		whileFalse: [subCollections add: (sourceStream upTo: anObject)].
	(aCollection isEmpty 
		or: [(sourceStream
				skip: -1;
				next) == anObject]) 
			ifTrue: [subCollections add: aCollection class new].
	^subCollections! !

!HTTPString class methodsFor: 'decoding' stamp: ' 14/4/07 23:09'!
trimBlanksFrom: aString 
	"^a String
I return a copy of aString with all leading and trailing blanks removed."

	| first last |
	first := 1.
	last := aString size.
	[last > 0 and: [(aString at: last) isSeparator]] 
		whileTrue: [last := last - 1].
	^last == 0 
		ifTrue: [String new]
		ifFalse: 
			[[first < last and: [(aString at: first) isSeparator]] 
				whileTrue: [first := first + 1].
			aString copyFrom: first to: last]! !

Object subclass: #HeaderField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

HeaderField class
	instanceVariableNames: 'httpFieldNameToClassDictionary'!

HeaderField class
	instanceVariableNames: 'httpFieldNameToClassDictionary'!

HeaderField subclass: #GenericHeaderField
	instanceVariableNames: 'name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!GenericHeaderField class methodsFor: 'instance creation' stamp: ' 14/4/07 23:10'!
newForFieldName: fieldNameString withValueFrom: fieldValueString 
	^self new forFieldName: fieldNameString andValue: fieldValueString! !

!GenericHeaderField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
combineWith: aHeaderField 
	"^self
I simply take my values and concatenate the values of aHeaderField."

	value := self value , ', ' , aHeaderField value.
	^self! !

!GenericHeaderField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
1 halt: 'use >>name instead'.
	^self name! !

!GenericHeaderField methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
forFieldName: fieldNameString andValue: fieldValueString 
	name := fieldNameString.
	value := fieldValueString.
	^self! !

!GenericHeaderField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
name
	^name! !

!GenericHeaderField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
value
	^value! !

!GenericHeaderField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
values
	^(HTTPString subCollectionsFrom: self value delimitedBy: $,) 
		collect: [:each | HTTPString trimBlanksFrom: each]! !

!GenericHeaderField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream nextPutAll: value.
	^self! !

!HeaderField class methodsFor: 'private' stamp: ' 14/4/07 23:10'!
classForFieldName: aString 
	"^a Class
If I can find a specific header field with a name matching aString I return that.  Otherwise I return the GenericHeaderField class."

	^self httpFieldNameToClassDictionary at: aString
		ifAbsent: [GenericHeaderField]! !

!HeaderField class methodsFor: 'instance creation' stamp: ' 14/4/07 23:10'!
fromLine: aString 
	| sourceStream fieldName fieldValue fieldClass |
	sourceStream := ReadStream on: aString.
	fieldName := (HTTPString trimBlanksFrom: (sourceStream upTo: $:)) 
				asUppercase.
	fieldClass := self classForFieldName: fieldName.
	fieldValue := HTTPString trimBlanksFrom: sourceStream upToEnd.
	^fieldClass newForFieldName: fieldName withValueFrom: fieldValue! !

!HeaderField class methodsFor: 'private' stamp: ' 14/4/07 23:10'!
httpFieldNameToClassDictionary
	"^a Class
I return the dictionarry of my subclasses keyed on the name of the field they represent.
Note that we only need *Request* headers listed in here because they are the only thing we will be parsing for."

	"After a change here, remeber to do 'HeaderField resetHttpFieldNameToClassDictionary' "

	httpFieldNameToClassDictionary isNil 
		ifTrue: 
			[| headerClasses |
			headerClasses := OrderedCollection new.
			headerClasses
				add: ContentDispositionField;
				add: HTTPContentLengthField;
				add: ContentTypeField;
				add: HTTPAcceptField;
				add: HTTPAuthorizationField;
				add: HTTPConnectionField;
				add: HTTPHostField;
				add: HTTPIfMatchField;
				add: HTTPIfModifiedSinceField;
				add: HTTPIfNoneMatchField;
				add: HTTPIfRangeField;
				add: HTTPIfUnmodifiedSinceField;
				add: HTTPRefererField;
				add: HTTPUserAgentField.
			httpFieldNameToClassDictionary := Dictionary new.
			headerClasses do: 
					[:aClass | 
					httpFieldNameToClassDictionary at: aClass fieldName asUppercase put: aClass]].
	^httpFieldNameToClassDictionary! !

!HeaderField class methodsFor: 'private' stamp: ' 14/4/07 23:10'!
newForFieldName: fieldNameString withValueFrom: fieldValueString 
	^self subclassResponsibility! !

!HeaderField class methodsFor: 'private' stamp: ' 14/4/07 23:10'!
resetHttpFieldNameToClassDictionary
	
	httpFieldNameToClassDictionary := nil .
	^self! !

!HeaderField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
combineWith: aHeaderField 
	SwazooHeaderFieldParseError raiseSignal: 'Not supported'! !

!HeaderField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^self subclassResponsibility! !

!HeaderField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isConditional
	^false! !

!HeaderField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isContentDisposition
	^false! !

!HeaderField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isContentType
	^false! !

!HeaderField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
name
	^self subclassResponsibility! !

!HeaderField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
printOn: aStream 
	aStream
		nextPutAll: self name;
		nextPutAll: ': '.
	self valuesAsStringOn: aStream.
	^self! !

!HeaderField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
values
	^self subclassResponsibility! !

!HeaderField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsString
	| targetStream |
	targetStream := WriteStream on: String new.
	self valuesAsStringOn: targetStream.
	^targetStream contents! !

!HeaderField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream
	^self subclassResponsibility! !

HeaderField subclass: #SpecificHeaderField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

SpecificHeaderField subclass: #ContentDispositionField
	instanceVariableNames: 'type parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!ContentDispositionField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Content-Disposition'! !

!ContentDispositionField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isContentDisposition
	^true! !

!ContentDispositionField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
parameterAt: aString 
	^parameters at: aString ifAbsent: [nil]! !

!ContentDispositionField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
parseValueFrom: aString 
	| sourceStream |
	sourceStream := aString readStream.
	type := HTTPString trimBlanksFrom: (sourceStream upTo: $;).
	parameters := self readParametersFrom: sourceStream.
	^self! !

SpecificHeaderField subclass: #ContentTypeField
	instanceVariableNames: 'mediaType transferCodings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!ContentTypeField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Content-Type'! !

!ContentTypeField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
defaultMediaType
	"^a String
See RFC 2616 '7.2.1 Type'.  If no media type is specified, application/octet-stream is the default."

	^'application/octet-stream'! !

!ContentTypeField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isContentType
	^true! !

!ContentTypeField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
mediaType
	^mediaType isNil ifTrue: [self defaultMediaType] ifFalse: [mediaType]! !

!ContentTypeField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
mediaType: aString 
	mediaType := aString.
	^self! !

!ContentTypeField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
parseValueFrom: aString 
	| sourceStream |
	sourceStream := aString readStream.
	mediaType := (HTTPString trimBlanksFrom: (sourceStream upTo: $;)).
	transferCodings := self readParametersFrom: sourceStream.
	^self! !

!ContentTypeField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
transferCodings
	transferCodings isNil ifTrue: [transferCodings := String new].
	^transferCodings! !

!ContentTypeField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream nextPutAll: self mediaType.
	self transferCodings isEmpty 
		ifFalse: 
			[aStream
				nextPut: Character space;
				nextPutAll: self transferCodings].
	^self! !

SpecificHeaderField subclass: #HTTPAcceptField
	instanceVariableNames: 'mediaTypes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPAcceptField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Accept'! !

!HTTPAcceptField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
combineWith: aHeaderField 
	"^self
I simply take my values and concatenate the values of aHeaderField."

	self mediaTypes addAll: aHeaderField mediaTypes.
	^self! !

!HTTPAcceptField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
mediaTypes
	mediaTypes isNil ifTrue: [mediaTypes := OrderedCollection new].
	^mediaTypes! !

!HTTPAcceptField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
parseValueFrom: aString 
	mediaTypes := HTTPString subCollectionsFrom: aString delimitedBy: $,.
	^self! !

!HTTPAcceptField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: targetStream 
	self mediaTypes isEmpty 
		ifFalse: 
			[targetStream nextPutAll: self mediaTypes first.
			2 to: self mediaTypes size
				do: 
					[:methodIndex | 
					targetStream
						nextPut: $,;
						nextPutAll: (self mediaTypes at: methodIndex)]].
	^self! !

SpecificHeaderField subclass: #HTTPAllowField
	instanceVariableNames: 'methods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPAllowField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Allow'! !

!HTTPAllowField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
methods
	methods isNil ifTrue: [methods := OrderedCollection new].
	^methods! !

!HTTPAllowField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: targetStream 
	self methods isEmpty 
		ifFalse: 
			[targetStream nextPutAll: self methods first.
			2 to: self methods size
				do: 
					[:methodIndex | 
					targetStream
						nextPut: $,;
						nextPutAll: (self methods at: methodIndex)]].
	^self! !

SpecificHeaderField subclass: #HTTPAuthorizationField
	instanceVariableNames: 'credentials'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

HTTPAuthorizationField subclass: #HTTPAuthorizationBasicField
	instanceVariableNames: 'userid password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPAuthorizationBasicField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
password
	"^a String
I return the password string (as defined in RFC 2617 pp.2) part of the user-pass value in my credentials."

	password isNil ifTrue: [self resolveUserPass].
	^password! !

!HTTPAuthorizationBasicField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
resolveUserPass
	"^self
I look at my credentials string and pull out the userid and password.  Note that having to check for atEnd before the upToEnd is for GemStone which crashes if upToEnd is used when already atEnd."

	"(Base64EncodingReadStream on: 'YnJ1Y2U6c3F1aWRzdXBwbGllZHBhc3N3b3Jk' ) upToEnd asString "

	| userPassString sourceStream |
	userPassString := HTTPString 
				stringFromBytes: (Base64EncodingReadStream on: self credentials) upToEnd.
	sourceStream := ReadStream on: userPassString.
	userid := sourceStream upTo: $:.
	password := sourceStream atEnd 
				ifTrue: [String new]
				ifFalse: [sourceStream upToEnd].
	^self! !

!HTTPAuthorizationBasicField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
userid
	"^a String
I return the userid string (as defined in RFC 2617 pp.2) part of the user-pass value in my credentials."

	userid isNil ifTrue: [self resolveUserPass].
	^userid! !

!HTTPAuthorizationBasicField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream nextPutAll: 'Basic '.
	super valuesAsStringOn: aStream.
	^self! !

HTTPAuthorizationField subclass: #HTTPAuthorizationDigestField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPAuthorizationDigestField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream nextPutAll: 'Digest '.
	super valuesAsStringOn: aStream.
	^self! !

!HTTPAuthorizationField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Authorization'! !

!HTTPAuthorizationField class methodsFor: 'private' stamp: ' 14/4/07 23:10'!
newForFieldName: fieldNameString withValueFrom: fieldValueString 
	"^an HTTPAuthorizationField
I return an instance of one of my concrete subclasses.  To get to this point, the field name *must* be 'AUTHORIZATION'."

	| sourceStream schemeName |
	sourceStream := ReadStream on: fieldValueString.
	schemeName := sourceStream upTo: Character space.
	^schemeName = 'Basic' 
		ifTrue: [HTTPAuthorizationBasicField newWithValueFrom: sourceStream upToEnd]
		ifFalse: [HTTPAuthorizationDigestField newWithValueFrom: sourceStream upToEnd]! !

!HTTPAuthorizationField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
credentials
	^credentials! !

!HTTPAuthorizationField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
parseValueFrom: aString 
	credentials := HTTPString trimBlanksFrom: aString.
	^self! !

!HTTPAuthorizationField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream nextPutAll: self credentials.
	^self! !

SpecificHeaderField subclass: #HTTPCacheControlField
	instanceVariableNames: 'directives private maxAge noStore noCache mustRevalidate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPCacheControlField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Cache-Control'! !

!HTTPCacheControlField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
directives
	"for easy setting directives in one string"
	^directives! !

!HTTPCacheControlField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
directives: aString 
	"for easy setting directives in one string"
	"example: 'no-store, no-cache, must-revalidate'"
	directives := aString! !

!HTTPCacheControlField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
maxAge
	"^an Integer or nil
I return my max age which is either an integer number of seconds for which the entity can be considdered fresh, or nil, in which case other headers such as Expires can be used by a cache to determine the expiration time of the entity."

	^maxAge! !

!HTTPCacheControlField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
maxAge: anIntegerOrNil 
	"^self
I record the number of seconds for which the resource is 'fresh' and after which will expire and become 'stale' for caching purposes.  Setting this to nil means the max age is unspecified, and this is the default.  This directive takes presidence over any Expires header when a cache or client is handling an HTTP message."

	maxAge := anIntegerOrNil.
	^self! !

!HTTPCacheControlField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
private
	"^a Boolean or nil
There are three possible values for private.  Explicity true (the entity can only be cached in private caches), explicity false (this is a public entity and can be held in a shared/public cache perhaps even when stale) or nil (the default which means that the entity may be held in a public shared cache, but only until it goes stale)."

	^private! !

!HTTPCacheControlField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
setNotPublicOrPrivate
	"^self
I am being told that the entity in my message is not explicity public or private.  This is the default and means that public caches may retain copies of the resource, but should not be as relaxed about the rules as with an explicitly public resource. c.f >>setPublic & >>setPrivate."

	private := nil.
	^self! !

!HTTPCacheControlField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
setPrivate
	"^self
I am being told that the entity in my message is a private one that can only be cached on private caches, i.e. caches that can be drawn upon a single clients.  An example of a private cache is the one *inside* your web browser.   This is probably what you want if the entity contains personal information."

	private := true.
	^self! !

!HTTPCacheControlField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
setPublic
	"^self
I am being told that the entity in my message is a public one that can be cached on public caches, i.e. caches that can be drawn upon by many clients.  This is probably not what you want if the entity contains personal information!!  c.f. >>setPrivate  Note that expicitly setting cache-control public actually loosens some other rules and means resources can be used by cached beyond their normal life."

	private := false.
	^self! !

!HTTPCacheControlField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream nextPut: Character space.
	self directives notNil ifTrue: [aStream nextPutAll: self directives].
	self private notNil 
		ifTrue: 
			[self writePublicOrPrivateTo: aStream.
			self maxAge notNil ifTrue: [aStream nextPutAll: ', ']].
	self maxAge notNil ifTrue: [self writeMaxAgeTo: aStream].
	^self! !

!HTTPCacheControlField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
writeMaxAgeTo: aStream 
	"^self
I write the maxAge directive to aStream"

	aStream nextPutAll: 'max-age='.
	self maxAge printOn: aStream.
	^self! !

!HTTPCacheControlField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
writePublicOrPrivateTo: aStream 
	"^self
I write the either the public or the private directive to aStream"

	self private 
		ifTrue: [aStream nextPutAll: 'private']
		ifFalse: [aStream nextPutAll: 'public'].
	^self! !

SpecificHeaderField subclass: #HTTPConnectionField
	instanceVariableNames: 'connectionToken'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPConnectionField commentStamp: '<historical>' prior: 0!
c.f. RFC 2616 14.10

   The Connection header has the following grammar:

       Connection = "Connection" ":" 1#(connection-token)
       connection-token  = token

!

!HTTPConnectionField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Connection'! !

!HTTPConnectionField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
connectionToken
	"^a String
Common values are 'close' and 'keep-alive'."

	^connectionToken! !

!HTTPConnectionField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
connectionToken: aString 
	"^self"

	connectionToken := aString.
	^self! !

!HTTPConnectionField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
connectionTokenIsClose
	^self connectionToken = 'close'! !

!HTTPConnectionField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
parseValueFrom: aString 
	connectionToken := HTTPString trimBlanksFrom: aString.
	^self! !

!HTTPConnectionField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
setToClose
	self connectionToken: 'close'.
	^self! !

!HTTPConnectionField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream nextPutAll: connectionToken.
	^self! !

SpecificHeaderField subclass: #HTTPContentLengthField
	instanceVariableNames: 'contentLength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPContentLengthField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Content-Length'! !

!HTTPContentLengthField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
contentLength
	^contentLength! !

!HTTPContentLengthField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
parseValueFrom: aString
	contentLength := aString asNumber.
	^self! !

!HTTPContentLengthField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	self contentLength printOn: aStream.
	^self! !

SpecificHeaderField subclass: #HTTPCookieField
	instanceVariableNames: 'values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPCookieField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Cookie'! !

SpecificHeaderField subclass: #HTTPDateField
	instanceVariableNames: 'date'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPDateField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Date'! !

!HTTPDateField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
date
	^date! !

!HTTPDateField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
date: aDate 
	"^self
Note that this is an HTTP Date, and so is really a timestamp :-/ "

	date := aDate.
	^self! !

!HTTPDateField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	self date asRFC1123StringOn: aStream.
	^self! !

SpecificHeaderField subclass: #HTTPETagField
	instanceVariableNames: 'entityTag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPETagField commentStamp: '<historical>' prior: 0!
RFC 2626 14.19 ETag

   The ETag response-header field provides the current value of the
   entity tag for the requested variant. The headers used with entity
   tags are described in sections 14.24, 14.26 and 14.44. The entity tag
   MAY be used for comparison with other entities from the same resource
   (see section 13.3.3).

      ETag = "ETag" ":" entity-tag

   Examples:

      ETag: "xyzzy"
      ETag: W/"xyzzy"
      ETag: ""

!

!HTTPETagField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'ETag'! !

!HTTPETagField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
entityTag
	^entityTag! !

!HTTPETagField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
entityTag: aString 
	entityTag := aString.
	^self! !

!HTTPETagField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream
		nextPut: $";
		nextPutAll: self entityTag;
		nextPut: $".
	^self! !

SpecificHeaderField subclass: #HTTPExpiresField
	instanceVariableNames: 'timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPExpiresField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Expires'! !

!HTTPExpiresField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
timestamp
	^timestamp! !

!HTTPExpiresField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
timestamp: aTimestamp
	timestamp := aTimestamp.! !

!HTTPExpiresField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	self timestamp asRFC1123StringOn: aStream.
	^self! !

SpecificHeaderField subclass: #HTTPHostField
	instanceVariableNames: 'hostName portNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPHostField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Host'! !

!HTTPHostField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
hostName
	^hostName! !

!HTTPHostField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
parseValueFrom: aString 
	| sourceStream portNumberString |
	sourceStream := ReadStream on: aString.
	hostName := sourceStream upTo: $:.
	portNumberString := sourceStream atEnd 
				ifTrue: [String new]
				ifFalse: [sourceStream upToEnd].
	portNumberString notEmpty 
		ifTrue: [portNumber := portNumberString asNumber].
	^self! !

!HTTPHostField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
portNumber
	^portNumber isNil ifTrue: [80] ifFalse: [portNumber]! !

!HTTPHostField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream nextPutAll: self hostName.
	portNumber notNil 
		ifTrue: 
			[aStream nextPut: $:.
			self portNumber printOn: aStream].
	^self! !

SpecificHeaderField subclass: #HTTPIfModifiedSinceField
	instanceVariableNames: 'date'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPIfModifiedSinceField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'If-Modified-Since'! !

!HTTPIfModifiedSinceField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
date
	^date! !

!HTTPIfModifiedSinceField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isCacheHitFor: anEntity 
	"^a Boolean
I return true if an anEntity is a cache hit given the conditional I represent.  So in my case, I'm looking to see that the entity has not changed since my date.
anEntity *must* respond to >>lastModified"

	^anEntity lastModified <= self date! !

!HTTPIfModifiedSinceField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isConditional
	^true! !

!HTTPIfModifiedSinceField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
parseValueFrom: aString 
	date := SpTimestamp fromRFC1123String: aString.
	^self! !

!HTTPIfModifiedSinceField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	self date asRFC1123StringOn: aStream.
	^self! !

SpecificHeaderField subclass: #HTTPIfRangeField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPIfRangeField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'If-Range'! !

SpecificHeaderField subclass: #HTTPIfUnmodifiedSinceField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPIfUnmodifiedSinceField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'If-Unmodified-Since'! !

!HTTPIfUnmodifiedSinceField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isCacheHitFor: anEntity 
	"^a Boolean
I return true if an anEntity is a cache hit given the conditional I represent. 
anEntity *must* respond to >>entutyTag"

	1 halt.
	^self! !

!HTTPIfUnmodifiedSinceField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isConditional
	^true! !

SpecificHeaderField subclass: #HTTPLastModifiedField
	instanceVariableNames: 'timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPLastModifiedField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Last-Modified'! !

!HTTPLastModifiedField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
timestamp
	^timestamp! !

!HTTPLastModifiedField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
timestamp: aTimestamp
	timestamp := aTimestamp.! !

!HTTPLastModifiedField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	self timestamp asRFC1123StringOn: aStream.
	^self! !

SpecificHeaderField subclass: #HTTPLocationField
	instanceVariableNames: 'uri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPLocationField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Location'! !

!HTTPLocationField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
uri
	^uri! !

!HTTPLocationField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
uri: aSwazooURI 
	uri := aSwazooURI.
	^self! !

!HTTPLocationField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
uriString: aString 
	uri := SwazooURI fromString: aString.
	^self! !

!HTTPLocationField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	self uri printOn: aStream.
	^self! !

SpecificHeaderField subclass: #HTTPMatchField
	instanceVariableNames: 'entityTags'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

HTTPMatchField subclass: #HTTPIfMatchField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPIfMatchField commentStamp: '<historical>' prior: 0!
From RFC 2616

14.24 If-Match

   The If-Match request-header field is used with a method to make it
   conditional. A client that has one or more entities previously
   obtained from the resource can verify that one of those entities is
   current by including a list of their associated entity tags in the
   If-Match header field. Entity tags are defined in section 3.11. The
   purpose of this feature is to allow efficient updates of cached
   information with a minimum amount of transaction overhead. It is also
   used, on updating requests, to prevent inadvertent modification of
   the wrong version of a resource. As a special case, the value "*"
   matches any current entity of the resource.

       If-Match = "If-Match" ":" ( "*" | 1#entity-tag )

   If any of the entity tags match the entity tag of the entity that
   would have been returned in the response to a similar GET request
   (without the If-Match header) on that resource, or if "*" is given

   and any current entity exists for that resource, then the server MAY
   perform the requested method as if the If-Match header field did not
   exist.

   A server MUST use the strong comparison function (see section 13.3.3)
   to compare the entity tags in If-Match.

   If none of the entity tags match, or if "*" is given and no current
   entity exists, the server MUST NOT perform the requested method, and
   MUST return a 412 (Precondition Failed) response. This behavior is
   most useful when the client wants to prevent an updating method, such
   as PUT, from modifying a resource that has changed since the client
   last retrieved it.

   If the request would, without the If-Match header field, result in
   anything other than a 2xx or 412 status, then the If-Match header
   MUST be ignored.

   The meaning of "If-Match: *" is that the method SHOULD be performed
   if the representation selected by the origin server (or by a cache,
   possibly using the Vary mechanism, see section 14.44) exists, and
   MUST NOT be performed if the representation does not exist.

   A request intended to update a resource (e.g., a PUT) MAY include an
   If-Match header field to signal that the request method MUST NOT be
   applied if the entity corresponding to the If-Match value (a single
   entity tag) is no longer a representation of that resource. This
   allows the user to indicate that they do not wish the request to be
   successful if the resource has been changed without their knowledge.
   Examples:

       If-Match: "xyzzy"
       If-Match: "xyzzy", "r2d2xxxx", "c3piozzzz"
       If-Match: *

   The result of a request having both an If-Match header field and
   either an If-None-Match or an If-Modified-Since header fields is
   undefined by this specification.

!

!HTTPIfMatchField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'If-Match'! !

!HTTPIfMatchField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isCacheHitFor: anEntity 
	"^a Boolean
I return true if an anEntity is a cache hit given the conditional I represent. 
anEntity *must* respond to >>entutyTag"

	1 halt.
	^self! !

HTTPMatchField subclass: #HTTPIfNoneMatchField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPIfNoneMatchField commentStamp: '<historical>' prior: 0!
This is a confitional header field.  The HTTP client is asking for a resource on the basis of this condition.  So, we need to have first found the resource, and then we can considder the condition, as follows ...

From RFC 2616:

14.26 If-None-Match

   The If-None-Match request-header field is used with a method to make
   it conditional. A client that has one or more entities previously
   obtained from the resource can verify that none of those entities is
   current by including a list of their associated entity tags in the
   If-None-Match header field. The purpose of this feature is to allow
   efficient updates of cached information with a minimum amount of
   transaction overhead. It is also used to prevent a method (e.g. PUT)
   from inadvertently modifying an existing resource when the client
   believes that the resource does not exist.

   As a special case, the value "*" matches any current entity of the
   resource.

       If-None-Match = "If-None-Match" ":" ( "*" | 1#entity-tag )

   If any of the entity tags match the entity tag of the entity that
   would have been returned in the response to a similar GET request
   (without the If-None-Match header) on that resource, or if "*" is
   given and any current entity exists for that resource, then the
   server MUST NOT perform the requested method, unless required to do
   so because the resource's modification date fails to match that
   supplied in an If-Modified-Since header field in the request.
   Instead, if the request method was GET or HEAD, the server SHOULD
   respond with a 304 (Not Modified) response, including the cache-
   related header fields (particularly ETag) of one of the entities that
   matched. For all other request methods, the server MUST respond with
   a status of 412 (Precondition Failed).

   See section 13.3.3 for rules on how to determine if two entities tags
   match. The weak comparison function can only be used with GET or HEAD
   requests.

   If none of the entity tags match, then the server MAY perform the
   requested method as if the If-None-Match header field did not exist,
   but MUST also ignore any If-Modified-Since header field(s) in the
   request. That is, if no entity tags match, then the server MUST NOT
   return a 304 (Not Modified) response.

   If the request would, without the If-None-Match header field, result
   in anything other than a 2xx or 304 status, then the If-None-Match
   header MUST be ignored. (See section 13.3.4 for a discussion of
   server behavior when both If-Modified-Since and If-None-Match appear
   in the same request.)

   The meaning of "If-None-Match: *" is that the method MUST NOT be
   performed if the representation selected by the origin server (or by
   a cache, possibly using the Vary mechanism, see section 14.44)
   exists, and SHOULD be performed if the representation does not exist.
   This feature is intended to be useful in preventing races between PUT
   operations.

   Examples:

       If-None-Match: "xyzzy"
       If-None-Match: W/"xyzzy"
       If-None-Match: "xyzzy", "r2d2xxxx", "c3piozzzz"
       If-None-Match: W/"xyzzy", W/"r2d2xxxx", W/"c3piozzzz"
       If-None-Match: *

   The result of a request having both an If-None-Match header field and
   either an If-Match or an If-Unmodified-Since header fields is
   undefined by this specification.!

!HTTPIfNoneMatchField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'If-None-Match'! !

!HTTPIfNoneMatchField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isCacheHitFor: anEntity 
	"^a Boolean
I return true if an anEntity is a cache hit given the conditional I represent.  So in my case, I'm looking to see that the entity has a tag which is in my collection of entityTags.
anEntity *must* respond to >>entityTag"

	^self entityTags includes: anEntity entityTag! !

!HTTPMatchField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
addEntityTag: aString 
	self entityTags add: aString.
	^self! !

!HTTPMatchField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
combineWith: aHeaderField 
	"^self
I add the entity tags of aHeaderField to my own collection of entity tags."

	self entityTags addAll: aHeaderField entityTags.
	^self! !

!HTTPMatchField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
entityTags
	^self matchesAnyCurrentEntity 
		ifTrue: [nil]
		ifFalse: 
			[entityTags isNil ifTrue: [entityTags := OrderedCollection new].
			entityTags]! !

!HTTPMatchField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isConditional
	^true! !

!HTTPMatchField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
matchesAnyCurrentEntity
	^entityTags = '*'! !

!HTTPMatchField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
parseValueFrom: aString 
	aString = '*' 
		ifTrue: [entityTags := aString]
		ifFalse: 
			[| sourceStream |
			entityTags := OrderedCollection new.
			sourceStream := ReadStream on: aString.
			[sourceStream atEnd] whileFalse: 
					[| entityTag |
					sourceStream upTo: $".
					entityTag := sourceStream upTo: $".
					entityTags add: entityTag.
					sourceStream upTo: $,]].
	^self! !

!HTTPMatchField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: targetStream 
	self write: self entityTags first asQuotedStringTo: targetStream.
	2 to: self entityTags size
		do: 
			[:tagIndex | 
			targetStream nextPut: $,.
			self write: (self entityTags at: tagIndex) asQuotedStringTo: targetStream].
	^self! !

!HTTPMatchField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
write: aString asQuotedStringTo: targetStream 
	"^self
See RFC 2616 2.2"

	targetStream nextPut: $".
	aString do: 
			[:character | 
			character == $" 
				ifTrue: [targetStream nextPutAll: '\"']
				ifFalse: [targetStream nextPut: character]].
	targetStream nextPut: $".
	^self! !

SpecificHeaderField subclass: #HTTPRefererField
	instanceVariableNames: 'uri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPRefererField commentStamp: '<historical>' prior: 0!
RFC 2616: 14.36 Referer

   The Referer[sic] request-header field allows the client to specify,
   for the server's benefit, the address (URI) of the resource from
   which the Request-URI was obtained (the "referrer", although the
   header field is misspelled.) The Referer request-header allows a
   server to generate lists of back-links to resources for interest,
   logging, optimized caching, etc. It also allows obsolete or mistyped
   links to be traced for maintenance. The Referer field MUST NOT be
   sent if the Request-URI was obtained from a source that does not have
   its own URI, such as input from the user keyboard.

       Referer        = "Referer" ":" ( absoluteURI | relativeURI )

   Example:

       Referer: http://www.w3.org/hypertext/DataSources/Overview.html

   If the field value is a relative URI, it SHOULD be interpreted
   relative to the Request-URI. The URI MUST NOT include a fragment. See
   section 15.1.3 for security considerations.

!

!HTTPRefererField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Referer'! !

!HTTPRefererField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
parseValueFrom: aString 
	uri := SwazooURI fromString: aString.
	^self! !

!HTTPRefererField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
uri
	^uri! !

!HTTPRefererField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	self uri printOn: aStream.
	^self! !

SpecificHeaderField subclass: #HTTPServerField
	instanceVariableNames: 'productTokens'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPServerField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Server'! !

!HTTPServerField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
productTokens
	^productTokens! !

!HTTPServerField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
productTokens: aString 
	productTokens := aString.
	^self! !

!HTTPServerField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream nextPutAll: self productTokens.
	^self! !

SpecificHeaderField subclass: #HTTPSetCookieField
	instanceVariableNames: 'cookies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPSetCookieField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'Set-Cookie'! !

!HTTPSetCookieField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
addCookie: aCookieString
	^self cookies add: aCookieString! !

!HTTPSetCookieField methodsFor: 'services' stamp: ' 14/4/07 23:10'!
combineWith: aSetCookieField 
	"^self
I add the cookies of aSetCookieField to my own collection of cookies."

	self cookies addAll: aSetCookieField cookies.
	^self! !

!HTTPSetCookieField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
cookies
	cookies isNil ifTrue: [cookies := OrderedCollection new].
	^cookies! !

!HTTPSetCookieField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream nextPutAll: (self cookies at: 1).
	2 to: self cookies size
		do: 
			[:cookieIndex | 
			aStream
				nextPutAll: ', ';
				nextPutAll: (self cookies at: cookieIndex)].
	^self! !

SpecificHeaderField subclass: #HTTPUserAgentField
	instanceVariableNames: 'productTokens'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPUserAgentField commentStamp: '<historical>' prior: 0!
RFC 2616: 14.43 User-Agent

   The User-Agent request-header field contains information about the
   user agent originating the request. This is for statistical purposes,
   the tracing of protocol violations, and automated recognition of user
   agents for the sake of tailoring responses to avoid particular user
   agent limitations. User agents SHOULD include this field with
   requests. The field can contain multiple product tokens (section 3.8)
   and comments identifying the agent and any subproducts which form a
   significant part of the user agent. By convention, the product tokens
   are listed in order of their significance for identifying the
   application.

       User-Agent     = "User-Agent" ":" 1*( product | comment )

   Example:

       User-Agent: CERN-LineMode/2.15 libwww/2.17b3!

!HTTPUserAgentField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'User-Agent'! !

!HTTPUserAgentField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
parseValueFrom: aString 
	"^self
I could try and parse out the product name and version numbers, but there is no need to worry about this at the moment, so I just record the string."

	productTokens := HTTPString trimBlanksFrom: aString.
	^self! !

!HTTPUserAgentField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
productTokens
	^productTokens! !

!HTTPUserAgentField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream nextPutAll: productTokens.
	^self! !

SpecificHeaderField subclass: #HTTPWWWAuthenticateField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

HTTPWWWAuthenticateField subclass: #HTTPWWWAuthenticateBasicField
	instanceVariableNames: 'realm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPWWWAuthenticateBasicField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isBasic
	"^a Boolean
I return true if I represent a header for basic authentication. c.f. RFC 2617 sec 2."

	^true! !

!HTTPWWWAuthenticateBasicField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
realm
	"^a String
I return the realm for which I represent an autentication challenge.  This string will be presented to the browser user in the login dialog."

	^realm! !

!HTTPWWWAuthenticateBasicField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
realm: anObject
	realm := anObject! !

!HTTPWWWAuthenticateBasicField methodsFor: 'printing' stamp: ' 14/4/07 23:10'!
valuesAsStringOn: aStream 
	aStream
		nextPutAll: 'Basic realm="';
		nextPutAll: self realm;
		nextPut: $".
	^self! !

HTTPWWWAuthenticateField subclass: #HTTPWWWAuthenticateDigestField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Headers'!

!HTTPWWWAuthenticateDigestField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isDigest
	"^a Boolean
I return true if I represent a header for digest authentication. c.f. RFC 2617 sec 3."

	^true! !

!HTTPWWWAuthenticateField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^'WWW-Authenticate'! !

!HTTPWWWAuthenticateField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isBasic
	"^a Boolean
I return true if I represent a header for basic authentication. c.f. RFC 2617 sec 2."

	^false! !

!HTTPWWWAuthenticateField methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isDigest
	"^a Boolean
I return true if I represent a header for digest authentication. c.f. RFC 2617 sec 3."

	^false! !

!SpecificHeaderField class methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
fieldName
	^self subclassResponsibility! !

!SpecificHeaderField class methodsFor: 'private' stamp: ' 14/4/07 23:10'!
newForFieldName: fieldNameString withValueFrom: fieldValueString 
	^self newWithValueFrom: fieldValueString! !

!SpecificHeaderField class methodsFor: 'private' stamp: ' 14/4/07 23:10'!
newWithValueFrom: fieldValueString 
	^self new valueFrom: fieldValueString! !

!SpecificHeaderField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
name
	^self class fieldName! !

!SpecificHeaderField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
parameterAt: aString ifAbsent: aBlock 
1 halt: 'use the transfer encodings of the field, not this'.
	^self parameters at: aString ifAbsent: aBlock! !

!SpecificHeaderField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
parseValueFrom: aString
	^self subclassResponsibility! !

!SpecificHeaderField methodsFor: 'private' stamp: ' 14/4/07 23:10'!
readParametersFrom: sourceStream 
	"^a Dictionary
c.f. RFC 2616 3.6 Transfer Codings"

	| parameters |
	parameters := Dictionary new.
	[sourceStream atEnd] whileFalse: 
			[| attribute value |
			attribute := HTTPString trimBlanksFrom: (sourceStream upTo: $=).
			value := HTTPString trimBlanksFrom: (sourceStream upTo: $;).
			parameters at: attribute put: value].
	^parameters! !

!SpecificHeaderField methodsFor: 'initialize-release' stamp: ' 14/4/07 23:10'!
valueFrom: fieldValueString 
	self parseValueFrom: fieldValueString.
	^self! !

!SpecificHeaderField methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
values
	^Array with: self value! !

Object subclass: #MimeObject
	instanceVariableNames: 'contentType value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

MimeObject subclass: #HTTPPostDatum
	instanceVariableNames: 'filename writeStream writeBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!HTTPPostDatum methodsFor: 'private-accessing' stamp: ' 14/4/07 23:10'!
defaultContentType
	^'text/plain'! !

!HTTPPostDatum methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
filename
	^filename! !

!HTTPPostDatum methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
filename: aString
	filename := aString! !

!HTTPPostDatum methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
filenameWithoutPath
	"M$ Internet Explorer includes full path in filename of uploaded file!! "
	self filename isNil ifTrue: [^nil].
	^(self filename includes: $\ ) 
		ifTrue: [self filename copyFrom: (self filename lastIndexOf: $\ )+1 to: self filename size]
		ifFalse: [self filename]! !

!HTTPPostDatum methodsFor: 'testing' stamp: 'mivsek 7/24/2007 21:04'!
isStreamed
	"this postDatum is streamed - it has an output stream to receive data into or a block 
	which will set it"
	^self writeStream notNil or: [self writeBlock notNil]! !

!HTTPPostDatum methodsFor: 'accessing' stamp: 'mivsek 7/24/2007 21:03'!
writeBlock
	^writeBlock! !

!HTTPPostDatum methodsFor: 'accessing' stamp: 'mivsek 7/24/2007 21:03'!
writeBlock: aBlockClosure
	"this block will be called just before start of streaming to writeStream. It can be used to 
	open the writeStream, because on that time we already know the filename of uploaded file. 
	As a parameter this postDatum is sent"
	writeBlock := aBlockClosure! !

!HTTPPostDatum methodsFor: 'accessing' stamp: 'mivsek 7/20/2007 23:44'!
writeStream
	^writeStream! !

!HTTPPostDatum methodsFor: 'accessing' stamp: 'mivsek 7/20/2007 23:44'!
writeStream: aWriteStream
	"a binary stream where to put directly a post data"
	writeStream := aWriteStream! !

!MimeObject methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
contentType
	^contentType isNil ifTrue: [self defaultContentType] ifFalse: [contentType]! !

!MimeObject methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
contentType: anObject
	contentType := anObject! !

!MimeObject methodsFor: 'private-accessing' stamp: ' 14/4/07 23:10'!
defaultContentType
	^'application/octet-stream'! !

!MimeObject methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
value
	^value! !

!MimeObject methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
value: anObject
	value := anObject! !

Object subclass: #Resource
	instanceVariableNames: 'enabled uriPattern parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Core'!

!Resource commentStamp: 'JM 6/6/2007 13:49' prior: 0!
Resource is an abstract class for all so called web resources. Such resource has its url address and can serve with responding to web requests. Every resource need to #answerTo: aHTTPRequest with aHTTPResponse. Site is a subclass of a Resource. You can subclass it with your own implementation. There is also a CompositeResource, which can hold many subresources. Site is also aCopmpositeResource and therefore you can add your own resources to your site.!

Resource subclass: #CompositeResource
	instanceVariableNames: 'children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Core'!

!CompositeResource methodsFor: 'adding/removing' stamp: ' 14/4/07 23:09'!
addResource: aResource
	self children add: aResource.
	aResource parent: self.
	aResource onResourceCreated.
	^aResource! !

!CompositeResource methodsFor: 'adding/removing' stamp: ' 14/4/07 23:09'!
addResources: anOrderedCollection
	anOrderedCollection do: [ :each |
		self addResource: each].
	^anOrderedCollection! !

!CompositeResource methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
children
	children isNil ifTrue: [self initChildren].
	^children! !

!CompositeResource methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
currentUrl
	| string |
	string := super currentUrl.
	^string last = $/
		ifTrue: [string]
		ifFalse: [string , '/']! !

!CompositeResource methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
hasNoResources
	^self children isEmpty! !

!CompositeResource methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
helpResolve: aResolution 
	^aResolution resolveCompositeResource: self! !

!CompositeResource methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
includesResource: aResource
	^self children includes: aResource.! !

!CompositeResource methodsFor: 'initialize-release' stamp: ' 14/4/07 23:09'!
initChildren
	children := OrderedCollection new.! !

!CompositeResource methodsFor: 'initialize-release' stamp: ' 14/4/07 23:09'!
initialize
	super initialize.
	self initChildren! !

!CompositeResource methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
isRootPath
	^self uriPattern = '/'! !

!CompositeResource methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
printUrlOn: aWriteStream 
	super printUrlOn: aWriteStream.
	self isRootPath ifFalse: [aWriteStream nextPut: $/]! !

!CompositeResource methodsFor: 'adding/removing' stamp: ' 14/4/07 23:09'!
removeResource: aResource
	self children remove: aResource ifAbsent: [nil]! !

CompositeResource subclass: #ServerRootComposite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!ServerRootComposite methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
helpResolve: aResolution 
	^aResolution resolveServerRoot: self! !

CompositeResource subclass: #Site
	instanceVariableNames: 'name serving'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Core'!

!Site commentStamp: 'JM 6/6/2007 13:49' prior: 0!
Site : Swazoo can serve many sites at once (virtual sites). Class Site is therefore a main class to start configuring your server. It holds an IP, port and hostname of your site.!

!Site class methodsFor: 'accessing' stamp: 'JM 4/17/2007 16:38'!
named: aString
	"return a website with that name"
	^SwazooServer singleton siteNamed: aString! !

!Site class methodsFor: 'instance creation' stamp: 'JM 4/17/2007 16:38'!
newNamed: aString
	| site |
	site := self new name: aString.
	SwazooServer singleton addSite: site.
	site initialize.
	^site! !

!Site methodsFor: 'accessing' stamp: ' 10/7/07 17:21'!
addAlias: anAlias 
	self ip isNil "initial uriPattern not yet setup"
		ifTrue: [self host: anAlias host ip: anAlias ip port: anAlias port]
		ifFalse: [self uriPattern add: anAlias]! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
address
	^self host! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
aliases
	^self uriPattern! !

!Site methodsFor: 'configuration' stamp: ' 14/4/07 23:09'!
compile: tag 
	^SwazooCompiler evaluate: tag! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
helpResolve: aResolution 
	^aResolution resolveSite: self! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
host
	"hostname of this site. Example: www.ibm.com. 
	hostname must be unique on that server.
	Don't mix with ip, which also can be something like www.ibm.com. 
	There can be many sites with different hostnames on the same ip !! "
	self uriPattern isEmpty ifTrue: [^''].
	^self uriPattern first host! !

!Site methodsFor: 'private' stamp: ' 14/4/07 23:09'!
host: aString
	self uriPattern first host: aString! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
host: aHostString ip: anIPString port: aNumber 
	"see comments in methods host and ip !! "
	"hostname must be unique!! "
	(SwazooServer singleton siteHostnamed: aHostString) notNil 
		ifTrue: [^SwazooSiteError error: 'Site with that hostname already exist!!'].
	self uriPattern isEmpty ifTrue: [self uriPattern add: SiteIdentifier new].
	self uriPattern first 
		setIp: anIPString
		port: aNumber
		host: aHostString! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
host: aHostString ip: anIPString port: aNumber sslPort: aSSLNumber
	"see comments in methods host and ip !! "
	"hostname must be unique!! "
	self host: aHostString ip: anIPString port: aNumber.
	self uriPattern size < 2 ifTrue: [self uriPattern add: SSLSiteIdentifier new].
	(self uriPattern at: 2)
		setIp: anIPString
		port: aSSLNumber
		host: aHostString! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
host: aHostString port: aNumber 
	"run on all ip interfaces on specified port"
	"hostname must be unique!! "
	self host: aHostString ip: '*' port: aNumber! !

!Site methodsFor: 'private-initialize' stamp: 'JM 6/6/2007 11:56'!
initUriPattern
	self uriPattern: OrderedCollection new.
	self uriPattern add: SiteIdentifier new.! !

!Site methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
initialize
	super initialize.
	self stop. "in case you initialize working site"
	self initUriPattern! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
ip
	"IP address of this site. Swazoo can have virtual sites, that is, more than one 
	site can share the same ip and port!!
	IP can be a number or full DNS name. For example: server.ibm.com or 234.12.45.66"
	^self uriPattern first ip! !

!Site methodsFor: 'private' stamp: ' 14/4/07 23:09'!
ip: aString
	self uriPattern first  ip: aString! !

!Site methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
isRootPath
	^false! !

!Site methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
isServing
	"is this site on-line?"
	^serving notNil and: [serving]! !

!Site methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
match: aSiteIdentifier 
	self uriPattern detect: [:each | each match: aSiteIdentifier]
		ifNone: [^false].
	^true! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
name
	"a short name of that site. Example: for host www.ibm.com, name it ibm"
	name isNil ifTrue: [^''].
	^name! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
name: aString
	"a short name of that site. Example: for host www.ibm.com, name it ibm"
	"name must be unique"
	(SwazooServer singleton siteNamed: aString) notNil 
		ifTrue: [^SwazooSiteError error: 'Site with that name already exist!!'].
	name := aString! !

!Site methodsFor: 'configuration' stamp: ' 14/4/07 23:09'!
nextTagFrom: aStream 
	aStream upTo: $<.
	^aStream atEnd ifTrue: [nil] ifFalse: [aStream upTo: $>]! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
port
	^self uriPattern first port! !

!Site methodsFor: 'private' stamp: ' 14/4/07 23:09'!
port: aNumber
	self uriPattern first port: aNumber! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
printUrlOn: aWriteStream 
	self uriPattern first printUrlOn: aWriteStream! !

!Site methodsFor: 'configuration' stamp: ' 14/4/07 23:09'!
readCompositeFrom: aStream storingInto: aComposite 
	| tag |
	
	[tag := self nextTagFrom: aStream.
	tag = '/CompositeResource']
		whileFalse: 
			[| thingy |
			thingy := self compile: tag.
			aComposite addResource: thingy.
			(thingy isKindOf: CompositeResource)
				ifTrue: [self readCompositeFrom: aStream storingInto: thingy]]! !

!Site methodsFor: 'configuration' stamp: ' 14/4/07 23:09'!
readFrom: aStream 
	"read configuration from an XML file, see sites.cnf"
	| tag |
	tag := self nextTagFrom: aStream.
	tag isNil ifTrue: [^nil].
	tag = 'Site' ifFalse: [^SwazooSiteError error: 'invalid site specification!!'].
	[tag := self nextTagFrom: aStream.
	tag = '/Site'] 	whileFalse: 
			[| thingy |
			thingy := self compile: tag.
			(thingy isKindOf: SiteIdentifier)
				ifTrue: [self addAlias: thingy]
				ifFalse: 
					[self addResource: thingy.
					(thingy isKindOf: CompositeResource) 
						ifTrue: [self readCompositeFrom: aStream storingInto: thingy]]]! !

!Site methodsFor: 'private' stamp: ' 14/4/07 23:09'!
serving: aBoolean
	serving := aBoolean! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
sslPort: aNumber
	self uriPattern size < 2 ifTrue: [self uriPattern add: SSLSiteIdentifier new].
	(self uriPattern at: 2)
		setIp: self ip
		port: aNumber
		host: self host! !

!Site methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
start
	| registry |
	registry := SwazooServer singleton.
	[self aliases do: [:each | | server |
		server := registry serverFor: each.
		server addSite: self.
		(registry servers includes: server) 
			ifFalse: 
				[registry addServer: server.
				server isServing ifFalse: [server start] ] ].
	] ifCurtailed: [self stop].
	self serving: true.! !

!Site methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
stop
	| registry |
	registry := SwazooServer singleton.
	self aliases do: [:each | | server |
		server := registry serverFor: each.
		(registry servers includes: server) 
			ifTrue: 
				[server removeSite: self.
				server hasNoSites ifTrue: 
					[registry removeServer: server.
					server stop]]].
		self serving: false.! !

!Site methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
uriPattern
	uriPattern isNil ifTrue: [self initUriPattern].
	^uriPattern! !

Resource subclass: #FileMappingResource
	instanceVariableNames: 'directoryIndex filePath'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Resources'!

!FileMappingResource class methodsFor: 'instance creation' stamp: ' 14/4/07 23:10'!
uriPattern: aString filePath: aFilePath 
	^(self uriPattern: aString) filePath: aFilePath! !

!FileMappingResource class methodsFor: 'instance creation' stamp: ' 14/4/07 23:10'!
uriPattern: aString filePath: aFilePath directoryIndex: anotherString 
	^(self uriPattern: aString)
		filePath: aFilePath;
		directoryIndex: anotherString! !

!FileMappingResource methodsFor: 'serving' stamp: ' 14/4/07 23:10'!
answerTo: aRequest 
	(self checkExistence: aRequest) ifFalse: [^nil].
	(self checkURI: aRequest) 
		ifFalse: 
			[| response |
			response := HTTPResponse movedPermanently.
			response headers 
				addField: (HTTPLocationField new uriString: aRequest uri identifier , '/').
			^response].
	^self file: (self fileFor: aRequest) answerTo: aRequest! !

!FileMappingResource methodsFor: 'private' stamp: ' 14/4/07 23:10'!
checkExistence: aRequest 
	(self rootFileFor: aRequest) exists ifFalse: [^false].
	^(self fileFor: aRequest) exists! !

!FileMappingResource methodsFor: 'private' stamp: ' 14/4/07 23:10'!
checkURI: aRequest 
	| needsFinalSlash |
	needsFinalSlash := (self rootFileFor: aRequest) isDirectory 
				and: [aRequest uri isDirectory not].
	^needsFinalSlash not! !

!FileMappingResource methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
directoryIndex
	^directoryIndex! !

!FileMappingResource methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
directoryIndex: aString 
	directoryIndex := aString! !

!FileMappingResource methodsFor: 'private' stamp: ' 14/4/07 23:10'!
file: aFilename answerTo: aRequest 
	^self subclassResponsibility! !

!FileMappingResource methodsFor: 'private' stamp: ' 14/4/07 23:10'!
fileDirectory
	^self filePath asFilename! !

!FileMappingResource methodsFor: 'private' stamp: ' 14/4/07 23:10'!
fileFor: aRequest 
	| fn |
	fn := self rootFileFor: aRequest.
	fn isDirectory ifTrue: [fn := fn construct: self directoryIndex].
	^fn! !

!FileMappingResource methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
filePath
	^filePath! !

!FileMappingResource methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
filePath: aString 
	filePath := aString! !

!FileMappingResource methodsFor: 'private-initialize' stamp: ' 14/4/07 23:10'!
initialize
	super initialize.
	self directoryIndex: 'index.html'! !

!FileMappingResource methodsFor: 'private' stamp: ' 14/4/07 23:10'!
rootFileFor: aRequest 
	^aRequest tailPath inject: self fileDirectory
		into: 
			[:subPath :each | 
			(#('.' '..') includes: (HTTPString trimBlanksFrom: each)) 
				ifTrue: [subPath]
				ifFalse: [subPath construct: each]]! !

FileMappingResource subclass: #FileResource
	instanceVariableNames: ''
	classVariableNames: 'ContentTypes'
	poolDictionaries: ''
	category: 'Swazoo-Resources'!

!FileResource class methodsFor: 'class initialization' stamp: ' 14/4/07 23:10'!
initialize
	"self initialize"

	ContentTypes := (Dictionary new)
				add: '.txt' -> 'text/plain';
				add: '.html' -> 'text/html';
				add: '.htm' -> 'text/html';
				add: '.css' -> 'text/css';
				add: '.png' -> 'image/png';
				add: '.gif' -> 'image/gif';
				add: '.jpg' -> 'image/jpeg';
				add: '.m3u' -> 'audio/mpegurl';
				add: '.ico' -> 'image/x-icon';
				add: '.pdf' -> 'application/pdf';
				yourself! !

!FileResource methodsFor: 'private' stamp: ' 14/4/07 23:10'!
contentTypeFor: aString 
	^ContentTypes at: aString ifAbsent: ['application/octet-stream']! !

!FileResource methodsFor: 'private' stamp: ' 14/4/07 23:10'!
file: aFilename answerTo: aRequest 
	| cacheControl response |
	cacheControl := SwazooCacheControl new request: aRequest
				cacheTarget: aFilename.
	response := cacheControl isNotModified 
				ifTrue: [HTTPResponse notModified]
				ifFalse: 
					[FileResponse ok entity: ((MimeObject new)
								value: aFilename;
								contentType: (self contentTypeFor: aFilename extension))].
	cacheControl addResponseHeaders: response.
	^response! !

FileResource subclass: #HomeResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Resources'!

!HomeResource methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
answerTo: aRequest 
	aRequest tailPath isEmpty ifTrue: [^nil].
	(self validateHomePath: aRequest tailPath first) ifFalse: [^nil].
	^super answerTo: aRequest! !

!HomeResource methodsFor: 'private' stamp: ' 14/4/07 23:10'!
rootFileFor: aRequest 
	| homeKey file |
	homeKey := aRequest tailPath first copyFrom: 2
				to: aRequest tailPath first size.
	file := (self fileDirectory construct: homeKey) construct: 'html'.
	(aRequest tailPath copyFrom: 2 to: aRequest tailPath size) 
		do: [:each | each = '..' ifFalse: [file := file construct: each]].
	^file! !

!HomeResource methodsFor: 'private' stamp: ' 14/4/07 23:10'!
validateHomePath: aString 
	^aString first = $~! !

Resource subclass: #HelloWorldResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Resources'!

!HelloWorldResource methodsFor: 'serving' stamp: 'JM 4/17/2007 16:31'!
answerTo: aRequest 
	| response |
	response := HTTPResponse ok.
	response
		contentType: 'text/html';
		entity: '<html><head><title>Hello World</title></head><body>Hello World!!</body></html>'.
	^response! !

Resource subclass: #RedirectionResource
	instanceVariableNames: 'targetUri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Resources'!

!RedirectionResource class methodsFor: 'instance creation' stamp: ' 14/4/07 23:10'!
uriPattern: aString targetUri: bString 
	^(self uriPattern: aString) targetUri: bString! !

!RedirectionResource methodsFor: 'serving' stamp: ' 14/4/07 23:10'!
answerTo: aRequest 
	| answer |
	answer := HTTPResponse movedPermanently.
	answer headers addField: (HTTPLocationField new uriString: self targetUri).
	^answer! !

!RedirectionResource methodsFor: 'private-initialize' stamp: ' 14/4/07 23:10'!
targetUri
	^targetUri! !

!RedirectionResource methodsFor: 'private-initialize' stamp: ' 14/4/07 23:10'!
targetUri: aString 
	targetUri := aString! !

!Resource class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
new
	^super new initialize! !

!Resource class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
uriPattern: aString 
	^self new uriPattern: aString! !

!Resource methodsFor: 'serving' stamp: ' 14/4/07 23:09'!
answerTo: aRequest
	"override in your Resource and return a HTTPResponse"
	^nil! !

!Resource methodsFor: 'authentication' stamp: ' 14/4/07 23:09'!
authenticationRealm
	"rfc2617 3.2.1: A string to be displayed to users so they know which username and
     password to use. This string should contain at least the name of
     the host performing the authentication and might additionally
     indicate the collection of users who might have access. An example
     might be 'registered_users@gotham.news.com' "
	^'Swazoo server'! !

!Resource methodsFor: 'authentication' stamp: ' 14/4/07 23:09'!
authenticationScheme
	"#Basic or #Digest, see rfc2617. Digest is recomended because password
	goes encrypted to server"
	^#Digest! !

!Resource methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
canAnswer
	^self isEnabled and: [self isValidlyConfigured]! !

!Resource methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
currentUrl
	| stream |
	stream := WriteStream on: String new.
	self printUrlOn: stream.
	^stream contents! !

!Resource methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
disable
	enabled := false! !

!Resource methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
enable
	enabled := true! !

!Resource methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
helpResolve: aResolution 
	^aResolution resolveLeafResource: self! !

!Resource methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
initUriPattern
	self uriPattern: ''! !

!Resource methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
initialize
	self enable.
	self initUriPattern! !

!Resource methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
isEnabled
	^enabled! !

!Resource methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
isValidlyConfigured
	^self uriPattern ~= ''! !

!Resource methodsFor: 'private' stamp: ' 14/4/07 23:09'!
match: anIdentifier 
	^self uriPattern match: anIdentifier! !

!Resource methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
onResourceCreated
	"Received after the resource has been added to its parent resource. Opportunity to perform initialization that depends on knowledge of the resource tree structure"! !

!Resource methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
parent
	^parent! !

!Resource methodsFor: 'private' stamp: ' 14/4/07 23:09'!
parent: aResource
	parent := aResource! !

!Resource methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
printUrlOn: aWriteStream 
	self parent printUrlOn: aWriteStream.
	aWriteStream nextPutAll: self uriPattern! !

!Resource methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
root
	^self parent isNil
		ifTrue: [self]
		ifFalse: [self parent root]! !

!Resource methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
start! !

!Resource methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
stop! !

!Resource methodsFor: 'authentication' stamp: 'JM 4/14/2007 23:25'!
unauthorizedResponse
	"Resource should call this method and return its result immediately, if request is not authorized 
	to access that resource and a HTTP authorization is needed"
"	^HTTPAuthenticationChallenge newForResource: self " ! !

!Resource methodsFor: 'authentication' stamp: ' 14/4/07 23:09'!
unauthorizedResponsePage
	"Resource should override this method with it's own html message"
	^'<HTML>
  <HEAD>
    <TITLE>Authentication error</TITLE>
  </HEAD>
  <BODY>
    <H1>401 Authentication error</H1>
    <P>Bad username or password</P>
  </BODY>
</HTML>'! !

!Resource methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
uriPattern
	^uriPattern! !

!Resource methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
uriPattern: anIdentifier 
	anIdentifier notNil ifTrue: [uriPattern := anIdentifier]! !

Object subclass: #SwazooCacheControl
	instanceVariableNames: 'request cacheTarget etag lastModified'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Resources'!

!SwazooCacheControl methodsFor: 'operations' stamp: ' 14/4/07 23:10'!
addNotModifedHeaders: aResponse 
	"RFC2616 10.3.5
	If the conditional GET used a strong cache validator (see section 13.3.3), the response SHOULD NOT include other entity-headers. ... this prevents inconsistencies between cached entity-bodies and updated headers. "

	self isRequestStrongValidator 
		ifTrue: [aResponse headers addField: (HTTPETagField new entityTag: self etag)]
		ifFalse: [self basicAddResponseHeaders: aResponse].
	^aResponse! !

!SwazooCacheControl methodsFor: 'operations' stamp: ' 14/4/07 23:10'!
addResponseHeaders: aResponse 
	"Add response headers to the response.
	We MUST differentiate between 200/302 responses"

	^aResponse isNotModified 
		ifTrue: [self addNotModifedHeaders: aResponse]
		ifFalse: [self basicAddResponseHeaders: aResponse]! !

!SwazooCacheControl methodsFor: 'operations' stamp: 'JM 4/18/2007 16:23'!
basicAddResponseHeaders: aResponse 
	"RFC 2616 13.3.4
	HTTP/1.1 origin servers: 
      	- SHOULD send an entity tag validator unless it is not feasible to generate one.
		- SHOULD send a Last-Modified value "

	aResponse headers addField: (HTTPETagField new entityTag: self etag).
	aResponse headers addField: (HTTPLastModifiedField new timestamp: self lastModified).
	^aResponse! !

!SwazooCacheControl methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
cacheTarget
	^cacheTarget! !

!SwazooCacheControl methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
etag
	etag isNil ifTrue: [etag := self generateETag].
	^etag! !

!SwazooCacheControl methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
etag: aString 
	etag := aString! !

!SwazooCacheControl methodsFor: 'operations' stamp: ' 14/4/07 23:10'!
generateETag
	^self cacheTarget etag! !

!SwazooCacheControl methodsFor: 'operations' stamp: ' 14/4/07 23:10'!
generateLastModified
	^self cacheTarget lastModified! !

!SwazooCacheControl methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isIfModifiedSince
	"Answers true if either 
		- the request does not included the header
		-or there is not a match"

	| ifModifiedSince |
	ifModifiedSince := request headers fieldOfClass: HTTPIfModifiedSinceField
				ifNone: [nil].
	^ifModifiedSince isNil or: [self lastModified > ifModifiedSince date]! !

!SwazooCacheControl methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isIfNoneMatch
	"Answers true if either 
		- the request does not included the header
		-or there is not a match"

	| field |
	field := request headers fieldOfClass: HTTPIfNoneMatchField ifNone: [nil].
	^field isNil or: [(field entityTags includes: self etag) not]! !

!SwazooCacheControl methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isNotModified
	"Compare the cacheTarget with the request headers and answer if the client's version is not modified.
	Takes into account http version, and uses best practices defined by HTTP spec"

	^self isIfNoneMatch not or: [self isIfModifiedSince not]! !

!SwazooCacheControl methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
isRequestStrongValidator
	| field |
	field := request headers fieldOfClass: HTTPIfNoneMatchField ifNone: [nil].
	^field notNil and: [field entityTags isEmpty not]! !

!SwazooCacheControl methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
lastModified
	lastModified isNil ifTrue: [lastModified := self generateLastModified].
	^lastModified! !

!SwazooCacheControl methodsFor: 'testing' stamp: ' 14/4/07 23:10'!
lastModified: aRFC1123TimeStampString 
	lastModified := aRFC1123TimeStampString! !

!SwazooCacheControl methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
request: aHTTPGet cacheTarget: anObject 
	request := aHTTPGet.
	cacheTarget := anObject! !

Object subclass: #SwazooCompiler
	instanceVariableNames: 'accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Compatibility'!

!SwazooCompiler class methodsFor: 'evaluation' stamp: ' 14/4/07 23:09'!
evaluate: aString 
	^SpEnvironment evaluate: aString in: self class environment! !

!SwazooCompiler class methodsFor: 'evaluation' stamp: ' 14/4/07 23:09'!
evaluate: aString receiver: anObject 
	^SpEnvironment 
		evaluate: aString
		receiver: anObject
		in: self class environment! !

Object subclass: #SwazooServer
	instanceVariableNames: 'sites servers'
	classVariableNames: 'Singleton'
	poolDictionaries: ''
	category: 'Swazoo-Core'!

!SwazooServer commentStamp: 'JM 6/6/2007 14:03' prior: 0!
SwazooServer is where it all begins in Swazoo!!
SwazooServer singleton : return one and only one server which holds the Sites. Also used to start and stop all sites ato once, to add new sited etc. When running, a collection of HTTPServers is also stored in SwazooServer singleton.

SwazooServer demoStart  will create and run a demo site on http://localhost:8888 which 
                              returns a web page with 'Hello World!!'!

!SwazooServer class methodsFor: 'configuration' stamp: ' 14/4/07 23:09'!
configureFrom: aFilenameString 
	| sites stream |
	self singleton removeAllSites.
	stream := aFilenameString asFilename readStream.
	[sites := self readSitesFrom: stream] ensure: [stream close].
	sites do: [:each | 
		self singleton addSite: each.
		each start]! !

!SwazooServer class methodsFor: 'start/stop' stamp: 'JM 7/26/2007 16:30'!
demoStart
	"on http://localhost:8888/ will return simple 'Hello World'"
	| site |
	site := self singleton siteNamed: 'demo'.
	site isNil ifTrue: [site := self prepareDemoSite].
	site start! !

!SwazooServer class methodsFor: 'start/stop' stamp: 'JM 7/26/2007 16:30'!
demoStop
	self stopSite: 'demo'! !

!SwazooServer class methodsFor: 'configuration' stamp: 'JM 4/17/2007 16:33'!
exampleConfigurationFile
	"example sites.cnf, which will serve static files from current directory and respond with
	'Hello Worlrd' from url http://localhost:8888/foo/Howdy"

"<Site>
 	<SiteIdentifier ip: '127.0.0.1' port: 8888 host: 'localhost' >
 	<CompositeResource uriPattern: '/'>
  		<CompositeResource uriPattern: 'foo'>
   			<HelloWorldResource uriPattern: 'Howdy'>
  		</CompositeResource>
 	</CompositeResource>
 	<FileResource uriPattern: '/' filePath: '.'>
</Site>
"! !

!SwazooServer class methodsFor: 'private' stamp: ' 14/4/07 23:09'!
initSingleton
	Singleton := super new initialize.! !

!SwazooServer class methodsFor: 'initialize' stamp: ' 14/4/07 23:09'!
initialize! !

!SwazooServer class methodsFor: 'private' stamp: ' 14/4/07 23:09'!
new
	^self shouldNotImplement! !

!SwazooServer class methodsFor: 'private' stamp: 'JM 4/17/2007 16:32'!
prepareDemoSite
	"on http://localhost:8888 to return 'Hello Word' "
	| site |
	site := Site newNamed: 'demo'. "which is also added to SwazoServer"
	site host: 'localhost' ip: 'localhost' port: 8888.
	site addResource: (HelloWorldResource uriPattern: '/').
	^site! !

!SwazooServer class methodsFor: 'private' stamp: ' 14/4/07 23:09'!
readSitesFrom: aStream 
	| sites instance |
	sites := OrderedCollection new.
	[instance := Site new readFrom: aStream.
	instance notNil] whileTrue: [sites add: instance].
	^sites! !

!SwazooServer class methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
singleton
	Singleton isNil ifTrue: [self initSingleton].
	^Singleton! !

!SwazooServer class methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
siteHostnamed: aString
	^self singleton siteHostnamed: aString! !

!SwazooServer class methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
siteNamed: aString
	^self singleton siteNamed: aString! !

!SwazooServer class methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
start
	"start all sites"
	self singleton start! !

!SwazooServer class methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
startSite: aString
	"start site with that name"
	self singleton startSite: aString! !

!SwazooServer class methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
stop
	"stop all sites"
	self singleton stop! !

!SwazooServer class methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
stopSite: aString
	"stop site with that name"
	self singleton stopSite: aString! !

!SwazooServer methodsFor: 'private-servers' stamp: ' 14/4/07 23:09'!
addServer: aHTTPServer
	self servers add: aHTTPServer! !

!SwazooServer methodsFor: 'sites' stamp: ' 14/4/07 23:09'!
addSite: aSite
	(self siteNamed: aSite name) notNil 
		ifTrue: [^SwazooSiteError error: 'Site with that name already exist!!'].
	(self siteHostnamed: aSite host) notNil 
		ifTrue: [^SwazooSiteError error: 'Site with that hostname already exist!!'].
	self sites add: aSite! !

!SwazooServer methodsFor: 'sites' stamp: ' 14/4/07 23:09'!
allSites
	^self sites copy! !

!SwazooServer methodsFor: 'initialize-release' stamp: ' 14/4/07 23:09'!
initServers
	servers := Set new.! !

!SwazooServer methodsFor: 'initialize-release' stamp: ' 14/4/07 23:09'!
initSites
	sites := OrderedCollection new.! !

!SwazooServer methodsFor: 'initialize-release' stamp: ' 14/4/07 23:09'!
initialize
	self initSites.
	self initServers.! !

!SwazooServer methodsFor: 'private-servers' stamp: ' 14/4/07 23:09'!
newServerFor: aSiteIdentifier
	^ aSiteIdentifier newServer.! !

!SwazooServer methodsFor: 'sites' stamp: ' 14/4/07 23:09'!
removeAllSites
	self sites copy do: [:each | self removeSite: each]! !

!SwazooServer methodsFor: 'private-servers' stamp: ' 14/4/07 23:09'!
removeServer: aHTTPServer
	self servers remove: aHTTPServer! !

!SwazooServer methodsFor: 'sites' stamp: ' 14/4/07 23:09'!
removeSite: aSite 
	aSite stop.
	self sites remove: aSite! !

!SwazooServer methodsFor: 'private-servers' stamp: ' 14/4/07 23:09'!
serverFor: aSiteIdentifier 
	^self servers detect: [:each | (each ip = aSiteIdentifier ip) & (each port = aSiteIdentifier port)]
		ifNone: [self newServerFor: aSiteIdentifier]! !

!SwazooServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
servers
	servers isNil ifTrue: [self initServers].
	^servers! !

!SwazooServer methodsFor: 'sites' stamp: 'JM 6/6/2007 12:45'!
siteHostnamed: aString
	"find a site with that hostname"
	| string |
	string := aString isNil ifTrue: [''] ifFalse: [aString asLowercase].
	^self sites detect: [:each | 
		each host notNil and: [each host asLowercase = string] ] ifNone: [nil].! !

!SwazooServer methodsFor: 'sites' stamp: ' 14/4/07 23:09'!
siteNamed: aString
	"find a site with that short name"
	| string |
	string := aString isNil ifTrue: [''] ifFalse: [aString asLowercase].
	^self sites detect: [:each | each name asLowercase = string] ifNone: [nil].! !

!SwazooServer methodsFor: 'private' stamp: ' 14/4/07 23:09'!
sites
	sites isNil ifTrue: [self initSites].
	^sites! !

!SwazooServer methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
start
	self sites do: [:site | site start].! !

!SwazooServer methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
startSite: aString
	"start site with that name"
	| site |
	site := self siteNamed: aString.
	site notNil ifTrue: [site start].! !

!SwazooServer methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
stop
	self sites do: [:site | site stop].! !

!SwazooServer methodsFor: 'start/stop' stamp: ' 14/4/07 23:09'!
stopSite: aString
	"stop site with that name"
	| site |
	site := self siteNamed: aString.
	site notNil ifTrue: [site stop].! !

Object subclass: #SwazooSocket
	instanceVariableNames: 'accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Compatibility'!

SwazooSocket class
	instanceVariableNames: 'socketClass'!

SwazooSocket class
	instanceVariableNames: 'socketClass'!

!SwazooSocket class methodsFor: 'private' stamp: ' 14/4/07 23:09'!
accessor: aSocketAccessor 
	^self new accessor: aSocketAccessor! !

!SwazooSocket class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
connectTo: aHostString port: anInteger 
	| newSocket |
	newSocket := self socketClass newTCPSocket.
	newSocket 
		connectTo: (SpIPAddress hostName: aHostString port: anInteger).
	^self accessor: newSocket! !

!SwazooSocket class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
connectedPair
	^self socketClass newSocketPair collect: [:each | self accessor: each]! !

!SwazooSocket class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
serverOnIP: anIPString port: anInteger 
	| newSocket |
	newSocket := self socketClass newTCPSocket.
	newSocket
		setAddressReuse: true;
		bindSocketAddress: (SpIPAddress hostName: anIPString port: anInteger).
	^self accessor: newSocket! !

!SwazooSocket class methodsFor: 'private' stamp: ' 14/4/07 23:09'!
socketClass
	"^a Class
I return the class of my underlying socket.  This may in fact be a pseudo socket thing if we are testing."

	socketClass isNil ifTrue: [socketClass := SpSocket].
	^socketClass! !

!SwazooSocket class methodsFor: 'private' stamp: ' 14/4/07 23:09'!
socketClass: aClass 
	"^self
I record the class of my underlying socket."

	socketClass := aClass.
	^self! !

!SwazooSocket methodsFor: 'server accessing' stamp: ' 14/4/07 23:09'!
accept
	^self class accessor: self accessor acceptRetryingIfTransientErrors! !

!SwazooSocket methodsFor: 'private' stamp: ' 14/4/07 23:09'!
accessor
	^accessor! !

!SwazooSocket methodsFor: 'private' stamp: ' 14/4/07 23:09'!
accessor: aSocketAccessor 
	accessor := aSocketAccessor.! !

!SwazooSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
close
	self accessor close! !

!SwazooSocket methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
isActive
	^self accessor isActive! !

!SwazooSocket methodsFor: 'server accessing' stamp: ' 14/4/07 23:09'!
listenFor: anInteger 
	self accessor listenBackloggingUpTo: anInteger! !

!SwazooSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
localAddress
	^self accessor getSocketName hostAddressString! !

!SwazooSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
read: anInteger 
	"^a ByteArray
I delegate this to my underlying socket.  I always return bytes, not characters.  Character encoding issues must be handles by the sender."

	| readByteArray |
	readByteArray := self accessor read: anInteger.
	^readByteArray! !

!SwazooSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
read: anInteger timeout: aNumberOfMilliseconds 
	^(self accessor waitForReadDataUpToMs: aNumberOfMilliseconds) 
		ifTrue: [self read: anInteger]
		ifFalse: [ByteArray new]! !

!SwazooSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
remoteAddress
	^self accessor getPeerName hostAddressString! !

!SwazooSocket methodsFor: 'private' stamp: ' 14/4/07 23:09'!
stream
	^SwazooStream socket: self! !

!SwazooSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
write: aByteArray 
	^self accessor write: aByteArray! !

!SwazooSocket methodsFor: 'accessing' stamp: 'mivsek 7/22/2007 19:14'!
writeFrom: aByteArray startingAt: start for: length
	^self accessor writeFrom: aByteArray startingAt: start for: length! !

Object subclass: #SwazooStream
	instanceVariableNames: 'socket readBuffer writeBuffer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Compatibility'!

!SwazooStream class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
connectedPair
	^SwazooSocket connectedPair collect: [:each | self socket: each]! !

!SwazooStream class methodsFor: 'instance creation' stamp: ' 10/7/07 18:48'!
on: aString 
	"use only for testing!!"
	^self new readBuffer: (ReadStream on: aString)! !

!SwazooStream class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
socket: aSwazooSocket 
	^self new setSocket: aSwazooSocket! !

!SwazooStream methodsFor: 'accessing' stamp: ' 10/7/07 17:21'!
atEnd
	"TCP socket data never ends!!"
	^false! !

!SwazooStream methodsFor: 'accessing' stamp: 'mivsek 7/23/2007 22:50'!
close
	self socket close.
	writeBuffer := nil.
	readBuffer := nil.  "to GC buffers"! !

!SwazooStream methodsFor: 'accessing' stamp: ' 10/7/07 17:21'!
cr
	self nextPut: Character cr asInteger! !

!SwazooStream methodsFor: 'accessing' stamp: ' 10/7/07 18:48'!
crlf
	"^self
	I write a CRLF pair to my underlying stream."

	self
		cr;
		lf.
	^self! !

!SwazooStream methodsFor: 'private' stamp: ' 10/7/07 17:21'!
fillBuffer
	self socket isNil ifTrue: [^self]. "if SwazooStream is used for tests only"
	self readBuffer: (ReadStream on: (self socket read: 1024) ).
	self readBuffer atEnd 
		ifTrue: [SpError raiseSignal: 'No data available.  Socket probably closed']! !

!SwazooStream methodsFor: 'accessing' stamp: 'mivsek 7/22/2007 19:14'!
flush
	"actually write to the tcp socket and clear write buffer"
	self writeBuffer flushTo: self socket.! !

!SwazooStream methodsFor: 'private-initialize' stamp: 'mivsek 7/22/2007 19:15'!
initWriteBuffer
	"temporary stream. flush it to socket ocassionaly!!"
	writeBuffer := SwazooBuffer new! !

!SwazooStream methodsFor: 'accessing' stamp: ' 10/7/07 18:48'!
lf
	self nextPut: Character lf asInteger! !

!SwazooStream methodsFor: 'accessing' stamp: ' 10/7/07 17:21'!
next
	^self nextByte asCharacter! !

!SwazooStream methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
next: anInteger 
	"^aString
	We use a Stream to convert the ByteArray to a String because in GemStone
	ByteArray>>asString always returns the string 'aByteArray' :-/
	... but of course, this does not handle character encoding properly."

	| targetStream |
	targetStream := WriteStream on: String new.
	(self nextBytes: anInteger) 
		do: [:aByte | targetStream nextPut: aByte asCharacter].
	^targetStream contents! !

!SwazooStream methodsFor: 'accessing-bytes' stamp: ' 10/7/07 17:21'!
nextByte
	self syncBuffer.
	^self readBuffer next.! !

!SwazooStream methodsFor: 'accessing-bytes' stamp: ' 10/7/07 17:21'!
nextBytes: requestedNumberOfBytes 
	"^a ByteArray
I read the requestedNumberOfBytes from my underlying ReadStream.  This is expexted to be the raw HTTP byte stream."
	| nextBytes |
	nextBytes := ByteArray new: requestedNumberOfBytes.
	1 to: requestedNumberOfBytes
		do: 
			[:index | 
			nextBytes at: index put: self nextByte].
	^nextBytes! !

!SwazooStream methodsFor: 'accessing' stamp: ' 10/7/07 17:21'!
nextLine
	"^a String
I return the next line from my underlying stream.  This means reading everything up to the next crlf."

	| targetStream |
	targetStream := WriteStream on: String new.
	self writeNextLineTo: targetStream.
	^targetStream contents! !

!SwazooStream methodsFor: 'accessing' stamp: ' 10/7/07 17:21'!
nextPut: aCharacter 
	self nextPutByte: aCharacter asInteger.
	^aCharacter! !

!SwazooStream methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
nextPutAll: aString 
	self nextPutBytes: aString asByteArray.
	^aString! !

!SwazooStream methodsFor: 'accessing-bytes' stamp: ' 10/7/07 17:21'!
nextPutByte: aByte 
	self writeBuffer nextPut: aByte.
	^aByte! !

!SwazooStream methodsFor: 'accessing-bytes' stamp: ' 10/7/07 17:21'!
nextPutBytes: aByteArray 
	self writeBuffer nextPutAll: aByteArray.
	^aByteArray! !

!SwazooStream methodsFor: 'accessing' stamp: ' 10/7/07 18:48'!
nextPutLine: aCollection 
	"^self
	I write aCollection to  stream followed by a >>crlf."

	self nextPutAll: aCollection.
	self crlf.
	^self! !

!SwazooStream methodsFor: 'accessing' stamp: ' 10/7/07 17:21'!
nextUnfoldedLine
	"^a String
c.f. RFC 2616 2.2 Basic Rules
   HTTP/1.1 header field values can be folded onto multiple lines if the
   continuation line begins with a space or horizontal tab.
So, I read the next line, the I read subsequent lines until I get to one that is not a contuation.  I add the contents of the continuation lines to the target stream.  I leave my underlying stream positioned at the immediately after the last continuation.  If the initial line is empty this is the last line in the stream, so I read no more."

	| targetStream |
	targetStream := WriteStream on: String new.
	self writeNextLineTo: targetStream.
	targetStream contents isEmpty 
		ifFalse: 
			[
			[| nextCharacter |
			nextCharacter := self peek.
			nextCharacter notNil 
				and: [nextCharacter == Character space or: [nextCharacter == Character tab]]] 
					whileTrue: [self writeNextLineTo: targetStream]].
	^targetStream contents! !

!SwazooStream methodsFor: 'accessing' stamp: ' 10/7/07 17:21'!
peek
	| byte |
	byte := self peekByte.
	^byte notNil 
		ifTrue: [byte asCharacter] 
		ifFalse: [nil]! !

!SwazooStream methodsFor: 'accessing-bytes' stamp: ' 14/4/07 23:09'!
peekByte
	self syncBuffer.
	^self readBuffer peek! !

!SwazooStream methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
print: anObject 
	anObject printOn: self! !

!SwazooStream methodsFor: 'private' stamp: ' 14/4/07 23:09'!
readBuffer
	"^a ReadStream
I return a ReadStream on the current read buffer.  See >>fillBuffer to understand how the buffer is filled."
	^readBuffer! !

!SwazooStream methodsFor: 'private' stamp: ' 14/4/07 23:09'!
readBuffer: aStream
	readBuffer := aStream! !

!SwazooStream methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
setSocket: aSwazooSocket 
	self socket: aSwazooSocket.
	self readBuffer: (ReadStream on: ByteArray new)! !

!SwazooStream methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
socket
	^socket! !

!SwazooStream methodsFor: 'private' stamp: ' 14/4/07 23:09'!
socket: aSocket
	socket := aSocket! !

!SwazooStream methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
space
	self nextPut: Character space! !

!SwazooStream methodsFor: 'private' stamp: ' 14/4/07 23:09'!
syncBuffer
	self readBuffer atEnd ifTrue: [self fillBuffer]! !

!SwazooStream methodsFor: 'accessing' stamp: ' 10/7/07 17:21'!
upTo: aByte
	| ws byte |
	ws := ByteArray new writeStream.
	[byte := self nextByte.
	byte = aByte] whileFalse: [ws nextPut: byte].
	^ws contents! !

!SwazooStream methodsFor: 'private' stamp: ' 10/7/07 17:21'!
writeBuffer
	writeBuffer isNil ifTrue: [self initWriteBuffer].
	^writeBuffer! !

!SwazooStream methodsFor: 'accessing' stamp: 'mivsek 7/10/2007 18:54'!
writeNextLineTo: aStream 
	aStream nextPutAll: (self upTo: Character cr asInteger) asString.
	^self peekByte = Character lf asInteger 
		ifTrue: [self nextByte]  "skip remaining linefeed"
		ifFalse: [SwazooHTTPParseError raiseSignal: 'CR without LF']! !

Object subclass: #SwazooTask
	instanceVariableNames: 'request response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Messages'!

!SwazooTask commentStamp: '<historical>' prior: 0!
A SwazooTask is simply a request-response pair.  This class just makes the task (ha!!) of dealing with requests and responses a bit easier.!

!SwazooTask methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
request
	^request! !

!SwazooTask methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
request: aHTTPRequest
	request := aHTTPRequest! !

!SwazooTask methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
response
	^response! !

!SwazooTask methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
response: aHTTPResponse
	response := aHTTPResponse! !

Object subclass: #SwazooURI
	instanceVariableNames: 'protocol hostname port identifier queries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!SwazooURI class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
fromString: aString
	^self new fromString: aString! !

!SwazooURI class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
value: aString
	^self new value: aString! !

!SwazooURI methodsFor: 'printing' stamp: ' 14/4/07 23:09'!
asString
	| targetStream |
	targetStream := WriteStream on: String new.
	self printOn: targetStream.
	^targetStream contents! !

!SwazooURI methodsFor: 'private' stamp: ' 14/4/07 23:09'!
defaultPort
	^80! !

!SwazooURI methodsFor: 'initialize-release' stamp: ' 14/4/07 23:09'!
fromStream: sourceStream 
	self readProtocolFrom: sourceStream.
	self readHostFrom: sourceStream.
	self readPortFrom: sourceStream.
	self readIdentifierFrom: sourceStream.
	self readQueryFrom: sourceStream.
	^self! !

!SwazooURI methodsFor: 'initialize-release' stamp: ' 14/4/07 23:09'!
fromString: aString 
	| sourceStream |
	sourceStream := ReadStream on: (HTTPString decodedHTTPFrom: aString).
	self fromStream: sourceStream.
	^self! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
host
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: self hostname.
	self port = self defaultPort 
		ifFalse: 
			[ws nextPut: $:.
			self port printOn: ws].
	^ws contents! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
host: aString 
	| rs |
	rs := ReadStream on: aString.
	self hostname: (rs upTo: $: ).
	rs atEnd ifFalse: [self port: rs upToEnd asNumber]! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
hostname
	^hostname! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
hostname: aHostname 
	hostname := aHostname! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
identifier
	^identifier! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
identifier: anObject
	identifier := anObject! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
identifierPath
	| parts |
	parts := (HTTPString subCollectionsFrom: self identifier delimitedBy: $/) 
				collect: [:each | HTTPString decodedHTTPFrom: each].
	self identifier first = $/ ifTrue: [parts addFirst: '/'].
	^parts reject: [:each | each isEmpty]! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
identifierPathString
	"^aString
I return the 'directory' part of the path name."

	| sourceStream targetStream |
	targetStream := WriteStream on: String new.
	sourceStream := ReadStream on: self identifier.
	[sourceStream atEnd] whileFalse: 
			[| fragment |
			fragment := sourceStream throughAll: '/'.
			fragment last = $/ ifTrue: [targetStream nextPutAll: fragment]].
	^targetStream contents! !

!SwazooURI methodsFor: 'accessing-queries' stamp: ' 14/4/07 23:09'!
includesQuery: aString 
	| result |
	result := self queries detect: [:aQuery | aQuery key = aString]
				ifNone: [nil].
	^result notNil! !

!SwazooURI methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
isDirectory
	^self identifier last = $/! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
port
	"^an Integer
The port number defaults to 80 for HTTP."

	^port isNil ifTrue: [80] ifFalse: [port]! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
port: anInteger 
	port := anInteger! !

!SwazooURI methodsFor: 'printing' stamp: 'JM 4/26/2007 16:17'!
printOn: targetStream 
	(self hostname notNil and: [self protocol notNil]) 
		ifTrue: 
			[targetStream
				nextPutAll: self protocol;
				nextPutAll: '://'].
	self hostname notNil ifTrue: [targetStream nextPutAll: self hostname].
	(self hostname notNil and: [self port notNil and: [self port ~= 80]]) 
		ifTrue: 
			[targetStream
				nextPut: $:;
				nextPutAll: self port printString].
	targetStream nextPutAll: self identifier.
	self printQueriesOn: targetStream.
	^self! !

!SwazooURI methodsFor: 'printing' stamp: ' 14/4/07 23:09'!
printQueriesOn: targetStream 
	| firstQuery |
	self queries isEmpty 
		ifFalse: 
			[firstQuery := self queries at: 1.
			targetStream
				nextPut: $?;
				nextPutAll: firstQuery key;
				nextPut: $=;
				nextPutAll: firstQuery value.
			2 to: self queries size
				do: 
					[:queryIndex | 
					| aQuery |
					aQuery := self queries at: queryIndex.
					targetStream
						nextPut: $&;
						nextPutAll: aQuery key;
						nextPut: $=;
						nextPutAll: aQuery value]].
	^self! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
protocol
	protocol isNil ifTrue: [self protocol: 'http'].
	^protocol! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
protocol: aString
	protocol := aString.! !

!SwazooURI methodsFor: 'accessing-queries' stamp: ' 14/4/07 23:09'!
queries
	"^an OrderedCollection
This is an ordered colleciton of associations.  It can't be a dictionary, because it is legal to have many entries with the same key value."

	queries isNil ifTrue: [queries := OrderedCollection new].
	^queries! !

!SwazooURI methodsFor: 'accessing-queries' stamp: ' 14/4/07 23:09'!
queries: anOrderedCollection 
	"^self
The queries must be an OrderedCollection of Associations c.f. >>queries"

	queries := anOrderedCollection.
	^self! !

!SwazooURI methodsFor: 'accessing-queries' stamp: ' 14/4/07 23:09'!
queriesNamed: aString 
	^self queries select: [:aQuery | aQuery key = aString]! !

!SwazooURI methodsFor: 'accessing-queries' stamp: ' 14/4/07 23:09'!
queryAt: aString 
	^self queryAt: aString ifAbsent: [nil]! !

!SwazooURI methodsFor: 'accessing-queries' stamp: ' 14/4/07 23:09'!
queryAt: aString ifAbsent: aBlock 
	"^aString
I return the value of the first query I find with the key aString.  If there are none I execute aBlock."

	| result |
	result := self queries detect: [:aQuery | aQuery key = aString]
				ifNone: [aBlock].
	^result == aBlock ifTrue: [aBlock value] ifFalse: [result value]! !

!SwazooURI methodsFor: 'private' stamp: ' 14/4/07 23:09'!
readHostFrom: aStream 
	"^self
I read the host name from the URI presumed to be in aStream.  The stream should be positioned right at the start, or just after the '//' of the protocol.  The host name is terminated by one of $:, $/, $? or the end of the stream depending on wether there is a port, path, query or nothing following the host.  If the host name is of zero length, I record a nil host name.  The stream is left positioned at the terminating character."

	| hostnameStream |
	hostnameStream := WriteStream on: String new.
	[|nextCharacter| 
	nextCharacter := aStream peek.
	#($: $/ $? nil) includes: nextCharacter]
		whileFalse: [hostnameStream nextPut: aStream next].
	 hostnameStream contents isEmpty ifFalse: [hostname := hostnameStream contents].
	^self! !

!SwazooURI methodsFor: 'private' stamp: ' 14/4/07 23:09'!
readIdentifierFrom: sourceStream 

	self identifier: (sourceStream upTo: $?).
	^self! !

!SwazooURI methodsFor: 'private' stamp: ' 14/4/07 23:09'!
readPortFrom: aStream 
	"^self
I read the port nnumber from the URI presumed to be in aStream.  If a port number has been specified, the stream should be positioned right at before a $: charcter.  So, if the next chacter is a :, we have a port number.  I read up to one of $/, $? or the end of the stream depending on wether there is a path, query or nothing following the host.  The stream is left positioned at the terminating character."

	| targetStream |
	targetStream := WriteStream on: String new.
	aStream peek == $: 
		ifTrue: 
			[| terminators |
			terminators := Array 
						with: $/
						with: $?
						with: nil.
			aStream next.
			
			[| nextCharacter |
			nextCharacter := aStream peek.
			terminators includes: nextCharacter] 
					whileFalse: 
						[| nextDigit |
						nextDigit := aStream next.
						nextDigit isDigit ifTrue: [targetStream nextPut: nextDigit]].
			targetStream contents isEmpty 
				ifFalse: [port := targetStream contents asNumber]].
	^self! !

!SwazooURI methodsFor: 'private' stamp: ' 14/4/07 23:09'!
readProtocolFrom: aStream 
	"^self
I read the protocol from the URI presumed to be in aStream.  The protocol preceeds '://' in the URI.  I leave the stream position either right after the '//' if there is a protocol, otherwise I reset the position to the start of the stream."

	| candidateProtocol |
	candidateProtocol := aStream upTo: $:.
	(aStream size - aStream position >= 2 
		and: [aStream next == $/ and: [aStream next == $/]]) 
			ifTrue: [self protocol: candidateProtocol]
			ifFalse: [aStream reset].
	^self! !

!SwazooURI methodsFor: 'private' stamp: ' 14/4/07 23:09'!
readQueryFrom: sourceStream 
	[sourceStream atEnd] whileFalse: 
		[| nameValue name value |
		nameValue := sourceStream upTo: $& .
		name := nameValue copyUpTo: $= .
		value := nameValue readStream upTo: $= "if any"; upToEnd.
		self queries add: name -> (HTTPString decodedHTTPFrom: value)].
	^self! !

!SwazooURI methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
value
	"1 halt: 'Use >>asString or >>printOn: instead'. "
	^self asString! !

Object subclass: #TestPseudoSocket
	instanceVariableNames: 'byteStreamToServer byteStreamFromServer clientWaitSemaphore serverWaitSemaphore ipAddress'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Tests'!

!TestPseudoSocket commentStamp: '<historical>' prior: 0!
TestPseudoSocket is a drop in replacement for a SwazooSocket that can be used during testing to feed bytes into a running SwazooHTTPServer and grab the responses without having to start a real socket pair.

So, to the HTTP server it must look like a server socket.  To the tester it must look like a write stream (to send bytes to the HTTP server) and a read stream (to read the HTTP responses).!

!TestPseudoSocket class methodsFor: 'instance creation' stamp: ' 14/4/07 23:10'!
newTCPSocket
	"^a TestPseudoSocket
I simply return a new instance of myself."

	^self new! !

!TestPseudoSocket class methodsFor: 'instance creation' stamp: ' 14/4/07 23:10'!
serverOnIP: host port: port 
	"^self
I'm only pretending to be a socket class, so I ignore the host and port."

	^self new! !

!TestPseudoSocket methodsFor: 'socket stuff' stamp: ' 14/4/07 23:10'!
acceptRetryingIfTransientErrors
	"^another TestSocketThing
	The sender expects me to block until a request comes in 'over the socket'.  What I really do is wait for someone to ask me to 'send in' a Byte array and then I return myself.  Note that I will only handle one request at a time!!"

	self serverWaitSemaphore wait.
	^self! !

!TestPseudoSocket methodsFor: 'socket stuff' stamp: ' 14/4/07 23:10'!
bindSocketAddress: anOSkIPAddress 
	"^self
This is a no-op for me."

	ipAddress := anOSkIPAddress.
	^self! !

!TestPseudoSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
byteStreamFromServer
	^byteStreamFromServer! !

!TestPseudoSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
byteStreamFromServer: aByteStream 
	byteStreamFromServer := aByteStream.
	^self! !

!TestPseudoSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
byteStreamToServer
	^byteStreamToServer! !

!TestPseudoSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
byteStreamToServer: aByteStream 
	byteStreamToServer := aByteStream.
	^self! !

!TestPseudoSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
clientWaitSemaphore
	"^a Semaphore
I return the semaphore I use to control 'client' activity."

	clientWaitSemaphore isNil ifTrue: [clientWaitSemaphore := Semaphore new].
	^clientWaitSemaphore! !

!TestPseudoSocket methodsFor: 'socket stuff' stamp: ' 14/4/07 23:10'!
close
	"^self
The server has finished with us at this point, so we signal the semaphore to give the client end chance to grab the response."

	self clientWaitSemaphore signal.
	^self! !

!TestPseudoSocket methodsFor: 'socket stuff' stamp: ' 14/4/07 23:10'!
flush
	^self! !

!TestPseudoSocket methodsFor: 'socket stuff' stamp: ' 14/4/07 23:10'!
getPeerName
	^ipAddress! !

!TestPseudoSocket methodsFor: 'socket stuff' stamp: ' 14/4/07 23:10'!
getSocketName
	^ipAddress! !

!TestPseudoSocket methodsFor: 'socket stuff' stamp: ' 14/4/07 23:10'!
isActive
	"^self
I am pretending to be a socket, and the sender wants to know if I am active.  Of course I am!!."

	^true! !

!TestPseudoSocket methodsFor: 'socket stuff' stamp: ' 14/4/07 23:10'!
listenBackloggingUpTo: anInteger 
	"^self
This is a no-op for me."

	^self! !

!TestPseudoSocket methodsFor: 'socket stuff' stamp: ' 14/4/07 23:10'!
listenFor: anInteger 
	"^self
This is a no-op for now."

	^self! !

!TestPseudoSocket methodsFor: 'stream-toServer' stamp: ' 14/4/07 23:10'!
next
	^self byteStreamToServer next! !

!TestPseudoSocket methodsFor: 'stream-fromServer' stamp: ' 14/4/07 23:10'!
nextPut: aCharacter 
	self byteStreamFromServer nextPut: aCharacter asInteger! !

!TestPseudoSocket methodsFor: 'stream-fromServer' stamp: ' 14/4/07 23:10'!
nextPutAll: aCollection 
	"^self
At present it seems that aCollection will always be a string of chacters."

	^self byteStreamFromServer nextPutAll: aCollection asByteArray! !

!TestPseudoSocket methodsFor: 'stream-fromServer' stamp: ' 14/4/07 23:10'!
nextPutBytes: aByteArray 
	self byteStreamFromServer nextPutAll: aByteArray! !

!TestPseudoSocket methodsFor: 'stream-toServer' stamp: ' 14/4/07 23:10'!
peek
	"^a Character
It seems that the HTTP server is expecting Characters not Bytes - this will have to change."

	^byteStreamToServer isNil 
		ifTrue: [nil]
		ifFalse: [self byteStreamToServer peek asCharacter]! !

!TestPseudoSocket methodsFor: 'stream-fromServer' stamp: ' 14/4/07 23:10'!
print: anObject 
	self nextPutAll: anObject printString asByteArray.
	^self! !

!TestPseudoSocket methodsFor: 'stream-toServer' stamp: ' 14/4/07 23:10'!
read: integerNumberOfBytes 
	"^a ByteArray
I read the next numberOfBytes from my underlying stream."

	^byteStreamToServer isNil 
		ifTrue: [ByteArray new]
		ifFalse: [self byteStreamToServer nextAvailable: integerNumberOfBytes]! !

!TestPseudoSocket methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
serverWaitSemaphore
	"^a Semaphore
I return the semaphore I use to control 'server' activity."

	serverWaitSemaphore isNil ifTrue: [serverWaitSemaphore := Semaphore new].
	^serverWaitSemaphore! !

!TestPseudoSocket methodsFor: 'socket stuff' stamp: ' 14/4/07 23:10'!
setAddressReuse: aBoolean 
	"^self
This is a no-op for me."

	^self! !

!TestPseudoSocket methodsFor: 'stream-toServer' stamp: ' 14/4/07 23:10'!
socket
	"^self
I am being asked this as if I am a socket stream.  I return myself because I'm pretending to be both the socket and the socket stream."

	^self! !

!TestPseudoSocket methodsFor: 'stream-fromServer' stamp: ' 14/4/07 23:10'!
space
	self nextPut: Character space.
	^self! !

!TestPseudoSocket methodsFor: 'socket stuff' stamp: ' 14/4/07 23:10'!
stream
	"^self
I have to pretend to be a socket stream too."

	^self! !

!TestPseudoSocket methodsFor: 'stream-toServer' stamp: ' 14/4/07 23:10'!
upTo: aCharacter 
	"a ByteString
For some reason, we have to look for a character in a ByteStream - this is a Swazoo thing."

	^(self byteStreamToServer upTo: aCharacter asInteger) asByteString! !

!TestPseudoSocket methodsFor: 'stream-fromServer' stamp: ' 14/4/07 23:10'!
write: aByteArray 
	"^an Integer
	I write the contents of the sourceByteArray to my underlying Socket.
	I return the number of bytes written."

	self byteStreamFromServer nextPutAll: aByteArray.
	^aByteArray size! !

!TestPseudoSocket methodsFor: 'actions-client' stamp: ' 14/4/07 23:10'!
writeBytesToServer: aByteArray 
	"^self
This is where we make the bytes available over the pseudo socket.  Unlike a socket this is a one off thing (at least in this implementation of the pseudo socket).  Once the bytes are written, control passes to the server and stays there until the server sends a close to what it thinks is the client socket, but is really me."

	| results |
	self byteStreamToServer: (ReadStream on: aByteArray).
	self byteStreamFromServer: (WriteStream on: (ByteArray new: 1000)).
	self serverWaitSemaphore signal.
	self clientWaitSemaphore wait.
	results := self byteStreamFromServer contents.
	self byteStreamToServer: nil.
	self byteStreamFromServer: nil.
	^results! !

Object subclass: #URIIdentifier
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

URIIdentifier subclass: #SiteIdentifier
	instanceVariableNames: 'ip port host'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!SiteIdentifier class methodsFor: 'instance creation' stamp: ' 10/7/07 17:21'!
host: hostName ip: anIP port: aPort 
	^self new 
		setIp: anIP
		port: aPort
		host: hostName! !

!SiteIdentifier class methodsFor: 'obsolete' stamp: ' 10/7/07 17:21'!
ip: anIP port: aPort host: hostName 
	^self new 
		setIp: anIP
		port: aPort
		host: hostName! !

!SiteIdentifier methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
currentUrl
	| stream |
	stream := WriteStream on: String new.
	self printUrlOn: stream.
	^stream contents! !

!SiteIdentifier methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
host
	^host! !

!SiteIdentifier methodsFor: 'private' stamp: ' 14/4/07 23:09'!
host: aString
	host := aString! !

!SiteIdentifier methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
ip
	^ip! !

!SiteIdentifier methodsFor: 'private' stamp: ' 14/4/07 23:09'!
ip: aString
	ip := aString! !

!SiteIdentifier methodsFor: 'initialize-release' stamp: ' 14/4/07 23:09'!
newServer
	^ HTTPServer new ip: self ip;  port: self port! !

!SiteIdentifier methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
port
	^port! !

!SiteIdentifier methodsFor: 'private' stamp: ' 14/4/07 23:09'!
port: aNumber
	port := aNumber! !

!SiteIdentifier methodsFor: 'private' stamp: ' 14/4/07 23:09'!
printHostPortStringOn: stream 
	stream nextPutAll: self host.
	self port = 80 ifFalse: [stream nextPut: $:; nextPutAll: self port printString]! !

!SiteIdentifier methodsFor: 'private' stamp: ' 14/4/07 23:09'!
printString
	^'a Swazoo.SiteIndentifier
	host: ', (self host isNil ifTrue: [''] ifFalse: [self host]), '
	ip: ', (self ip isNil ifTrue: [''] ifFalse: [self ip]), '
	port: ', self port printString! !

!SiteIdentifier methodsFor: 'private' stamp: ' 14/4/07 23:09'!
printUrlOn: aWriteStream 
	aWriteStream nextPutAll: 'http://'.
	self printHostPortStringOn: aWriteStream! !

!SiteIdentifier methodsFor: 'initialize-release' stamp: ' 14/4/07 23:09'!
setIp: anIP port: aPort host: hostName 
	self ip: anIP.
	self port: aPort.
	self host: hostName! !

!SiteIdentifier methodsFor: 'private' stamp: 'JM 4/17/2007 09:48'!
valueMatch: aSiteIdentifier
	"ip can be in numbers or named!!"
	| myIP otherIP |
	myIP := SpIPAddress hostName: self ip port: self port.
	otherIP := SpIPAddress hostName: aSiteIdentifier ip port: aSiteIdentifier port.
	^myIP hostAddress = otherIP hostAddress 
		and: [myIP portNumber = otherIP portNumber
			and: [self host match: aSiteIdentifier host] ]! !

!URIIdentifier methodsFor: 'comparing' stamp: ' 14/4/07 23:09'!
= anIdentifier 
	^self match: anIdentifier! !

!URIIdentifier methodsFor: 'comparing' stamp: ' 14/4/07 23:09'!
hash
	^1! !

!URIIdentifier methodsFor: 'testing' stamp: ' 14/4/07 23:09'!
match: anotherIdentifier 
	^(self typeMatch: anotherIdentifier) 
		and: [self valueMatch: anotherIdentifier]! !

!URIIdentifier methodsFor: 'private' stamp: ' 14/4/07 23:09'!
typeMatch: anotherIdentifier 
	^self class == anotherIdentifier class! !

!URIIdentifier methodsFor: 'private' stamp: ' 14/4/07 23:09'!
valueMatch: anotherIdentifier 
	^self subclassResponsibility! !

Object subclass: #URIResolution
	instanceVariableNames: 'position request'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP'!

!URIResolution class methodsFor: 'instance creation' stamp: ' 14/4/07 23:09'!
resolveRequest: aRequest startingAt: aResource 
	^(self new initializeRequest: aRequest) visitResource: aResource! !

!URIResolution methodsFor: 'private' stamp: ' 14/4/07 23:09'!
advance
	self position: self position + 1! !

!URIResolution methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
atEnd
	^self position = self request uri identifierPath size! !

!URIResolution methodsFor: 'private' stamp: ' 14/4/07 23:09'!
currentIdentifier
	^self currentPath last! !

!URIResolution methodsFor: 'private' stamp: ' 14/4/07 23:09'!
currentPath
	^self request uri identifierPath copyFrom: 1 to: self position! !

!URIResolution methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
fullPath
	^self request uri identifierPath! !

!URIResolution methodsFor: 'private' stamp: ' 14/4/07 23:09'!
getAnswerFrom: aResource
	^aResource answerTo: self request! !

!URIResolution methodsFor: 'private-initialize' stamp: ' 14/4/07 23:09'!
initializeRequest: aRequest 
	self request: aRequest.
	self request resolution: self.
	self position: 1! !

!URIResolution methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
position
	^position! !

!URIResolution methodsFor: 'private' stamp: ' 14/4/07 23:09'!
position: anInteger
	position := anInteger! !

!URIResolution methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
request
	^request! !

!URIResolution methodsFor: 'private' stamp: ' 14/4/07 23:09'!
request: aRequest
	request := aRequest! !

!URIResolution methodsFor: 'resolving' stamp: ' 14/4/07 23:09'!
resolveCompositeResource: aResource 
	(aResource canAnswer and: [aResource match: self currentIdentifier]) 
		ifFalse: [^nil].
	^self visitChildrenOf: aResource advancing: true! !

!URIResolution methodsFor: 'resolving' stamp: ' 14/4/07 23:09'!
resolveLeafResource: aResource 
	(aResource canAnswer and: [self stringMatch: aResource]) ifFalse: [^nil].
	^self getAnswerFrom: aResource! !

!URIResolution methodsFor: 'resolving' stamp: ' 14/4/07 23:09'!
resolveServerRoot: aServerRoot 
	^self resolveTransparentComposite: aServerRoot! !

!URIResolution methodsFor: 'resolving' stamp: ' 14/4/07 23:09'!
resolveSite: aSite 
	(aSite canAnswer and: [self siteMatch: aSite]) ifFalse: [^nil].
	^self visitChildrenOf: aSite advancing: false! !

!URIResolution methodsFor: 'resolving' stamp: ' 14/4/07 23:09'!
resolveTransparentComposite: aCompositeResource 
	^self visitChildrenOf: aCompositeResource advancing: false! !

!URIResolution methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
resourcePath
	^self request uri identifierPath copyFrom: 1 to: self position! !

!URIResolution methodsFor: 'private' stamp: ' 14/4/07 23:09'!
retreat
	self position: self position - 1.
	^nil! !

!URIResolution methodsFor: 'private' stamp: ' 10/7/07 17:21'!
siteMatch: aSite 
	| siteIdentifier hostName |
	hostName := self request headers 
				fieldOfClass: HTTPHostField
				ifPresent: [:field | field hostName]
				ifAbsent: [self request requestLine requestURI hostname].
	siteIdentifier := SiteIdentifier 
				host: (hostName notNil ifTrue: [hostName] ifFalse: [''])
				ip: self request ip
				port: self request port.
	^aSite match: siteIdentifier! !

!URIResolution methodsFor: 'private' stamp: ' 14/4/07 23:09'!
stringMatch: aResource 
	^aResource uriPattern = self currentIdentifier! !

!URIResolution methodsFor: 'accessing' stamp: ' 14/4/07 23:09'!
tailPath
	| fullPath |
	fullPath := self fullPath.
	^fullPath copyFrom: self position + 1 to: fullPath size! !

!URIResolution methodsFor: 'private' stamp: ' 14/4/07 23:09'!
tailStream
	^ReadStream on: self tailPath! !

!URIResolution methodsFor: 'resolving' stamp: ' 14/4/07 23:09'!
visitChildrenOf: aResource advancing: aBoolean 
	| response |
	self atEnd & aBoolean ifTrue: [^self getAnswerFrom: aResource].
	aBoolean ifTrue: [self advance].
	aResource children do: 
			[:each | 
			response := self visitResource: each.
			response isNil ifFalse: [^response]].
	^aBoolean ifTrue: [self retreat] ifFalse: [nil]! !

!URIResolution methodsFor: 'resolving' stamp: ' 14/4/07 23:09'!
visitResource: aResource 
	^aResource helpResolve: self! !

SpError subclass: #SwazooHTTPParseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Exceptions'!

SpError subclass: #SwazooHTTPRequestError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Exceptions'!

SwazooHTTPRequestError subclass: #SwazooHTTPPostError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Exceptions'!

SwazooHTTPRequestError subclass: #SwazooHTTPPutError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Exceptions'!

SpError subclass: #SwazooHeaderFieldParseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Exceptions'!

SpError subclass: #SwazooSiteError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Exceptions'!

!SpFilename methodsFor: '*Swazoo' stamp: 'JM 4/18/2007 16:23'!
etag
	"^a String
	The etag of a file entity is taken to be the date last modified as a String. 
	We use the SpTimestamp in "

	^self lastModified asRFC1123String! !

!SpFilename methodsFor: '*Swazoo' stamp: 'JM 4/18/2007 16:22'!
lastModified
"	| info |
	info := self dates at: #modified. 
	^SpTimestamp fromDate: info first andTime: info last
"
	^SpTimestamp now.! !

Error subclass: #HTTPException
	instanceVariableNames: 'response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-Exceptions'!

!HTTPException commentStamp: '<historical>' prior: 0!
HTTPException immediatelly returns attached HTTP response to client. That way it is easier to respond with different status codes (like 201 Created). Not only error ones!! You can respond somewhere deeply in code of your resource with raising that exception and adding a prepared HTTPResponse. 
This exception is non-resumable!!

Example of ways to raise http response (200 Ok):

	HTTPException raiseResponse: (HTTPResponse new code: 200).
	HTTPException raiseResponseCode: 200.
	HTTPException ok.

Instance Variables:
	response	<HTTPResponse>	a response to be sent to client

!

!HTTPException class methodsFor: 'responses-succesfull' stamp: ' 14/4/07 23:10'!
accepted
	^self raiseResponse: (HTTPResponse new code: 202)! !

!HTTPException class methodsFor: 'responses-server error' stamp: ' 14/4/07 23:10'!
badGateway
	^self raiseResponse: (HTTPResponse new code: 502)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
badRequest
	^self raiseResponse: (HTTPResponse new code: 400)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
conflict
	^self raiseResponse: (HTTPResponse new code: 409)! !

!HTTPException class methodsFor: 'responses-informational' stamp: ' 14/4/07 23:10'!
continue
	^self raiseResponse: (HTTPResponse new code: 100)! !

!HTTPException class methodsFor: 'responses-succesfull' stamp: ' 14/4/07 23:10'!
created
	^self raiseResponse: (HTTPResponse new code: 201)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
expectationFailed
	^self raiseResponse: (HTTPResponse new code: 416)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
forbidden
	^self raiseResponse: (HTTPResponse new code: 403)! !

!HTTPException class methodsFor: 'responses-redirection' stamp: ' 14/4/07 23:10'!
found
	^self raiseResponse: (HTTPResponse new code: 302)! !

!HTTPException class methodsFor: 'responses-server error' stamp: ' 14/4/07 23:10'!
gatewayTimeout
	^self raiseResponse: (HTTPResponse new code: 504)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
gone
	^self raiseResponse: (HTTPResponse new code: 410)! !

!HTTPException class methodsFor: 'responses-server error' stamp: ' 14/4/07 23:10'!
httpVersionNotSupported
	^self raiseResponse: (HTTPResponse new code: 505)! !

!HTTPException class methodsFor: 'responses-server error' stamp: ' 14/4/07 23:10'!
internalServerError
	^self raiseResponse: (HTTPResponse new code: 500)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
lengthRequired
	^self raiseResponse: (HTTPResponse new code: 411)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
methodNotAllowed
	^self raiseResponse: (HTTPResponse new code: 405)! !

!HTTPException class methodsFor: 'responses-redirection' stamp: ' 14/4/07 23:10'!
movedPermanently
	^self raiseResponse: (HTTPResponse new code: 301)! !

!HTTPException class methodsFor: 'responses-redirection' stamp: ' 14/4/07 23:10'!
multipleChoices
	^self raiseResponse: (HTTPResponse new code: 300)! !

!HTTPException class methodsFor: 'responses-succesfull' stamp: ' 14/4/07 23:10'!
noContent
	^self raiseResponse: (HTTPResponse new code: 204)! !

!HTTPException class methodsFor: 'responses-succesfull' stamp: ' 14/4/07 23:10'!
nonAuthorativeInformation
	^self raiseResponse: (HTTPResponse new code: 203)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
notAcceptable
	^self raiseResponse: (HTTPResponse new code: 406)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
notFound
	^self raiseResponse: (HTTPResponse new code: 404)! !

!HTTPException class methodsFor: 'responses-server error' stamp: ' 14/4/07 23:10'!
notImplemented
	^self raiseResponse: (HTTPResponse new code: 501)! !

!HTTPException class methodsFor: 'responses-redirection' stamp: ' 14/4/07 23:10'!
notModified
	^self raiseResponse: (HTTPResponse new code: 304)! !

!HTTPException class methodsFor: 'responses-succesfull' stamp: ' 14/4/07 23:10'!
ok
	^self raiseResponse: HTTPResponse ok! !

!HTTPException class methodsFor: 'responses-succesfull' stamp: ' 14/4/07 23:10'!
partialContent
	^self raiseResponse: (HTTPResponse new code: 206)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
paymentRequired
	^self raiseResponse: (HTTPResponse new code: 402)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
preconditionFailed
	^self raiseResponse: (HTTPResponse new code: 412)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
proxyAuthenticationRequired
	^self raiseResponse: (HTTPResponse new code: 407)! !

!HTTPException class methodsFor: 'signalling' stamp: ' 14/4/07 23:10'!
raiseResponse: aHTTPResponse
	"Raise an exception to immediatelly return that response."
	^self new 
		response: aHTTPResponse;
		raiseSignal.! !

!HTTPException class methodsFor: 'signalling' stamp: ' 14/4/07 23:10'!
raiseResponseCode: aNumber
	"Raise an exception to immediatelly return http response with that code"
	^self raiseResponse: (HTTPResponse new code: aNumber)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
requestEntityTooLarge
	^self raiseResponse: (HTTPResponse new code: 413)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
requestTimeout
	^self raiseResponse: (HTTPResponse new code: 408)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
requestURITooLong
	^self raiseResponse: (HTTPResponse new code: 414)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
requestedRangeNotSatisfiable
	^self raiseResponse: (HTTPResponse new code: 416)! !

!HTTPException class methodsFor: 'responses-succesfull' stamp: ' 14/4/07 23:10'!
resetContent
	^self raiseResponse: (HTTPResponse new code: 205)! !

!HTTPException class methodsFor: 'responses-redirection' stamp: ' 14/4/07 23:10'!
seeOther
	^self raiseResponse: (HTTPResponse new code: 303)! !

!HTTPException class methodsFor: 'responses-server error' stamp: ' 14/4/07 23:10'!
serviceUnavailable
	^self raiseResponse: (HTTPResponse new code: 503)! !

!HTTPException class methodsFor: 'responses-informational' stamp: ' 14/4/07 23:10'!
switchingProtocols
	^self raiseResponse: (HTTPResponse new code: 101)! !

!HTTPException class methodsFor: 'responses-redirection' stamp: ' 14/4/07 23:10'!
temporaryRedirect
	^self raiseResponse: (HTTPResponse new code: 307)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
unathorized
	^self raiseResponse: (HTTPResponse new code: 401)! !

!HTTPException class methodsFor: 'responses-client error' stamp: ' 14/4/07 23:10'!
unsupportedMediaType
	^self raiseResponse: (HTTPResponse new code: 415)! !

!HTTPException class methodsFor: 'responses-redirection' stamp: ' 14/4/07 23:10'!
useProxy
	^self raiseResponse: (HTTPResponse new code: 305)! !

!HTTPException methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
response
	^response! !

!HTTPException methodsFor: 'accessing' stamp: ' 14/4/07 23:10'!
response: aHTTPResponse
	response := aHTTPResponse! !
HTTPResponse initialize!
HTTPServer initialize!
FileResource initialize!
SwazooServer initialize!
